#!/bin/ksh
# Script to run the EXAMPLE_FWD code
#
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# This script runs only ONE test for NOAA-16 AVHRR
# 
# R Saunders Jan 2008
#
# use function for rttov ids
. ./rttov_id.ksh

# Test case
PLATFORM=noaa
INST=avhrr
SATID=16
FILE_PROFILE=../data/prof_trop_f2.dat
ZENANGLE=0.
SURF_TYPE=1	# Sea
WATER_TYPE=1    # Ocean
NLEVELS=43      # Num of profile levels
#directory for RT coefs
DATART=../rtcoef_rttov7
# Test output directory
TEST=${TEST:=../test} ; export TEST
REF_TEST=${REF_TEST:=../reftest} ; export REF_TEST

echo " "
echo " "
echo " Test forward  ${INST} "
echo " "

echo  "Platform   ${PLATFORM}"
echo  "Instrument ${INST}"
echo  "Satellite  ${SATID}"
echo  "File for profile  ${FILE_PROFILE}"
echo  "Zenith angle ${ZENANGLE}"
echo  "Azimuth angle ${AZANGLE}"
echo  "Surface type ${SURF_TYPE}"
echo  "Water type ${WATER_TYPE}"
echo  "Number of levels of profile ${NLEVELS}"


# call to function rttov_id
rttov_id ${PLATFORM} ${INST}
# diff command ignoring case of letters and spaces
DIFF='diff -biw'
###################################################
# The followind sed command file is to change all occurence of
# .xxx or -.yyy into 0.xxx and -0.yyy to allow a good comparison
# between the refernce test and fortran output (especially for HP)
cat <<EOF > sed.cmd
s/ \./0\./g
s/ -\./-0\./g
EOF
#

# Profile file
rm -f prof.dat
if [ ! -s ${FILE_PROFILE} ] ; then
  echo "file ${FILE_PROFILE}  does not exist "
  exit
fi
cp ${FILE_PROFILE}  prof.dat

# Coefficient files
COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
COEF_NAME_BIN=rtcoef_${PLATFORM}_${SATID}_${INST}.bin
rm -f ${COEF_NAME}
rm -f ${COEF_NAME_BIN}
if [ -s  $DATART/${COEF_NAME} ] ; then
  ln -s $DATART/${COEF_NAME} ${COEF_NAME}
fi
if [ -s  $DATART/${COEF_NAME_BIN} ] ; then
  ln -s $DATART/${COEF_NAME_BIN} ${COEF_NAME_BIN}
fi

../bin/example_fwd.exe << EOF
${PLATFORM_N}   , platform name
${SATID}        , satellite number (NOAAxx = xx)
${INST_N}       , instrument number
${SURF_TYPE}    , Surface type (0=land, 1=sea, 2=ice/snow)
${WATER_TYPE}   , Water type (0=fresh water, 1=ocean water)
${NLEVELS}      , number of levels
${ZENANGLE}     , Zenith angle
 1, 1, 0.0      , channel 1, valid, emissivity=0.0
 2, 1, 0.0      , channel 2, valid, emissivity=0.0
 3, 1, 0.0      , channel 3, valid, emissivity=0.0
EOF

rm -f ${COEF_NAME} ${COEF_NAME_BIN}
rm -f prof.dat 

sed -f sed.cmd print.dat > ${TEST}/example_fwd.lst 
${DIFF} ${REF_TEST}/example_fwd.lst ${TEST}/example_fwd.lst > ${TEST}/example_fwd.diff

echo 
echo "result is in ../test/example_fwd.lst file "
echo

exit
