#!/bin/ksh
#
# Functionality tests for RTTOV-9 for TL/AD/K.
# This tests all aspects of the code for all
# sensors except AIRS/IASI. 
#
# ROger Saunders 2008
#
######## Edit this section for your pathnames#######
DATAIN=../data; export DATAIN
DATART=../rtcoef_rttov7; export DATART
DATART8F3=../rtcoef_rttov8/fastem3; export DATART8F3
DATART9F3=../rtcoef_rttov9; export DATART9F3
DATART8P7=../rtcoef_rttov8/rttov7pred; export DATART8P7
DATART8P8=../rtcoef_rttov8/rttov8pred; export DATART8P8
DATART8P8CO2=../rtcoef_rttov8/rttov8pred/co2; export DATART8P8CO2
REF_TEST=../reftest_rttov9; export REF_TEST
TEST=../test_rttov9; export TEST
PLEVS='plevs_43lev_nointerp.dat'
SAVE_PLEVS='plevs_43lev_nointerp.dat'
PLEVS1='plevs_101lev_nointerp.dat'
###################################################
SAVE_DATART=$DATART
for TEST_NUMBER in 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28
#uncomment next line and comment above to run just 1 or 2 tests
#for TEST_NUMBER in 17
do
  DATART=${SAVE_DATART}
  PLEVS=${SAVE_PLEVS}
  case ${TEST_NUMBER} in
    # Test 1 HIRS N15 Emi=0.98 Ang=45 Artic rttov-7 predictors sea ozone
    '01') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_hirs_98.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0';
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 2 HIRS N15 ISEM Ang=0 tropics rttov-7 predictors sea ozone
    '02') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_irem.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 3 HIRS N15 ISEM  Ang=45  arctic rttov-7 predictors sea ozone 1 chan 10 profiles
    '03') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_hirs_1.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='10';;

    # Test 4 HIRS N15 ISEM Ang=0  tropics rttov-8 predictors sea ozone
    '04') SAVE_DATART=${DATART} ;
          DATART=${DATART8P8} ;
          PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_irem.dat';
	  PROFILE='prof_trop_ozone.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 5 HIRS N15 ISEM Ang=60 sea Arctic rttov-8 predictors ozone
    '05') SAVE_DATART=${DATART} ;
          DATART=${DATART8P8} ;
          PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_hirs_1.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='10';;

    # Test 6 HIRS N15 ISEM Ang=30 land  Arctic rttov-7 predictors ozone
    '06') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_irem.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='30.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='0'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 
    
    # Test 7 HIRS N15 ISEM Ang=45 arctic sea-ice rttov-7 predictors ozone
    '07') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_irem.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='2'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 8 HIRS N15 Emi=0.8 Ang=30 tropics rttov-7 predictor overcast cloud 400hPa
    '08') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_ircloud1.dat';
	  PROFILE='prof_trop_cld.dat'; ZENANGLE='30.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 9 HIRS N15 Emi=0.8 Ang=0 tropics rttov-7 predictor 60%/400hPa cloud 
    '09') PLATFORM='noaa'; INST='hirs'; SATID='15';CHANNELS='input_ircloud1.dat';
	  PROFILE='prof_trop_fracld.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 10 HIRS METOP-1 ISEM Ang=45 arctic rttov-8 predictor Ozone+CO2 sea 101lev rt file
    '10') SAVE_DATART=${DATART} ;
          DATART=${DATART8P8CO2} ;
          PLEVS=${PLEVS1}
          PLATFORM='metop'; INST='hirs'; SATID='2';CHANNELS='input_irem.dat';
	  PROFILE='prof_co2_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 11 AMSU-A N15 Emi=0.6 Ang=30 Tropical sea
    '11') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_amsua.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='30.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 
    
    # Test 12 AMSU-A N15 FASTEM-2 Ang=0 Arctic sea
    '12') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_amsua_f2.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 
    
    # Test 13 AMSU-A N15 FASTEM-3 Ang=0 Arctic sea 
    '13') SAVE_DATART=${DATART} ;
	  DATART=${DATART9F3} ;
          PLEVS=${SAVE_PLEVS}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_amsua_f2.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    	  
    # Test 14 AMSU-A N15 Ang=45 Arctic FASTEM-2 land
    '14') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_amsua_f2.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='0'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 
    
    # Test 15 AMSU-A N15 Ang=45 Arctic FASTEM-3 land 1 channel 10 profiles
    '15') SAVE_DATART=${DATART} ;
	  DATART=${DATART9F3} ;
          PLEVS=${SAVE_PLEVS}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_amsua_1.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='0'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='10';; 

    # Test 16 AMSU-A N15  Ang=45 tropics FASTEM-2 CLWP sea
    '16') PLEVS=${SAVE_PLEVS}
          PLATFORM='noaa'; INST='amsua'; SATID='15';CHANNELS='input_lwp_f2.dat';
	  PROFILE='prof_lwp_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 17 MSU N14 FASTEM-3 Ang=45 Tropical sea
    '17') SAVE_DATART=${DATART} ;
	  DATART=${DATART9F3} ;
          PLEVS=${SAVE_PLEVS}
          PLATFORM='noaa'; INST='msu'; SATID='14';CHANNELS='input_msu_f2.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 
    
    # Test 18 SSM/I F-15 FASTEM-3 Ang=55. Tropical (currently RT-7 replace with RT-9)
    '18') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='dmsp'; INST='ssmi'; SATID='15';CHANNELS='input_ssmi_f3.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='55.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 19 SSM/I F-15 FASTEM-2 Ang=55. Tropical 
    '19') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='dmsp'; INST='ssmi'; SATID='15';CHANNELS='input_ssmi_f2.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='55.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 

    # Test 20 SSU N14 Emi=1 Ang=60 Arctic sea
    '20') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='noaa'; INST='ssu'; SATID='14';CHANNELS='input_ssu.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 21 AMSU-B N15 FASTEM-2 Ang=0 Tropical
    '21') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='noaa'; INST='amsub'; SATID='15';CHANNELS='input_amsub_f2.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 

    # Test 22 AMSU-B N15  Ang=60 Arctic FASTEM-3
    '22') SAVE_DATART=${DATART} ;
	  DATART=${DATART9F3} ;
          PLEVS=${SAVE_PLEVS}
          PLATFORM='noaa'; INST='amsub'; SATID='15';CHANNELS='input_amsub_f2.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 23 MODIS-1 ISEM Ang=30 Tropical
    '23') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='eos'; INST='modis'; SATID='1';CHANNELS='input_modis.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='30.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 24 AMSR-E Aqua arctic test 55 deg FASTEM-3 sea (currently RT-7 replace with RT-9) 
    '24') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='eos'; INST='amsr'; SATID='2';CHANNELS='input_amsre.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='55.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 25 SSMIS Fastem-3 Ang=55 Sea Tropical (currently RT-7 replace with RT-9)
    '25') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='dmsp'; INST='ssmis'; SATID='16';CHANNELS='input_ssmis_f3.dat';
	  PROFILE='prof_trop_f2.dat'; ZENANGLE='55.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
 
    # Test 26 Windsat Coriolis Fastem-3 Ang=45 Artic 
    '26') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
	  DATART=${DATART9F3} ;
          PLATFORM='coriolis'; INST='windsat'; SATID='1';CHANNELS='input_windsat.dat';
	  PROFILE='prof_windy.dat'; ZENANGLE='30.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 27 HIRS METOP-1 ISEM Ang=60 arctic rttov-7 predictors sea ozone
    '27') PLEVS=${SAVE_PLEVS}
          DATART=${SAVE_DATART}
          PLATFORM='metop'; INST='hirs'; SATID='2';CHANNELS='input_irem.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';;
 
    # Test 28 HIRS METOP-1 ISEM Ang=45 arctic rttov-8 predictor Ozone+CO2 sea 101lev rt file
    '28') SAVE_DATART=${DATART} ;
          DATART=${DATART8P8CO2} ;
          PLEVS=${PLEVS1}
          PLATFORM='metop'; INST='hirs'; SATID='2';CHANNELS='input_irem.dat';
	  PROFILE='prof_co2_101lev.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';;

  esac

  ./single_test_rttov9.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${AEROSOLS} ${CLOUDS} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF}
  
done
echo  " "
echo  "Test RTTOV9 successfully completed"

exit
