#!/bin/ksh
#
# Functionality tests for RTTOV-9 for TL/AD/K.
# This tests aspects of the code for the high 
# resolution sounders. Not all users will want
# to run this test due to the size of the coeff files
# 
# ROger Saunders 2008
#
######## Edit this section for your pathnames#######
DATAIN=../data; export DATAIN
DATART7=../rtcoef_rttov7; export DATART7
DATART8=../rtcoef_rttov8/rttov8pred; export DATART8
DATART=../rtcoef_rttov9; export DATART
REF_TEST=../reftest_rttov9; export REF_TEST
TEST=../test_rttov9; export TEST
#PLEVS43='plevs_43lev_nointerp.dat'
#PLEVS101='plevs_101lev_nointerp.dat'
PLEVS43='plevs_43lev.dat'
PLEVS101='plevs_101lev.dat'
###################################################
SAVE_DATART=$DATART
for TEST_NUMBER in 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 
#uncomment next line and comment above to run just 1 or 2 tests
#for TEST_NUMBER in 61
do
  DATART=${SAVE_DATART}
  PLEVS=${PLEVS43}
  case ${TEST_NUMBER} in
    # Test 51 AIRS  Emi=0.98 Ang=45 Artic rttov-7 predictors sea ozone
    '51') SAVE_DATART=${DATART} ;
          DATART=${DATART7} ;
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs_98.dat';
	  PROFILE='prof_arc_f2.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0';
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 52 AIRS ISEM Ang=0 arctic rttov-8 predictors sea ozone
    '52') SAVE_DATART=${DATART} ;
          DATART=${DATART8} ;
          PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev_rt8.dat'; ZENANGLE='0.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 53 AIRS ISEM  Ang=45  tropics rttov-9 predictors sea ozone 
    '53') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 54 AIRS ISEM Ang=60  tropics rttov-9 predictors sea ozone 1 chan 10 profs
    '54') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs_1.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='60.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='10';;
    
    # Test 55 AIRS ISEM Ang=45 sea tropical rttov-9 predictors all gases
    '55') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 56 AIRS ISEM Ang=45 tropical rttov-9 predictors land all gases
    '56') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='0'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';; 

    # Test 57 AIRS ISEM Ang=45 tropical rttov-9 predictors land all gases 1km elevation
    '57') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='0'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='1.'; REFRACTION='1'; FULL='1'; NPROF='2';; 
    
    # Test 58 AIRS emis=0.95 Ang=45 tropical sea rttov-7 predictors ozone 60% /400hPa simple cloud
    '58') SAVE_DATART=${DATART} ;
          PLEVS=${PLEVS43}
          DATART=${DATART7} ;
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs_95.dat';
	  PROFILE='prof_trop_fracld.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 59 AIRS ISEM Ang=45 topical rttov-9 predictor refraction
    '59') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';;

    # Test 60 AIRS ISEM Ang=45 tropical rttov-9 predictor Solar
    '60') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';SUNZENANGLE='35.'; SUNAZANGLE='179';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='1'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 61 AIRS ISEM Ang=45 tropical rttov-9 predictor aerosol
    '61') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; DEFPAR='1'; CLIMAT='5'; EFFDIAM='4'; SHAPE='1';
	  PROFILE='prof_101lev.dat'; ZENANGLE='30.'; AZANGLE='0.'; SUNRAD='0'; AEROSOLS='1'; CLOUDS='1'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 62 AIRS ISEM Ang=45 tropical rttov-9 predictor clouds
    '62') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat'; PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; EFFDIAM='4'; SHAPE='1';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='1'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 63 AIRS ISEM Ang=45 tropical rttov-9 predictor refraction, solar, aerosol, clouds
    '63') PLEVS=${PLEVS101}
          PLATFORM='eos'; INST='airs'; SATID='2';CHANNELS='input_airs.dat';SUNZENANGLE='35.'; SUNAZANGLE='179';
           PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; DEFPAR='1'; CLIMAT='5'; EFFDIAM='4'; SHAPE='1';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='1'; AEROSOLS='1'; CLOUDS='1'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';;

    # Test 64 IASI emis=0.95 Ang=45 tropical sea rttov-7 predictors ozone 60% /400hPa simple cloud
    '64') SAVE_DATART=${DATART} ;
          PLEVS=${PLEVS43}
          DATART=${DATART7} ;
          PLATFORM='metop'; INST='iasi'; SATID='2';CHANNELS='input_iasi_95.dat';
	  PROFILE='prof_trop_fracld.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;
    
    # Test 65 IASI ISEM Ang=45 topical rttov-9 predictor refraction+solar
    '65') PLEVS=${PLEVS101}
          PLATFORM='metop'; INST='iasi'; SATID='2';CHANNELS='input_iasi_isem.dat';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='1'; AEROSOLS='0'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';
      SUNZENANGLE='35.'; SUNAZANGLE='179';;

    # Test 66 IASI ISEM Ang=45 topical rttov-9 predictor aerosols
    '66') PLEVS=${PLEVS101}
          PLATFORM='metop'; INST='iasi'; SATID='2';CHANNELS='input_iasi_isem1.dat'; PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; DEFPAR='1'; CLIMAT='5'; 
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='1'; CLOUDS='0'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 67 IASI ISEM Ang=45 topical rttov-9 predictor clouds
    '67') PLEVS=${PLEVS101}
          PLATFORM='metop'; INST='iasi'; SATID='2';CHANNELS='input_iasi_isem1.dat';  PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; EFFDIAM='4'; SHAPE='1';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='0'; AEROSOLS='0'; CLOUDS='1'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='0'; FULL='1'; NPROF='2';;

    # Test 68 IASI ISEM Ang=45 topical rttov-9 predictor all
    '68') PLEVS=${PLEVS101}
          PLATFORM='metop'; INST='iasi'; SATID='2';CHANNELS='input_iasi_isem1.dat'; PROFILEAER='prof_aerosl_cl.dat'; 
          PROFILECLD='prof_cloud.dat'; PROFILECLDCFR='prof_cfrac.dat'; DEFPAR='1'; CLIMAT='5'; EFFDIAM='4'; SHAPE='1';
	  PROFILE='prof_101lev.dat'; ZENANGLE='45.'; AZANGLE='45.'; SUNRAD='1'; AEROSOLS='1'; CLOUDS='1'; 
      SURF_TYPE='1'; WATER_TYPE='1'; LATITUDE='45.'; ELEVATION='0'; REFRACTION='1'; FULL='1'; NPROF='2';
      SUNZENANGLE='35.'; SUNAZANGLE='179';;

  esac

  if [ ${TEST_NUMBER} -eq '60' ] ; then
  ./single_test_rttov9_sun.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${SUNZENANGLE} ${SUNAZANGLE} ${AEROSOLS} ${CLOUDS} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} ${ELEVATION} \
  ${REFRACTION} ${FULL} ${NPROF} 
  elif [ ${TEST_NUMBER} -eq '61' ] ; then
  ./single_test_rttov9_scatt.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${PROFILEAER} \
  ${PROFILECLD} ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${AEROSOLS} ${DEFPAR} ${CLIMAT} ${CLOUDS} ${EFFDIAM} ${SHAPE} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} \
  ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF} 
  elif [ ${TEST_NUMBER} -eq '62' ] ; then
  ./single_test_rttov9_noaero.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} \
  ${PROFILEAER} ${PROFILECLD} ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${AEROSOLS} ${CLOUDS} ${EFFDIAM} ${SHAPE} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} \
  ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF}
  elif [ ${TEST_NUMBER} -eq '63' ] ; then
  ./single_test_rttov9_scatt_sun.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} \
  ${PROFILE} ${PLEVS} ${PROFILEAER} ${PROFILECLD} \
  ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${SUNZENANGLE} ${SUNAZANGLE} ${AEROSOLS} ${DEFPAR} ${CLIMAT} ${CLOUDS} ${EFFDIAM} ${SHAPE} ${SURF_TYPE} \
  ${WATER_TYPE} ${LATITUDE} ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF}
  elif [ ${TEST_NUMBER} -eq '65' ] ; then
  ./single_test_rttov9_sun.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${SUNZENANGLE} ${SUNAZANGLE} ${AEROSOLS} ${CLOUDS} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} ${ELEVATION} \
  ${REFRACTION} ${FULL} ${NPROF}
  elif [ ${TEST_NUMBER} -eq '66' ] ; then
  ./single_test_rttov9_scatt_nocld.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${PROFILEAER} ${PROFILECLD} \
  ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} ${SUNRAD} ${AEROSOLS} ${DEFPAR} ${CLIMAT} ${CLOUDS} ${SURF_TYPE} \
  ${WATER_TYPE} ${LATITUDE} ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF}
  elif [ ${TEST_NUMBER} -eq '67' ] ; then
  ./single_test_rttov9_noaero.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${PROFILEAER} ${PROFILECLD} \
  ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${AEROSOLS} ${CLOUDS} ${EFFDIAM} ${SHAPE} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} \
  ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF} 
  elif [ ${TEST_NUMBER} -eq '68' ] ; then
  ./single_test_rttov9_scatt_sun.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${PROFILEAER} ${PROFILECLD} \
  ${PROFILECLDCFR} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${SUNZENANGLE} ${SUNAZANGLE} ${AEROSOLS} ${DEFPAR} ${CLIMAT} ${CLOUDS} ${EFFDIAM} ${SHAPE} ${SURF_TYPE} \
  ${WATER_TYPE} ${LATITUDE} ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF}
  else
  ./single_test_rttov9.ksh ${TEST_NUMBER} ${PLATFORM} ${INST} ${SATID} ${CHANNELS} ${PROFILE} ${PLEVS} ${ZENANGLE} ${AZANGLE} \
  ${SUNRAD} ${AEROSOLS} ${CLOUDS} ${SURF_TYPE} ${WATER_TYPE} ${LATITUDE} ${ELEVATION} ${REFRACTION} ${FULL} ${NPROF} 
  fi
done

echo  " "
echo  "Test RTTOV9 hires successfully completed"

exit
