Program example_fwd
  !
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  !     *************************************************************
  !
  !     TEST PROGRAM FOR RTTOV SUITE FORWARD MODEL ONLY
  !          RTTOV VERSION 9
  ! To run this program you must have the following files
  ! either resident in the same directory or set up as a
  ! symbolic link:
  !   prof.dat       --    input profile
  !   rtcoef_platform_id_sensor.dat --  coefficient file to match
  !   the sensor you request in the input dialogue
  ! There are scripts available to set up the files above and
  ! run this program (e.g. test_rttov9.ksh)
  ! The output is generated in a file called print.dat.
  !
  !
  ! If the user wants to use this example to create his own
  ! program he will have to modify the code between
  ! comment lines of that kind:
  !     !================================
  !     !======Read =====start===========
  !          code to be modified
  !     !======Read ===== end ===========
  !     !================================
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !  1.0    27/04/2004   orginal (based on tstrad) P. Brunel
  !  1.1    09/08/2004   modified to allow for variable no. channels/per profile
  !                       R. Saunders
  !  1.2    13/04/2007   Modified for RTTOV-90
  !  1.3    31/07/2007   Modified for RTTOV-91 R Saunders
  !  1.4    11/10/2007   Parallel version P.Marguinaud
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  Use rttov_const, Only :  &
       & errorstatus_success,&
       & errorstatus_warning,&
       & errorstatus_fatal  ,& 
       & q_mixratio_to_ppmv

  Use rttov_types, Only : &
       & rttov_coef     ,&
       & profile_Type   ,&
       & transmission_Type ,&
       & radiance_Type  ,&
       & rttov_coef_scatt_ir ,&
       & rttov_optpar_ir

  Use parkind1, Only : jpim     ,jprb
  !
  Implicit None
  !
#ifdef _RTTOV_EXAMPLE_FWD_PARALLEL
#include "rttov_parallel_direct.interface"
#define rttov_direct rttov_parallel_direct
#else
#include "rttov_direct.interface"
#endif
#include "rttov_setup.interface"
#include "rttov_errorhandling.interface"
#include "rttov_dealloc_coef.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_errorreport.interface"
#include "rttov_alloc_prof.interface"

  !--------------------------
  !
  Integer(Kind=jpim) :: iup=20      ! unit for profile file
  Integer(Kind=jpim) :: ioout=21    ! unit for output

  ! One profile per call and one sensor only for this simple example
  Integer (Kind=jpim) :: nprof = 1           ! Number of profiles per call
  Integer(Kind=jpim)  :: nrttovid = 1        ! Number of sensor coeff files to read

  ! RTTOV_errorhandling interface
  !====================
  Integer(Kind=jpim) :: Err_Unit        ! Logical error unit (<0 for default)
  Integer(Kind=jpim) :: verbosity_level ! (<0 for default)

  ! RTTOV_setup interface
  !====================
  Integer(Kind=jpim), Allocatable :: setup_errorstatus(:)  ! setup return code
  Integer(Kind=jpim), Allocatable :: instrument(:,:) !to contain platform id, sat id and sensor id
  Type( rttov_coef ), allocatable      :: coef(:)         ! optical depth coefficients
  Type(rttov_coef_scatt_ir),allocatable:: coef_scatt_ir(:)! scattering coeffs
  Type(rttov_optpar_ir),allocatable    :: optp(:)         ! optical properties

  ! RTTOV interface
  !====================
  Integer(Kind=jpim), Allocatable :: rttov_errorstatus(:)  ! rttov error return code
  Integer(Kind=jpim) :: nchannels
  Integer(Kind=jpim), Allocatable :: channels   (:)
  Integer(Kind=jpim), Allocatable :: lprofiles  (:)
  Type(profile_Type), Allocatable :: profiles(:)    
  Logical, Allocatable         :: calcemis(:)
  Real(Kind=jprb), Allocatable :: emissivity (:)
  Type(transmission_Type)  :: transmission ! transmittances and layer optical depths
  Type(radiance_Type)      :: radiance

  Integer(Kind=jpim) :: alloc_status(20)
  Integer(Kind=jpim) :: errorstatus
  Real(Kind=jprb),    Allocatable :: input_emissivity (:)
  Character (len=80) :: errMessage
  Character (len=6)  :: NameOfRoutine = 'exampl'

  ! variables for input
  !====================
  Integer(Kind=jpim), Parameter :: mxchn = 9000 ! max number of channels
  Integer(Kind=jpim) :: input_chan(mxchn)
  Real(Kind=jprb)    :: input_ems(mxchn)
  Real(Kind=jprb)    :: zenith
  Real(Kind=jprb)    :: azimut
  Real(Kind=jprb)    :: lat
  Real(Kind=jprb)    :: zerht
  Real(Kind=jprb)    :: sunzang
  Real(Kind=jprb)    :: sunazang
  Integer(Kind=jpim) :: nlevels
  Integer(Kind=jpim) :: ivch, ich
  Integer(Kind=jpim) :: asw          ! allocate or deallocate switch
  Real(Kind=jprb)    :: ems_val
  Integer(Kind=jpim), Allocatable :: nchan(:) ! number of channels per profile
  Integer(Kind=jpim) :: isurf
  Integer(Kind=jpim) :: nwater
  logical :: refrac
  logical :: solrad
  logical :: laerosl
  logical :: lclouds
  logical :: all_channels
  Logical :: addinterp  ! switch for the interpolator
  ! printing arrays
  Real(Kind=jprb), Allocatable :: pr_radcld(:)
  Real(Kind=jprb), Allocatable :: pr_trans(:)
  Real(Kind=jprb), Allocatable :: pr_emis(:)
  Real(Kind=jprb), Allocatable :: pr_trans_lev(:,:)
  Character (len=3)  :: cref
  Character (len=3)  :: caer
  Character (len=3)  :: ccld
  Character (len=3)  :: csun
  ! loop variables
  Integer(Kind=jpim) :: j, jch
  Integer(Kind=jpim) :: np, nch
  Integer(Kind=jpim) :: ilev, nprint
  Integer            :: ios

  !- End of header --------------------------------------------------------
  
  errorstatus     = 0_jpim
  alloc_status(:) = 0_jpim
  allocate (instrument(3,nrttovid),stat= alloc_status(1))

  !=====================================================
  !========== Interactive inputs == start ==============
  Write(0,*) 'enter platform number'
  Read(*,*) instrument(1,nrttovid)
  Write(0,*) 'enter satellite number '
  Read(*,*) instrument(2,nrttovid)
  Write(0,*) 'enter instrument number'
  Read(*,*) instrument(3,nrttovid)
  Write(0,*) 'enter surface type (0=land, 1=sea, 2=ice/snow)'
  Read(*,*) isurf
  Write(0,*) ' Water type (0=fresh water, 1=ocean water)'
  Read(*,*) nwater
  Write(0,*) 'enter number of profile levels'
  Read(*,*) nlevels
  Write(0,*) 'enter zenith angle in degrees'
  Read(*,*) zenith
  !
  ! Prescribe other inputs
  azimut = 0._jprb   ! Satellite azimuth angle
  sunzang = 0._jprb  ! solar zenith angle
  sunazang = 0._jprb ! solar azimuth angle
  lat = 0._jprb      ! profile latitude 
  zerht = 0._jprb    ! elevation of surface
  !
  ! Set flags
  refrac=.True.      ! include refraction in path calc
  solrad=.False.     ! Do not include reflected solar
  laerosl=.False.    ! Don't include aerosol effects
  lclouds=.False.    ! Don;t include cloud effects
  all_channels=.True.! Read all channels into memory from coef file 
  addinterp = .True. ! Allow interpolation of input profile 
  cref = 'YES'
  caer = ' NO'
  ccld = ' NO'
  csun = ' NO'
  !
  Allocate (nchan(nprof))
  nchan(:) = 0_jpim
  Read(*,*,iostat=ios) ich, ivch, ems_val ! channel number, validity, emissivity
  Do While (ios == 0 )
     If( ivch /= 0 ) Then
        nchan(nprof) = nchan(nprof) + 1_jpim
        input_chan(nchan(nprof)) = ich
        input_ems(nchan(nprof)) = ems_val
     Endif
     Read(*,*,iostat=ios) ich, ivch, ems_val
  End Do
  nchannels = nchan(nprof)   ! Number of valid channels to compute radiances

  !Pack channels and emmissivity arrays
  Allocate(channels(nchan(nprof)))  ! Note these array sizes nchan can vary per profile
  Allocate(emissivity(nchan(nprof))) ! but for this example assume 1 profile/call with same channels
  Allocate(lprofiles(nchan(nprof)))

  channels(:)     = input_chan(1:nchannels)
  emissivity(:)   = input_ems(1:nchannels)
  ! Build the list of profile indices
  nch = 0_jpim
  Do j = 1 , nprof
    DO  jch = 1,nchan(j)
      nch = nch +1_jpim
      lprofiles( nch ) = j
    End Do
  End Do
  !
  !========== Interactive inputs == end ==============
  !===================================================

  !Initialise error management with default value for
  ! the error unit number and
  ! Fatal error message output
  Err_unit = -1
  !verbosity_level = 1
  ! All error message output
  verbosity_level = 3_jpim
  Call rttov_errorhandling(Err_unit, verbosity_level)
  !
  allocate ( setup_errorstatus(nrttovid),stat= alloc_status(1))
  allocate (coef(nrttovid),stat= alloc_status(2))
  allocate (coef_scatt_ir(nrttovid),stat= alloc_status(3))
  allocate (optp(nrttovid),stat= alloc_status(4))

  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error for setup_errorstatus")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If
  !
  !Read and initialise coefficients
      Call rttov_setup (&
      & setup_errorstatus,  &! out
      & Err_unit,           &! in
      & verbosity_level,    &! in
      & nrttovid,           &! in
      & laerosl,            &! in
      & lclouds,            &! in
      & coef,               &! out
      & coef_scatt_ir,      &! out
      & optp,               &
      & instrument)         ! in

  if(any(setup_errorstatus(:) /= errorstatus_success ) ) then
     write ( *,* ) 'rttov_setup fatal error'
     stop
  endif
  deallocate( setup_errorstatus ,stat=alloc_status(1))
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error for setup_errorstatus")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  ! security if input number of channels is higher than number
  ! stored in coeffs
  If( nchannels > coef(nrttovid) % fmv_chn ) Then
     nchannels = coef(nrttovid) % fmv_chn
     nchan(nprof) = coef(nrttovid) % fmv_chn
  Endif

  !Open output file which prints results
  Open(IOOUT,file='print.dat',status='unknown',form='formatted',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening the output file ios= ',ios
     Stop
  Endif

  !===============================================
  !========== Read profile == start ==============
  Open(iup, file='prof.dat',status='old',iostat=ios)
  If( ios /= 0 ) Then
     Write(*,*) 'error opening profile file ios= ',ios
     Stop
  Endif

  ! Do allocation of input profile arrays with the number of levels.
  asw = 1 ! allocate
  allocate( profiles(nprof),stat= alloc_status(1))
  profiles(1) % nlevels = nlevels
  call rttov_alloc_prof (errorstatus,nprof,profiles,nlevels,coef_scatt_ir(nrttovid),asw, &
    & addaerosl = laerosl, addclouds = lclouds, init = .true. )

  ! Presures are from reference profile
  profiles(1) % p(:) = coef(nrttovid) % ref_prfl_p(:)

  ! read pressure, temp (K), WV (lnq), O3 (ppmv)
  ! take care of doing the unit conversions to
  ! hPa, K and ppmv
  Read(iup,*) profiles(1) % t(:)
  Read(iup,*) profiles(1) % q(:)
  Read(iup,*) profiles(1) % o3(:)
  Read(iup,*) profiles(1) % clw(:)
  ! 2 meter air variables
  Read(iup,*) profiles(1) % s2m % t ,&
        & profiles(1) % s2m % q ,&
        & profiles(1) % s2m % p ,&
        & profiles(1) % s2m % u ,&
        & profiles(1) % s2m % v 

  !  Convert lnq to q in ppmv for profile
  profiles(1) %    q(:) = (Exp(profiles(1) %     q(:)) / 1000._JPRB) * q_mixratio_to_ppmv
  profiles(1) % s2m % q = (Exp(profiles(1) %  s2m % q) / 1000._JPRB) * q_mixratio_to_ppmv

  ! Skin variables
  Read(iup,*) profiles(1) % skin % t ,&
        & profiles(1) % skin % fastem 

  ! Cloud variables
  Read(iup,*) profiles(1) % ctp,&
        & profiles(1) % cfraction 
  Close(iup)
  ! we have an ozone profile
  profiles(1) % ozone_Data =.True.
  ! we do not have CO2 profile
  profiles(1) % co2_Data   =.False.
  profiles(1) %  co2(:) =0._jprb
  ! we do not have n2o profile
  profiles(1) % n2o_data = .false.
  profiles(1) %  n2o(:) =0._jprb
  ! we do not have CH4 profile
  profiles(1) % ch4_Data   =.False.
  profiles(1) %  ch4(:) =0._jprb
  ! we do not have CO profile
  profiles(1) % co_Data   =.False.
  profiles(1) %  co(:) =0._jprb
  ! check Cloud liquid water profile
  profiles(1) % clw_Data   = profiles(1) % clw(1) >= 0.0_JPRB
  ! Other variables from interactive inputs
  profiles(1) % zenangle   = zenith
  profiles(1) % azangle    = azimut
  profiles(1)  % latitude      = LAT
  profiles(1)  % elevation     = ZERHT
  profiles(1)  % sunzenangle   = SUNZANG
  profiles(1)  % sunazangle    = SUNAZANG
  profiles(1)  % addsolar      = solrad
  profiles(1)  % addrefrac     = refrac
  ! surface type
  profiles(1) % skin % surftype  = isurf
  profiles(1) % skin % watertype = nwater
  profiles(1) % aer_data   = laerosl
  profiles(1) % cld_data   = lclouds
  profiles(1) %idg         = 0._jprb
  profiles(1) %ish         = 0._jprb
  if( lclouds ) then
    profiles(1) %cloud(:,:)  = 0._jprb
    profiles(1) %cfrac(:,:)  = 0._jprb
  endif
  !========== Read profile == end ==============
  !=============================================

  ! allocate radiance results arrays with number of channels
  asw = 1 ! allocate
  call rttov_alloc_rad (errorstatus,nchannels,radiance,nlevels,asw)
  If( errorstatus /= 0) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for radiance arrays")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Stop
  Endif
  Allocate( calcemis ( nchannels ) ,stat= alloc_status(2))
  Allocate( input_emissivity ( nchannels ) ,stat= alloc_status(3))

  ! allocate transmittance structure
  Allocate( transmission % tau_layers(nlevels,nchannels ) ,stat= alloc_status(4))
  Allocate( transmission % tau_total (nchannels ) ,stat= alloc_status(5))

  ! Allocate Error flag
  Allocate( rttov_errorstatus(1)           ,stat= alloc_status(7))

  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error prior to rttov_direct")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  ! save input values of emissivities for all calculations
  ! calculate emissivity where the input emissivity value is less than 0.01
  input_emissivity(:) = emissivity(:)
  calcemis(:) = emissivity(:) < 0.01_JPRB
  ! Call RTTOV forward model
    call rttov_direct(  &
          & rttov_errorstatus, &! out error flag
          & nprof,             &! in  number of profiles computed
          & nchannels,         &! in  total number of channels computed for all profiles
          & channels,          &! in  array of channel indices
          & lprofiles,         &! in  array of profile indices
          & addinterp,         &! in  switch for profile interpolation
          & profiles,          &! in  profile array
          & coef(nrttovid),    &! in  coeff array
          & coef_scatt_ir(nrttovid), &! in  scatterinf coeff array
          & optp(nrttovid),    &! in  optical props array
          & solrad,            &! in  solar calc flag
          & laerosl,           &! in  aerosol radiance flag
          & lclouds,           &! in  cloudy radiance flag
          & calcemis,          &! in  flag for internal emissivity calc 
          & emissivity,        &! inout input emissivities per channel
          & transmission,      &! out array of transmittances 
          & radiance  )         ! inout computed radiance array

  If ( Any( rttov_errorstatus(:) == errorstatus_warning ) ) Then
     Write ( ioout, * ) 'rttov_direct warning'
  End If

  If ( Any( rttov_errorstatus(:) == errorstatus_fatal ) ) Then
     Write ( 0, * ) 'rttov_direct error'
     Stop
  End If

  ! transfer data to printing arrays
  Allocate(pr_radcld(nchannels)  ,stat= alloc_status(1))
  Allocate(pr_trans(nchannels)   ,stat= alloc_status(2))
  Allocate(pr_emis(nchannels)    ,stat= alloc_status(3))
  Allocate(pr_trans_lev(nlevels,nchannels) ,stat= alloc_status(4))
  If( Any(alloc_status /= 0) ) Then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error for printing arrays")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Stop
  End If

  pr_radcld(:) = 0.0_JPRB
  pr_trans(:)  = 0.0_JPRB
  pr_emis(:)   = 0.0_JPRB
  pr_trans_lev(:,:) = 0.0_JPRB
  !
  Do j = 1 , nchannels
     pr_radcld(j) = radiance % cloudy(j)
     pr_trans(j)  = Transmission % tau_total(j)
     pr_emis(j)   = emissivity(j)
     Do ilev = 1 ,  nlevels
        pr_trans_lev(ilev,j) = Transmission % tau_layers(ilev,J)
     Enddo
  Enddo
  !
  !     OUTPUT RESULTS
  !
  NPRINT = 1+ Int((nchannels-1)/10)
  Write(IOOUT,*)' -----------------'
  Write(IOOUT,*)' Instrument ', instrument(3,nrttovid)
  Write(IOOUT,*)' -----------------'
  Write(IOOUT,*)' '
  WRITE(IOOUT,777) profiles(nprof)%zenangle,profiles(nprof)%azangle, &
      & csun,profiles(nprof)%sunzenangle,profiles(nprof)%sunazangle,profiles(nprof)%skin%surftype,&
      & profiles(nprof)%skin%watertype,profiles(nprof)%latitude,profiles(nprof)%elevation,cref,caer,ccld,&
      & instrument(2,nrttovid)

  WRITE(IOOUT,*)'CHANNELS PROCESSED:'
  WRITE(IOOUT,111) (channels(J), J = 1,nchannels)
  WRITE (IOOUT,*)' '
  Write(IOOUT,222) radiance % bt(:)
  Write(IOOUT,*)' '
  Write(IOOUT,*)'CALCULATED RADIANCES: SAT =', instrument(2,nrttovid)
  Write(IOOUT,222) radiance % total(:)
  Write(IOOUT,*)' '
  Write(IOOUT,*)'CALCULATED OVERCAST RADIANCES: SAT =', instrument(2,nrttovid)
  Write(IOOUT,222) pr_radcld(:)
  Write (IOOUT,*)' '
  Write(IOOUT,*)'CALCULATED SURFACE TO SPACE TRANSMITTANCE: S'&
             & ,'AT =',instrument(2,nrttovid) 
  Write(IOOUT,4444) pr_trans(:)
  Write (IOOUT,*)' '
  Write(IOOUT,*)'CALCULATED SURFACE EMISSIVITIES '&
             & ,'SAT =',instrument(2,nrttovid) 
  Write(IOOUT,444) pr_emis(:)
  !
  !
  If(nchan(nprof) <= 20)Then
     Do  NP = 1 , NPRINT
        Write (IOOUT,*)' '
        Write (IOOUT,*)'Level to space transmittances for channels'
        Write(IOOUT,1115) (channels(j),&
                & J = 1+(NP-1)*10,Min(Int(10+(NP-1)*10,jpim),nchannels)) 
        Do  ILEV = 1 , NLEVELS
           Write(IOOUT,4445)ILEV,(pr_trans_lev(ilev,J),&
                   & J = 1+(NP-1)*10,Min(Int(10+(NP-1)*10,jpim),nchannels)) 
        End Do
        Write(IOOUT,1115) (CHANNELS(J),&
                & J = 1+(NP-1)*10,Min(Int(10+(NP-1)*10,jpim),nchannels)) 
     End Do
  Endif
  !
  ! Deallocate arrays
     deallocate( channels   ,stat=alloc_status(1))
     deallocate( lprofiles  ,stat=alloc_status(2))
     deallocate( emissivity ,stat=alloc_status(3))
     deallocate( input_emissivity ,stat=alloc_status(4))
     deallocate( calcemis   ,stat=alloc_status(5))

    ! deallocate transmittances
     Deallocate( transmission % tau_total   ,stat= alloc_status(7))
     Deallocate( transmission % tau_layers  ,stat= alloc_status(8))
    ! dealloc printing arrays
     deallocate(pr_radcld ,stat= alloc_status(9))
     deallocate(pr_trans ,stat= alloc_status(10))
     deallocate(pr_emis ,stat= alloc_status(11))
     deallocate(pr_trans_lev ,stat= alloc_status(12))
     If( any(alloc_status /= 0) ) then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "mem deallocation error")' )
        Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
        Stop
     End If

     asw = 0 ! deallocate radiance arrays
     call rttov_alloc_rad (errorstatus,nchannels,radiance,nlevels,asw)
     If(errorstatus /= errorstatus_success) Then
        Write( errMessage, '( "radiance deallocation error")' )
        Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Endif

     asw = 0 ! deallocate profile arrays
     call rttov_alloc_prof (errorstatus,nprof,profiles,nlevels,coef_scatt_ir(nrttovid),asw, &
    & addaerosl = laerosl, addclouds = lclouds )
     deallocate( profiles,stat=alloc_status(1))
     If(errorstatus /= errorstatus_success .or. alloc_status(1) /= 0) Then
        Write( errMessage, '( "profile deallocation error")' )
        Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Endif

     Call rttov_dealloc_coef (errorstatus, coef(nrttovid),coef_scatt_ir(nrttovid),optp(nrttovid))
     If(errorstatus /= errorstatus_success) Then
        Write( errMessage, '( "coef deallocation error")' )
        Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
     Endif

   !Close output file
    Close(IOOUT,iostat=ios)
    If( ios /= 0 ) Then
      Write(*,*) 'error closing the output file ios= ',ios
      Stop
    Endif

111  FORMAT(1X,10I8)
1115 Format(3X,10I8)
222  Format(1X,10F8.2)
444  Format(1X,10F8.3)
4444 Format(1X,10F8.4)
4445 Format(1X,I2,10F8.4)
777 FORMAT( &
       & ' ZENITH ANGLE       =',F7.2,/ &
       & ' AZIMUTH ANGLE      =',F7.2,/&
       & ' SOLAR RADIATION    =',A7,/&
       & ' SOLAR ZENITH ANGLE =',F7.2,/&
       & ' SOLAR AZIMUTH ANGLE=',F7.2,/ &
       & ' SURFACE TYPE       =',I7,/&
       & ' WATER TYPE         =',I7,/ &
       & ' LATITUDE           =',F7.2,/&
       & ' ELEVATION          =',F7.2/&
       & ' REFRACTION         =',A7,/&
       & ' AEROSOLS           =',A7,/&
       & ' CLOUDS             =',A7,//,&
       &'CALCULATED BRIGHTNESS TEMPERATURES: SAT =',I2 )

End Program example_fwd
