Subroutine rttov_check_temp( &
      & errorstatus,     &
      & nprofiles,       &
      & nchannels,       &
      & channels,        &
      & addinterp,       &
      & nlevels,         &
      & coef,            &
      & coef_scatt_ir,   &
      & optp,            &
      & addsolar,        &
      & addaerosl,       &
      & addclouds,       &
      & asw,             &
      & temp0,    temp1,    temp2,    &
      & temp0_tl, temp1_tl, temp2_tl, &
      & temp0_ad, temp1_ad, temp2_ad, &
      & temp0_k,  temp1_k,  temp2_k   &
      & )    


  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  ! 
  ! Description:
  !    Allocates/deallocates temporary data for rttov_direct
  !

  Use parkind1
  Use rttov_types
  Use rttov_const

  Implicit None

  Integer(Kind=jpim),       Intent(in)    :: nprofiles                    
  Integer(Kind=jpim),       Intent(in)    :: nchannels                    
  Integer(Kind=jpim),       Intent(in)    :: channels       (nchannels)   
  Logical,                  Intent(in)    :: addsolar
  Logical,                  Intent(in)    :: addaerosl                    
  Logical,                  Intent(in)    :: addclouds                    
  Logical,                  Intent(in)    :: addinterp                    

  Integer(Kind=jpim),       Intent(in)    :: nlevels
  Type(rttov_coef),         Intent(in), Target :: coef                         
  Type(rttov_coef_scatt_ir),Intent(in), Target :: coef_scatt_ir                
  Type(rttov_optpar_ir),    Intent(in), Target :: optp                         

  Integer(Kind=jpim),       Intent(out)   :: errorstatus 
  Integer(Kind=jpim),       Intent(in)    :: asw
  Type(rttov_temp),   Pointer, Optional                :: temp0, temp0_tl, temp0_ad, temp0_k
  Type(rttov_temp),   Target,  Optional, Intent(inout) :: temp1, temp1_tl, temp1_ad, temp1_k
  Type(rttov_temp),   Target,  Optional, Intent(in)    :: temp2, temp2_tl, temp2_ad, temp2_k

!INTF

#include "rttov_errorreport.interface" 


  If( present( temp0 ) )    &
    Call check_temp(        &
      & errorstatus,        &
      & nprofiles,          &
      & nchannels,          &
      & channels,           &
      & addinterp,          &
      & nlevels,            &
      & coef,               &
      & coef_scatt_ir,      &
      & optp,               &
      & addsolar,           &
      & addaerosl,          &
      & addclouds,          &
      & asw,                &
      & temp0,              &
      & temp1,              &
      & temp2 )    

  If( errorstatus .ne. errorstatus_success ) Return

  If( present( temp0_tl ) ) &
    Call check_temp(        &
      & errorstatus,        &
      & nprofiles,          &
      & nchannels,          &
      & channels,           &
      & addinterp,          &
      & nlevels,            &
      & coef,               &
      & coef_scatt_ir,      &
      & optp,               &
      & addsolar,           &
      & addaerosl,          &
      & addclouds,          &
      & asw,                &
      & temp0_tl,           &
      & temp1_tl,           &
      & temp2_tl )    

  If( errorstatus .ne. errorstatus_success ) Return

  If( present( temp0_ad ) ) &
    Call check_temp(        &
      & errorstatus,        &
      & nprofiles,          &
      & nchannels,          &
      & channels,           &
      & addinterp,          &
      & nlevels,            &
      & coef,               &
      & coef_scatt_ir,      &
      & optp,               &
      & addsolar,           &
      & addaerosl,          &
      & addclouds,          &
      & asw,                &
      & temp0_ad,           &
      & temp1_ad,           &
      & temp2_ad )    

  If( errorstatus .ne. errorstatus_success ) Return

  If( present( temp0_k ) )  &
    Call check_temp(        &
      & errorstatus,        &
      & nchannels,          &
      & nchannels,          &
      & channels,           &
      & addinterp,          &
      & nlevels,            &
      & coef,               &
      & coef_scatt_ir,      &
      & optp,               &
      & addsolar,           &
      & addaerosl,          &
      & addclouds,          &
      & asw,                &
      & temp0_k,            &
      & temp1_k,            &
      & temp2_k )    

  Contains
  
  Subroutine check_temp( &
      & errorstatus,     &
      & nprofiles,       &
      & nchannels,       &
      & channels,        &
      & addinterp,       &
      & nlevels,         &
      & coef,            &
      & coef_scatt_ir,   &
      & optp,            &
      & addsolar,        &
      & addaerosl,       &
      & addclouds,       &
      & asw,             &
      & temp0,           &
      & temp1,           &
      & temp2 )    

  Integer(Kind=jpim),       Intent(in)    :: nprofiles                    
  Integer(Kind=jpim),       Intent(in)    :: nchannels                    
  Integer(Kind=jpim),       Intent(in)    :: channels       (nchannels)   
  Logical,                  Intent(in)    :: addsolar
  Logical,                  Intent(in)    :: addaerosl                    
  Logical,                  Intent(in)    :: addclouds                    
  Logical,                  Intent(in)    :: addinterp                    

  Integer(Kind=jpim),       Intent(in)    :: nlevels
  Type(rttov_coef),         Intent(in), Target    :: coef                         
  Type(rttov_coef_scatt_ir),Intent(in), Target    :: coef_scatt_ir                
  Type(rttov_optpar_ir),    Intent(in), Target    :: optp                         

  Integer(Kind=jpim),       Intent(out)   :: errorstatus 
  Integer(Kind=jpim),       Intent(in)    :: asw
  Type(rttov_temp),   Pointer :: temp0
  Type(rttov_temp),   Target, Optional, Intent(inout) :: temp1
  Type(rttov_temp),   Target, Optional, Intent(in)    :: temp2

#include "rttov_alloc_temp.interface"
  
  Character*(*), Parameter :: NameOfRoutine = 'rttov_check_temp'
  Character*128 :: errMessage
  
  errorstatus = errorstatus_success

  If( asw .eq. 1_jpim ) Then
    If( Present( temp2 ) ) Then
      
      If( ( .not. associated( temp2%coef, coef ) )                   .or. &
      &   ( .not. associated( temp2%coef_scatt_ir, coef_scatt_ir ) ) .or. &
      &   ( .not. associated( temp2%optp, optp ) )                   .or. &
      &   ( temp2%nchannels .ne. nchannels )                         .or. &
      &   ( temp2%addsolar  .neqv. addsolar  )                       .or. &
      &   ( temp2%addaerosl .neqv. addaerosl )                       .or. &
      &   ( temp2%addclouds .neqv. addclouds )                       .or. &
      &   ( temp2%addinterp .neqv. addinterp )                       .or. &
      &   ( temp2%nlevels   .ne. nlevels   ) ) Then
        errorstatus = errorstatus_fatal
        Write( errMessage, '( "rttov_check_temp fatal error dimensions mismatch")' )
        Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
        Return
      EndIf
      temp0 => temp2
    Else
      Call rttov_alloc_temp( &
        & errorstatus,       &
        & nprofiles,         &
        & nchannels,         &
        & channels,          &
        & addinterp,         &
        & nlevels,           &
        & coef,              &
        & coef_scatt_ir,     &
        & optp,              &
        & addsolar,          &
        & addaerosl,         &
        & addclouds,         &
        & 1_jpim,            & 
        & temp1 )    
      If( errorstatus .eq. errorstatus_fatal ) Then
        Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
        Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
        Return
      EndIf
      temp0 => temp1
    EndIf
  Else
    If( Present( temp2 ) ) Then
      nullify(temp0)
    Else
      Call rttov_alloc_temp( &
        & errorstatus,       &
        & nprofiles,         &
        & nchannels,         &
        & channels,          &
        & addinterp,         &
        & nlevels,           &
        & coef,              &
        & coef_scatt_ir,     &
        & optp,              &
        & addsolar,          &
        & addaerosl,         &
        & addclouds,         &
        & 0_jpim,            &
        &  temp1 )    
      If( errorstatus .eq. errorstatus_fatal ) Then
        Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
        Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
        Return
      EndIf
      temp0 => temp1
    EndIf
  EndIf

  End Subroutine

End Subroutine
