!
Subroutine rttov_distribcoeffs  (&
      & kmyproc,       &! id proc
      & kioproc,       &! proc for io
      & laerosol,      &!
      & lcloud,        &!
      & coef,          &! inout 
      & coef_scatt_ir, &! inout
      & optp)           ! inout
  ! Description:
  !
  ! Communicate the coefficient reading by proc 1 to other procs
  !
  ! Copyright:
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       13/05/2004  Original
  !  1.1       20/07/2008  Marco Matricardi. ECMWF. Corrected the allocation
  !                        of the coeff's arrays 
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :  gas_id_mixed, ngases_max
  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
       & rttov_coef,          &
       & rttov_coef_scatt_ir, &
       & rttov_optpar_ir   
 
  Use parkind1, Only : jpim     ,jprb
  Implicit None

  ! subroutine arguments
  ! scalar arguments with intent(in):
  Integer(Kind=jpim), Intent(in) :: kmyproc  ! logical processor id
  Integer(Kind=jpim), Intent(in) :: kioproc  ! processor  dedicated for io
  Logical, Intent(in) :: laerosol  ! Switch for aerosol 
  Logical, Intent(in) :: lcloud    ! Switch for cloud

  ! scalar arguments with intent(inout):
  Type( rttov_coef ), Intent (inout) :: coef   ! coefficients 
  Type( rttov_coef_scatt_ir),Intent (inout) :: coef_scatt_ir
  Type( rttov_optpar_ir),Intent (inout)     :: optp

!INTF
#include "rttov_distribcoef_scatt_ir.interface"

  ! Local Scalars:
  Integer(Kind=jpim)  :: kcoef(101)
  Real(Kind=JPRB)     :: zcoef(4)
  Real(Kind=JPRB),Allocatable :: zcmplx(:,:)
  Character (len=144) :: ccoef
  Integer(Kind=jpim)  :: kdim,itag
  Integer(Kind=jpim)  :: i
  Logical             :: lallo

  ! Functions:
  REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

  ! 0 Initialise variables
  !---------------------------------------------
  IF (LHOOK) CALL DR_HOOK('RTTOV_DISTRIBCOEFFS',0_jpim,ZHOOK_HANDLE)

  kcoef(:) = 0
  zcoef(:) = 0.0
  ccoef =  'xxxx'
  lallo = kmyproc.ne.kioproc

  If (kmyproc == kioproc) Then
    kcoef(1)= coef % id_platform
    kcoef(2)= coef % id_sat
    kcoef(3)= coef % id_inst
    kcoef(4)= coef % id_sensor
    kcoef(5)= coef % id_comp_lvl
    kcoef(6:8)= coef % id_creation_date
    kcoef(9)= coef%fmv_model_ver
    kcoef(10)= coef % fmv_chn
    kcoef(11)= coef % fmv_gas
    kcoef(12)= coef % nmixed
    kcoef(13)= coef % nwater
    kcoef(14)= coef % nozone
    kcoef(15)= coef % nwvcont
    kcoef(16)= coef % nco2
    kcoef(17)= coef % nn2o
    kcoef(18)= coef % nco
    kcoef(19)= coef % nch4
    kcoef(20)= coef % nlevels
    kcoef(21)= coef % ncmixed
    kcoef(22)= coef % ncwater
    kcoef(23)= coef % ncozone
    kcoef(24)= coef % ncwvcont
    kcoef(25)= coef % ncco2
    kcoef(26)= coef % ncn2o
    kcoef(27)= coef % ncco
    kcoef(28)= coef % ncch4
    kcoef(29)= coef % nintmixed
    kcoef(30)= coef % nintwater
    kcoef(31)= coef % nintozone
    kcoef(32)= coef % nintwvcont
    kcoef(33)= coef % nintco2
    kcoef(34)= coef % nintn2o
    kcoef(35)= coef % nintco
    kcoef(36)= coef % nintch4
    kcoef(37)= coef % ws_nomega
    kcoef(38)= coef % fastem_ver
    kcoef(39)= coef % fastem_coef_nb
    kcoef(40)= coef % ssirem_ver
    kcoef(41)= coef % mwcldtop
    kcoef(42)= coef % nlayers

! Check if array are associated in PE1 to send the information to other PEs

    kcoef(43:101)=0

    ! Integer arrays
    If (Associated( coef % fmv_gas_id))   kcoef(43)=1
    If (Associated( coef % fmv_gas_pos))  kcoef(44)=1
    If (Associated( coef % fmv_var))      kcoef(45)=1
    If (Associated( coef % fmv_coe))      kcoef(46)=1
    If (Associated( coef % fmv_lvl))      kcoef(47)=1
    If (Associated( coef % gaz_units))    kcoef(48)=1
    If (Associated( coef % ff_ori_chn))   kcoef(49)=1
    If (Associated( coef % ff_val_chn))   kcoef(50)=1
    If (Associated( coef % tt_chn))       kcoef(51)=1
    If (Associated( coef % tt_val_chn))   kcoef(52)=1
    If (Associated( coef % ss_chn))       kcoef(53)=1
    If (Associated( coef % ss_val_chn))   kcoef(54)=1
    If (Associated( coef % woc_chn))      kcoef(55)=1
    If (Associated( coef % fastem_polar)) kcoef(56)=1
    If (Associated( coef % ssirem_chn))   kcoef(57)=1
    ! Real arrays
    If (Associated( coef % ff_cwn))       kcoef(58)=1
    If (Associated( coef % ff_bco))       kcoef(59)=1
    If (Associated( coef % ff_bcs))       kcoef(60)=1
    If (Associated( coef % ff_gam))       kcoef(61)=1
    If (Associated( coef % tt_cwn))       kcoef(62)=1
    If (Associated( coef % tt_a0))        kcoef(63)=1
    If (Associated( coef % tt_a1))        kcoef(64)=1
    If (Associated( coef % ss_cwn))       kcoef(65)=1
    If (Associated( coef % ss_solar_spectrum)) kcoef(66)=1
    If (Associated( coef % woc_cwn))      kcoef(67)=1
    If (Associated( coef % ws_npoint))    kcoef(68)=1
    If (Associated( coef % ws_k_omega))   kcoef(69)=1
    If (Associated( coef % fastem_coef))  kcoef(70)=1
    If (Associated( coef % ssirem_a0))    kcoef(71)=1
    If (Associated( coef % ssirem_a1))    kcoef(72)=1
    If (Associated( coef % ssirem_a2))    kcoef(73)=1
    If (Associated( coef % ssirem_xzn1))  kcoef(74)=1
    If (Associated( coef % ssirem_xzn2))  kcoef(75)=1
    If (Associated( coef % ref_prfl_p))   kcoef(76)=1
    If (Associated( coef % ref_prfl_t))   kcoef(77)=1
    If (Associated( coef % ref_prfl_mr))  kcoef(78)=1
    If (Associated( coef % lim_prfl_p))   kcoef(79)=1
    If (Associated( coef % lim_prfl_tmax)) kcoef(80)=1
    If (Associated( coef % lim_prfl_tmin)) kcoef(81)=1
    If (Associated( coef % lim_prfl_gmax)) kcoef(82)=1
    If (Associated( coef % lim_prfl_gmin)) kcoef(83)=1
    If (Associated( coef % mixedgas))     kcoef(84)=1
    If (Associated( coef % watervapour))  kcoef(85)=1
    If (Associated( coef % ozone))        kcoef(86)=1
    If (Associated( coef % wvcont))       kcoef(87)=1
    If (Associated( coef % co2))          kcoef(88)=1
    If (Associated( coef % n2o))          kcoef(89)=1
    If (Associated( coef % co))           kcoef(90)=1
    If (Associated( coef % ch4))          kcoef(91)=1
    If (Associated( coef % mixedgasint))  kcoef(92)=1
    If (Associated( coef % watervapourint)) kcoef(93)=1
    If (Associated( coef % ozoneint))     kcoef(94)=1
    If (Associated( coef % wvcontint))    kcoef(95)=1
    If (Associated( coef % co2int))       kcoef(96)=1
    If (Associated( coef % n2oint))       kcoef(97)=1
    If (Associated( coef % coint))        kcoef(98)=1
    If (Associated( coef % ch4int))       kcoef(99)=1
    ! Complex arrays - treated as reals
    If (Associated( coef % woc_waopc_ow)) kcoef(100)=1
    If (Associated( coef % woc_waopc_fw)) kcoef(101)=1

    ! Characters
    ccoef= coef % id_creation//coef % id_Common_name//coef % fmv_model_def

    ! Real scalars
    zcoef(1)= coef % fc_speedl
    zcoef(2)= coef % fc_planck_c1
    zcoef(3)= coef % fc_planck_c2
    zcoef(4)= coef % fc_sat_height

  End If

! Send 'constant' information to other PEs

  itag=7000
#ifdef _RTTOV_DO_DISTRIBCOEF
  Call broadcint(kcoef,Size(kcoef),kioproc,ITAG+1)
  Call broadcreal(zcoef,Size(zcoef),kioproc,ITAG+2)
  Call broadcchar(ccoef,144,kioproc,ITAG+3)
#endif

  If (kmyproc.ne.kioproc) Then
    coef % id_platform     =kcoef(1)
    coef % id_sat          =kcoef(2)
    coef % id_inst         =kcoef(3)
    coef % id_sensor       =kcoef(4)
    coef % id_comp_lvl     =kcoef(5)
    coef % id_creation_date=kcoef(6:8)
    coef % fmv_model_ver   =kcoef(9)
    coef % fmv_chn         =kcoef(10)
    coef % fmv_gas         =kcoef(11)
    coef % nmixed          =kcoef(12)
    coef % nwater          =kcoef(13)
    coef % nozone          =kcoef(14)
    coef % nwvcont         =kcoef(15)
    coef % nco2            =kcoef(16)
    coef % nn2o            =kcoef(17)
    coef % nco             =kcoef(18)
    coef % nch4            =kcoef(19)
    coef % nlevels         =kcoef(20)
    coef % ncmixed         =kcoef(21)
    coef % ncwater         =kcoef(22)
    coef % ncozone         =kcoef(23)
    coef % ncwvcont        =kcoef(24)
    coef % ncco2           =kcoef(25)
    coef % ncn2o           =kcoef(26)
    coef % ncco            =kcoef(27)
    coef % ncch4           =kcoef(28)
    coef % nintmixed       =kcoef(29)
    coef % nintwater       =kcoef(30)
    coef % nintozone       =kcoef(31)
    coef % nintwvcont      =kcoef(32)
    coef % nintco2         =kcoef(33)
    coef % nintn2o         =kcoef(34)
    coef % nintco          =kcoef(35)
    coef % nintch4         =kcoef(36)
    coef % ws_nomega       =kcoef(37)
    coef % fastem_ver      =kcoef(38)
    coef % fastem_coef_nb  =kcoef(39)
    coef % ssirem_ver      =kcoef(40)
    coef % mwcldtop        =kcoef(41)
    coef % nlayers         =kcoef(42)

    coef % id_creation=ccoef(1:80)
    coef % id_Common_name=ccoef(81:112)
    coef % fmv_model_def=ccoef(113:144)
  
    coef % fc_speedl=zcoef(1)
    coef % fc_planck_c1=zcoef(2)
    coef % fc_planck_c2=zcoef(3)
    coef % fc_sat_height=zcoef(4)
  End If
  
! Send 'arrays' information if associated to other PEs

  If( kcoef(43) == 1 ) Then
    if (lallo) then
       if(Associated(coef % fmv_gas_id)) Deallocate( coef % fmv_gas_id)
       Allocate(coef % fmv_gas_id ( coef % fmv_gas ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fmv_gas_id,coef % fmv_gas,kioproc,ITAG+41)
#endif
  End If

  If( kcoef(44) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fmv_gas_pos))  Deallocate(coef % fmv_gas_pos)
      Allocate(coef % fmv_gas_pos ( ngases_max ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fmv_gas_pos, ngases_max,kioproc,ITAG+42)
#endif
  End If

  If( kcoef(45) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fmv_var)) Deallocate(coef % fmv_var)
      Allocate(coef % fmv_var ( coef % fmv_gas ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fmv_var,coef % fmv_gas,kioproc,ITAG+43)
#endif
  End If

  If( kcoef(46) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fmv_coe)) Deallocate(coef % fmv_coe)
      Allocate(coef % fmv_coe ( coef % fmv_gas ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fmv_coe,coef % fmv_gas,kioproc,ITAG+44)
#endif
  End If

  If( kcoef(47) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fmv_lvl)) Deallocate(coef % fmv_lvl)
      Allocate(coef % fmv_lvl ( coef % fmv_gas ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fmv_lvl,coef % fmv_gas,kioproc,ITAG+45)
#endif
  End If

  If( kcoef(48) == 1 ) Then
    if (lallo) then
      if(Associated(coef % gaz_units)) Deallocate(coef % gaz_units)
      Allocate(coef % gaz_units ( coef % fmv_gas ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % gaz_units,coef % fmv_gas,kioproc,ITAG+46)
#endif
  End If

  If( kcoef(49) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_ori_chn)) Deallocate(coef % ff_ori_chn)
      Allocate(coef % ff_ori_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % ff_ori_chn,coef % fmv_chn,kioproc,ITAG+47)
#endif
  End If

  If( kcoef(50) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_val_chn)) Deallocate(coef % ff_val_chn)
      Allocate(coef % ff_val_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % ff_val_chn,coef % fmv_chn,kioproc,ITAG+48)
#endif
  End If

  If( kcoef(51) == 1 ) Then
    if (lallo) then
      if(Associated(coef % tt_chn)) Deallocate(coef % tt_chn)
      Allocate(coef % tt_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % tt_chn,coef % fmv_chn,kioproc,ITAG+49)
#endif
  End If

  If( kcoef(52) == 1 ) Then
    if (lallo) then
      if(Associated(coef % tt_val_chn)) Deallocate(coef % tt_val_chn)
      Allocate(coef % tt_val_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % tt_val_chn,coef % fmv_chn,kioproc,ITAG+50)
#endif
  End If

  If( kcoef(53) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ss_chn)) Deallocate(coef % ss_chn)
      Allocate(coef % ss_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % ss_chn,coef % fmv_chn,kioproc,ITAG+51)
#endif
  End If

  If( kcoef(54) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ss_val_chn)) Deallocate(coef % ss_val_chn)
      Allocate(coef % ss_val_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % ss_val_chn,coef % fmv_chn,kioproc,ITAG+52)
#endif
  End If

  If( kcoef(55) == 1 ) Then
    if (lallo) then
      if(Associated(coef % woc_chn)) Deallocate(coef % woc_chn)
      Allocate(coef % woc_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % woc_chn,coef % fmv_chn,kioproc,ITAG+53)
#endif
  End If

  If( kcoef(56) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fastem_polar)) Deallocate(coef % fastem_polar)
      Allocate(coef % fastem_polar ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % fastem_polar,coef % fmv_chn,kioproc,ITAG+54)
#endif
  End If

  If( kcoef(57) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_chn)) Deallocate(coef % ssirem_chn)
      Allocate(coef % ssirem_chn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcint(coef % ssirem_chn,coef % fmv_chn,kioproc,ITAG+55)
#endif
  End If

  If( kcoef(58) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_cwn)) Deallocate(coef % ff_cwn)
      Allocate(coef % ff_cwn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ff_cwn,coef % fmv_chn,kioproc,ITAG+56)
#endif
  End If

  If( kcoef(59) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_bco)) Deallocate(coef % ff_bco)
      Allocate(coef % ff_bco ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ff_bco,coef % fmv_chn,kioproc,ITAG+57)
#endif
  End If

  If( kcoef(60) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_bcs)) Deallocate(coef % ff_bcs)
      Allocate(coef % ff_bcs ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ff_bcs,coef % fmv_chn,kioproc,ITAG+58)
#endif
  End If

  If( kcoef(61) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ff_gam)) Deallocate(coef % ff_gam)
      Allocate(coef % ff_gam ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ff_gam,coef % fmv_chn,kioproc,ITAG+59)
#endif
  End If

  If( kcoef(62) == 1 ) Then
    if (lallo) then
      if(Associated(coef % tt_cwn)) Deallocate(coef % tt_cwn)
      Allocate(coef % tt_cwn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % tt_cwn,coef % fmv_chn,kioproc,ITAG+60)
#endif
  End If

  If( kcoef(63) == 1 ) Then
    if (lallo) then
      if(Associated(coef % tt_a0)) Deallocate(coef % tt_a0)
      Allocate(coef % tt_a0 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % tt_a0,coef % fmv_chn,kioproc,ITAG+61)
#endif
  End If

  If( kcoef(64) == 1 ) Then
    if (lallo) then
      if(Associated(coef % tt_a1)) Deallocate(coef % tt_a1)
      Allocate(coef % tt_a1 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % tt_a1,coef % fmv_chn,kioproc,ITAG+62)
#endif
  End If


  If( kcoef(65) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ss_cwn)) Deallocate(coef % ss_cwn)
      Allocate(coef % ss_cwn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ss_cwn,coef % fmv_chn,kioproc,ITAG+63)
#endif
  End If

  If( kcoef(66) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ss_solar_spectrum)) Deallocate(coef % ss_solar_spectrum)
      Allocate(coef % ss_solar_spectrum ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ss_solar_spectrum,coef % fmv_chn,kioproc,ITAG+64)
#endif
  End If

  If( kcoef(67) == 1 ) Then
    if (lallo) then
      if(Associated(coef % woc_cwn)) Deallocate(coef % woc_cwn)
      Allocate(coef % woc_cwn ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % woc_cwn,coef % fmv_chn,kioproc,ITAG+65)
#endif
  End If

  If( kcoef(68) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ws_npoint)) Deallocate(coef % ws_npoint)
      Allocate(coef % ws_npoint ( coef % ws_nomega ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ws_npoint,coef % ws_nomega,kioproc,ITAG+66)
#endif
  End If

  If( kcoef(69) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ws_k_omega)) Deallocate(coef % ws_k_omega)
      Allocate(coef % ws_k_omega ( coef % ws_nomega ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ws_k_omega,coef % ws_nomega,kioproc,ITAG+67)
#endif
  End If

  If( kcoef(70) == 1 ) Then
    if (lallo) then
      if(Associated(coef % fastem_coef)) Deallocate(coef % fastem_coef)
      Allocate(coef % fastem_coef ( coef % fastem_coef_nb ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % fastem_coef,coef % fastem_coef_nb,kioproc,ITAG+68)
#endif
  End If

  If( kcoef(71) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_a0)) Deallocate(coef % ssirem_a0)
      Allocate(coef % ssirem_a0 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ssirem_a0,coef % fmv_chn,kioproc,ITAG+69)
#endif
  End If

  If( kcoef(72) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_a1)) Deallocate(coef % ssirem_a1)
      Allocate(coef % ssirem_a1 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ssirem_a1,coef % fmv_chn,kioproc,ITAG+70)
#endif
  End If

  If( kcoef(73) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_a2)) Deallocate(coef % ssirem_a2)
      Allocate(coef % ssirem_a2 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ssirem_a2,coef % fmv_chn,kioproc,ITAG+71)
#endif
  End If

  If( kcoef(74) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_xzn1)) Deallocate(coef % ssirem_xzn1)
      Allocate(coef % ssirem_xzn1 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ssirem_xzn1,coef % fmv_chn,kioproc,ITAG+72)
#endif
  End If

  If( kcoef(75) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ssirem_xzn2)) Deallocate(coef % ssirem_xzn2)
      Allocate(coef % ssirem_xzn2 ( coef % fmv_chn ))
    endif
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ssirem_xzn2,coef % fmv_chn,kioproc,ITAG+73)
#endif
  End If

  If( kcoef(76) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ref_prfl_p)) Deallocate(coef % ref_prfl_p)
      Allocate(coef % ref_prfl_p ( &
                     & coef % fmv_lvl(gas_id_mixed)))
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ref_prfl_p,kdim,kioproc,ITAG+74)
#endif
  End If

  If( kcoef(77) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ref_prfl_t)) Deallocate(coef % ref_prfl_t) 
        Allocate(coef % ref_prfl_t ( &
                     & coef % fmv_lvl(gas_id_mixed),coef % fmv_gas ) )
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)*coef % fmv_gas
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ref_prfl_t,kdim,kioproc,ITAG+75)
#endif
  End If

  If( kcoef(78) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ref_prfl_mr)) Deallocate(coef % ref_prfl_mr)
       Allocate(coef % ref_prfl_mr ( &
                     & coef % fmv_lvl(gas_id_mixed),coef % fmv_gas ) )
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)*coef % fmv_gas
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ref_prfl_mr,kdim,kioproc,ITAG+76)
#endif
  End If

  If( kcoef(79) == 1 ) Then
    if (lallo) then
      if(Associated(coef % lim_prfl_p)) Deallocate(coef % lim_prfl_p)
      Allocate(coef % lim_prfl_p ( &
                     & coef % fmv_lvl(gas_id_mixed)))
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % lim_prfl_p,kdim,kioproc,ITAG+77)
#endif
  End If

  If( kcoef(80) == 1 ) Then
    if (lallo) then
      if(Associated(coef % lim_prfl_tmax)) Deallocate(coef % lim_prfl_tmax)
      Allocate(coef % lim_prfl_tmax ( &
                     & coef % fmv_lvl(gas_id_mixed)))
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % lim_prfl_tmax,kdim,kioproc,ITAG+78)
#endif
  End If

  If( kcoef(81) == 1 ) Then
    if (lallo) then
      if(Associated(coef % lim_prfl_tmin)) Deallocate(coef % lim_prfl_tmin)
      Allocate(coef % lim_prfl_tmin ( &
                     & coef % fmv_lvl(gas_id_mixed)))
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % lim_prfl_tmin,kdim,kioproc,ITAG+79)
#endif
  End If

  If( kcoef(82) == 1 ) Then
    if (lallo) then
      if(Associated(coef % lim_prfl_gmax)) Deallocate(coef % lim_prfl_gmax)
      Allocate(coef % lim_prfl_gmax ( &
                     & coef % fmv_lvl(gas_id_mixed),coef % fmv_gas ) )
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)*coef % fmv_gas
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % lim_prfl_gmax,kdim,kioproc,ITAG+80)
#endif
  End If

  If( kcoef(83) == 1 ) Then
    if (lallo) then
      if(Associated(coef % lim_prfl_gmin)) Deallocate(coef % lim_prfl_gmin)
      Allocate(coef % lim_prfl_gmin ( &
                     & coef % fmv_lvl(gas_id_mixed),coef % fmv_gas ) )
    endif
    Kdim= coef % fmv_lvl(gas_id_mixed)*coef % fmv_gas
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % lim_prfl_gmin,kdim,kioproc,ITAG+81)
#endif
  End If

  If( kcoef(84) == 1 ) Then
    if (lallo) then
      if(Associated(coef % mixedgas)) Deallocate(coef % mixedgas)
      Allocate(coef % mixedgas ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncmixed)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncmixed
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % mixedgas,kdim,kioproc,ITAG+82)
#endif
  End If

  If( kcoef(85) == 1 ) Then
    if (lallo) then
      if(Associated(coef % watervapour)) Deallocate(coef % watervapour)
      Allocate(coef % watervapour ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncwater)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncwater
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % watervapour,kdim,kioproc,ITAG+83)
#endif
  End If

  If( kcoef(86) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ozone)) Deallocate(coef % ozone)
      Allocate(coef % ozone ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncozone)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncozone
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ozone,kdim,kioproc,ITAG+84)
#endif
  End If

  If( kcoef(87) == 1 ) Then
    if (lallo) then
      if(Associated(coef % wvcont)) Deallocate(coef % wvcont)
      Allocate(coef % wvcont ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncwvcont)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncwvcont
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % wvcont,kdim,kioproc,ITAG+85)
#endif
  End If

  If( kcoef(88) == 1 ) Then
    if (lallo) then
      if(Associated(coef % co2)) Deallocate(coef % co2)
      Allocate(coef % co2 ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncco2)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncco2
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % co2,kdim,kioproc,ITAG+86)
#endif
  End If

  If( kcoef(89) == 1 ) Then
    if (lallo) then
      if(Associated(coef % n2o)) Deallocate(coef % n2o)
      Allocate(coef % n2o ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncn2o)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncn2o
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % n2o,kdim,kioproc,ITAG+87)
#endif
  End If

  If( kcoef(90) == 1 ) Then
    if (lallo) then
      if(Associated(coef % co)) Deallocate(coef % co)
      Allocate(coef % co ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncco)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncco
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % co,kdim,kioproc,ITAG+88)
#endif
  End If

  If( kcoef(91) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ch4)) Deallocate(coef % ch4)
      Allocate(coef % ch4 ( coef % nlevels, &
                     & coef % fmv_chn, coef % ncch4)) 
    endif
    Kdim=  coef % nlevels*coef % fmv_chn*coef % ncch4
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ch4,kdim,kioproc,ITAG+89)
#endif
  End If

  If( kcoef(92) == 1 ) Then
    if (lallo) then
      if(Associated(coef % mixedgasint)) Deallocate(coef % mixedgasint)
      Allocate(coef % mixedgasint ( 2, coef % nintmixed)) 
    endif
    Kdim=  2*coef % nintmixed
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % mixedgasint,kdim,kioproc,ITAG+90)
#endif
  End If

  If( kcoef(93) == 1 ) Then
    if (lallo) then
      if(Associated(coef % watervapourint)) Deallocate(coef % watervapourint)
      Allocate(coef % watervapourint ( 2, coef % nintwater)) 
    endif
    Kdim=  2*coef % nintwater
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % watervapourint,kdim,kioproc,ITAG+91)
#endif
  End If

  If( kcoef(94) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ozoneint)) Deallocate(coef % ozoneint)
      Allocate(coef % ozoneint ( 2, coef % nintozone)) 
    endif
    Kdim=  2*coef % nintozone
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ozoneint,kdim,kioproc,ITAG+92)
#endif
  End If

  If( kcoef(95) == 1 ) Then
    if (lallo) then
      if(Associated(coef % wvcontint)) Deallocate(coef % wvcontint)
      Allocate(coef % wvcontint ( 2, coef % nintwvcont)) 
    endif
    Kdim=  2*coef % nintwvcont
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % wvcontint,kdim,kioproc,ITAG+93)
#endif
  End If

  If( kcoef(96) == 1 ) Then
    if (lallo) then
      if(Associated(coef % co2int)) Deallocate(coef % co2int)
      Allocate(coef % co2int ( 2, coef % nintco2)) 
    endif
    Kdim=  2*coef % nintco2
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % co2int,kdim,kioproc,ITAG+94)
#endif
  End If

  If( kcoef(97) == 1 ) Then
    if (lallo) then
      if(Associated(coef % n2oint)) Deallocate(coef % n2oint)
      Allocate(coef % n2oint ( 2, coef % nintn2o)) 
    endif
    Kdim=  2*coef % nintn2o
#ifdef _RTTOV_DO_DISTRIBCOEF
   Call broadcreal(coef % n2oint,kdim,kioproc,ITAG+95)
#endif
  End If

  If( kcoef(98) == 1 ) Then
    if (lallo) then
      if(Associated(coef % coint)) Deallocate(coef % coint)
      Allocate(coef % coint ( 2, coef % nintco)) 
    endif
    Kdim=  2*coef % nintco
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % coint,kdim,kioproc,ITAG+96)
#endif
  End If

  If( kcoef(99) == 1 ) Then
    if (lallo) then
      if(Associated(coef % ch4int)) Deallocate(coef % ch4int)
      Allocate(coef % ch4int ( 2, coef % nintch4)) 
    endif
    Kdim=  2*coef % nintch4
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(coef % ch4int,kdim,kioproc,ITAG+97)
#endif
  End If

  If( kcoef(100) == 1 ) Then
    if (lallo) then
      if(Associated(coef % woc_waopc_ow)) Deallocate(coef % woc_waopc_ow)
      Allocate(coef % woc_waopc_ow ( coef % fmv_chn )) 
    endif
    Allocate(zcmplx(2,coef % fmv_chn))
    If (kmyproc == kioproc) Then
      zcmplx(1,:) = Real(coef % woc_waopc_ow, Kind = JPRB)
      zcmplx(2,:) = Aimag(coef % woc_waopc_ow)
    Endif
    Kdim=  2*coef % fmv_chn
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(zcmplx,kdim,kioproc,ITAG+98)
#endif

    If (kmyproc.ne.kioproc) Then
      coef % woc_waopc_ow=Cmplx(zcmplx(1,:),zcmplx(2,:), Kind = JPRB)
    Endif
    Deallocate(zcmplx)
  End If

  If( kcoef(101) == 1 ) Then
    if (lallo) then
      if(Associated(coef % woc_waopc_fw)) Deallocate(coef % woc_waopc_fw)
      Allocate(coef % woc_waopc_fw ( coef % fmv_chn )) 
    endif
    Allocate(zcmplx(2,coef % fmv_chn))
    If (kmyproc == kioproc) Then
      zcmplx(1,:) = Real(coef % woc_waopc_fw, Kind = JPRB)
      zcmplx(2,:) = Aimag(coef % woc_waopc_fw)
    Endif
    Kdim=  2*coef % fmv_chn
#ifdef _RTTOV_DO_DISTRIBCOEF
    Call broadcreal(zcmplx,kdim,kioproc,ITAG+99)
#endif

    If (kmyproc.ne.kioproc) Then
      coef % woc_waopc_fw=Cmplx(zcmplx(1,:),zcmplx(2,:), Kind = JPRB)
    Endif
    Deallocate(zcmplx)
  End If

  ! Aerosol and cloud coefficients
  !===============================

  If(laerosol .OR. lcloud)Then
    Call rttov_distribcoef_scatt_ir(kmyproc,kioproc,coef_scatt_ir)
  End If

  If(laerosol)Then
    If (lallo) then
      If( Associated( optp % optpaer )) Deallocate( optp % optpaer )
      Allocate( optp % optpaer ( coef_scatt_ir % fmv_aer_comp ))
    End If
    Do i = 1, coef_scatt_ir % fmv_aer_comp
      If (lallo) then
        Nullify( optp % optpaer(i) % fmv_aer_rh )
        Nullify( optp % optpaer(i) % fmv_wcl_rh )
        Nullify( optp % optpaer(i) % fmv_aer_rh_val )
        Nullify( optp % optpaer(i) % fmv_wcl_rh_val )
        Nullify( optp % optpaer(i) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpaer(i) % ifmv_wcl_ph_val )
        Nullify( optp % optpaer(i) % fmv_aer_ph_val )
        Nullify( optp % optpaer(i) % fmv_aer_ph_val_cos )
        Nullify( optp % optpaer(i) % ifmv_aer_ph_val )
        Nullify( optp % optpaer(i) % fmv_wcl_ph_val )
        Nullify( optp % optpaer(i) % fmv_icl_ph_val )
        Nullify( optp % optpaer(i) % fmv_icl_ph_val_cos )
        Nullify( optp % optpaer(i) % ifmv_icl_ph_val )
        Nullify( optp % optpaer(i) % fmv_icl_dg )
        Nullify( optp % optpaer(i) % channels_solar )
        Nullify( optp % optpaer(i) % abs )
        Nullify( optp % optpaer(i) % sca )
        Nullify( optp % optpaer(i) % bpr )
        Nullify( optp % optpaer(i) % pha )
        Nullify( optp % optpaer(i) % confac )
      End If
      Call rttov_distribcoef_scatt_ir(kmyproc,kioproc,optp % optpaer(i))
    End Do
  End If

  If(lcloud)Then
    If (lallo) then
      If( Associated( optp % optpwcl )) Deallocate( optp % optpwcl )
      Allocate( optp % optpwcl ( coef_scatt_ir % fmv_wcl_comp ))

      If( Associated( optp % optpicl )) Deallocate( optp % optpicl )
      Allocate( optp % optpicl ( coef_scatt_ir % fmv_icl_ishp ))
    End If

    Do i = 1, coef_scatt_ir % fmv_wcl_comp
      If (lallo) then
        Nullify( optp % optpwcl(i) % fmv_aer_rh )
        Nullify( optp % optpwcl(i) % fmv_wcl_rh )
        Nullify( optp % optpwcl(i) % fmv_aer_rh_val )
        Nullify( optp % optpwcl(i) % fmv_wcl_rh_val )
        Nullify( optp % optpwcl(i) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpwcl(i) % ifmv_wcl_ph_val )
        Nullify( optp % optpwcl(i) % fmv_aer_ph_val )
        Nullify( optp % optpwcl(i) % fmv_aer_ph_val_cos )
        Nullify( optp % optpwcl(i) % ifmv_aer_ph_val )
        Nullify( optp % optpwcl(i) % fmv_wcl_ph_val )
        Nullify( optp % optpwcl(i) % fmv_icl_ph_val )
        Nullify( optp % optpwcl(i) % fmv_icl_ph_val_cos )
        Nullify( optp % optpwcl(i) % ifmv_icl_ph_val )
        Nullify( optp % optpwcl(i) % fmv_icl_dg )
        Nullify( optp % optpwcl(i) % channels_solar )
        Nullify( optp % optpwcl(i) % abs )
        Nullify( optp % optpwcl(i) % sca )
        Nullify( optp % optpwcl(i) % bpr )
        Nullify( optp % optpwcl(i) % pha )
        Nullify( optp % optpwcl(i) % confac )
      End If
      Call rttov_distribcoef_scatt_ir(kmyproc,kioproc,optp % optpwcl(i))
    End Do     

    Do i = 1, coef_scatt_ir % fmv_icl_ishp
      If (lallo) then
        Nullify( optp % optpicl(i) % fmv_aer_rh )
        Nullify( optp % optpicl(i) % fmv_wcl_rh )
        Nullify( optp % optpicl(i) % fmv_aer_rh_val )
        Nullify( optp % optpicl(i) % fmv_wcl_rh_val )
        Nullify( optp % optpicl(i) % fmv_wcl_ph_val_cos )
        Nullify( optp % optpicl(i) % ifmv_wcl_ph_val )
        Nullify( optp % optpicl(i) % fmv_aer_ph_val )
        Nullify( optp % optpicl(i) % fmv_aer_ph_val_cos )
        Nullify( optp % optpicl(i) % ifmv_aer_ph_val )
        Nullify( optp % optpicl(i) % fmv_wcl_ph_val )
        Nullify( optp % optpicl(i) % fmv_icl_ph_val )
        Nullify( optp % optpicl(i) % fmv_icl_ph_val_cos )
        Nullify( optp % optpicl(i) % ifmv_icl_ph_val )
        Nullify( optp % optpicl(i) % fmv_icl_dg )
        Nullify( optp % optpicl(i) % channels_solar )
        Nullify( optp % optpicl(i) % abs )
        Nullify( optp % optpicl(i) % sca )
        Nullify( optp % optpicl(i) % bpr )
        Nullify( optp % optpicl(i) % pha )
        Nullify( optp % optpicl(i) % confac )
      End If
      Call rttov_distribcoef_scatt_ir(kmyproc,kioproc,optp % optpicl(i))
    End Do     
  End If

  IF (LHOOK) CALL DR_HOOK('RTTOV_DISTRIBCOEFFS',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_distribcoeffs
