!
Subroutine rttov_findnextsection( fileunit,readstatus,section )
  ! Description:
  ! Read file (unit fileunit) until reach next section of
  ! coefficient file
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.1       23/01/2006  Increased size of section array (M .Matricardi)
  !  1.2       01/11/2007  Removed hardcoded section length (A. Geer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & nsections           ,&
        & lensection ,&
        & section_types 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),           Intent(in)  :: fileunit
  Integer,                      Intent(out) :: readstatus
  Character(len=lensection),    Intent(out) :: section

!INTF

  !local variables:
  Integer(Kind=jpim) :: i
  Logical :: sectionfound
  Character(len=80) :: line
REAL(KIND=JPRB) :: ZHOOK_HANDLE
  !- End of header --------------------------------------------------------


IF (LHOOK) CALL DR_HOOK('RTTOV_FINDNEXTSECTION',0_jpim,ZHOOK_HANDLE)
  section = ''
  sectionfound = .False.

  readfile: Do

     Read( unit=fileunit,fmt='(a)',iostat=readstatus ) line
     If ( readstatus /= 0 ) Exit

     line = Adjustl(line)
     If ( line(1:1) == '!' .Or. line == '' ) Then
        Cycle !skip blank/comment lines
     Else If ( .Not. sectionfound ) Then
        !check for a section name
        Do i = 1, nsections
           If ( section_types(i) == line ) Then
              sectionfound = .True.
              section = section_types(i)
              Exit
           End If
        End Do
     Else
        !reposition file at the start of the line and exit
        Backspace( fileunit )
        Exit readfile
     End If

  End Do readfile
IF (LHOOK) CALL DR_HOOK('RTTOV_FINDNEXTSECTION',1_jpim,ZHOOK_HANDLE)



End Subroutine rttov_findnextsection
