Subroutine rttov_make_radiance_inc( coef, radiance_inc, channels, nchannels, switchrad )

Use parkind1, Only : jpim, jprb
Use rttov_types, Only : rttov_coef, radiance_type

!
! computes a sensible radiance variation, either
! in brightness temperature or in radiance
!
Implicit None

Type(rttov_coef),    Intent(in)    :: coef
Type(radiance_type), Intent(inout) :: radiance_inc
Integer(kind=jpim),  Intent(in)    :: nchannels
Integer(Kind=jpim),  Intent(in)    :: channels(nchannels)
Logical,             Intent(in)    :: switchrad

!INTF

Integer(Kind=jpim) :: ichan

If( switchrad ) Then
  radiance_inc%bt = .1_jprb
Else
  Do ichan = 1, nchannels
  radiance_inc%total(ichan) = .1_jprb * dbdt( coef%ff_cwn( channels( ichan ) ), 280._jprb )  
  EndDo
EndIf

Contains

 Real(Kind=jprb) Function dbdt( nu, t )

  real(Kind=jprb), intent(in)  :: nu   ! wavenumber cm-1
  real(Kind=jprb), intent(in)  :: t    ! temperature K
  real (Kind=jprb), parameter ::c1 = .000011910659_jprb
  real (Kind=jprb), parameter ::c2 = 1.438833_jprb
  real(Kind=jprb) :: c1_nu3, c2_nu
  real(Kind=jprb) :: b

  c1_nu3 = c1 * nu**3
  c2_nu = c2 * nu
  b = c1_nu3 / ( exp (c2_nu / t ) -1 )
  dbdt = ( c2_nu * b * (c1_nu3 + b) ) / (c1_nu3 * t**2)

 End Function dbdt


End Subroutine
