Subroutine rttov_opdep_9_k( &
      & nchannels,      &! in
      & nprofiles,      &! in
      & nlevels,        &! in
      & channels,       &! in
      & lprofiles,      &! in
      & predictors,     &! in
      & predictors_k,   &! inout
      & aux,            &! in
      & aux_k,          &! inout
      & coef,           &! in
      & opdp_path,      &! in
      & opdp_path_k,    &! inout
      & opdp_ref)        ! in

! History:
! Version   Date     Comment
! -------   ----     -------
!  1.0    01/06/2005  Marco Matricardi (ECMWF):
!            --       New routine based on rttov_transmit_k.F90.
!                     Variable trace gases, CO2, N2O,CO and CH4
!                     have been introduced for AIRS and IASI
! 1.1      06/02/2007 Removed polaristion index R Saunders
! 1.2      07/04/2008 Marco Matricardi (ECMWF):
!                     modified the routine to be consistent
!                     with the LBLRTM coefficients
! 1.3      23/04/2008 Removed remaining use of coef%watervapourint array P. Brunel
!
! Adjoint variables
! input transmission_k% tau_surf and transmission_k% tau_layer set inside integrate_k
!
! input/output aux_k 
!
! output predictors_k initialised inside rttov_k (need input
!    intent for memory allocation in calling routine)
!

  Use rttov_const, Only: &
        & sensor_id_mw    
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_types, Only : &
        & rttov_coef     ,&
        & predictors_Type,&
        & opdp_path_Type ,&
        & profile_aux

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)    :: nchannels
  Integer(Kind=jpim),    Intent(in)    :: nprofiles
  Integer(Kind=jpim),    Intent(in)    :: nlevels
  Integer(Kind=jpim),    Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),    Intent(in)    :: lprofiles(nchannels)
  Type(predictors_Type), Intent(in)    :: predictors
  Type(predictors_Type), Intent(inout) :: predictors_k
  Type(rttov_coef)     , Intent(in)    :: coef
  Type(profile_aux)   ,  Intent(in)    :: aux( nprofiles )
  Type(profile_aux)   ,  Intent(inout) :: aux_k( nchannels )
  Type(opdp_path_Type),  Intent(inout) :: opdp_path          ! optical depths
  Type(opdp_path_Type),  Intent(inout) :: opdp_path_k            
  Real(Kind=jprb),       Intent(in)    :: opdp_ref(nlevels,nchannels)

!INTF

  !local variables:
  Real(Kind=jprb) :: opticaldepth_k(nlevels,nchannels)
  Real(Kind=jprb) :: chanx 
  Real(Kind=jprb),  Pointer :: debye_prof(:,:)
  Real(Kind=jprb),  Pointer :: debye_prof_k(:,:)
  Integer(Kind=jpim) :: lev,chan,j,ii
  Integer(Kind=jpim) :: prof

  ! cloud liquid water local variables
  Real(Kind=jprb) :: zf, zf_sq, z34_dif, z45_dif, z1_sq, z2_sq, z1_div, z2_div
  Real(Kind=jprb) :: z1_den, z2_den, zastar, z1_prod, z2_prod, z3_prod, z4_prod
  Real(Kind=jprb) :: zbstar, zbstar_sq, za2star, za2star_sq, zdiv, zgstar
  Real(Kind=jprb) :: z1f_sq_z1_sq, z2f_sq_z2_sq
  Real(Kind=jprb) :: z34_dif_k, z45_dif_k, z1_sq_k, z2_sq_k, z1_div_k, z2_div_k
  Real(Kind=jprb) :: z1_den_k, z2_den_k, zastar_k, z1_prod_k, z2_prod_k, z3_prod_k, z4_prod_k
  Real(Kind=jprb) :: zbstar_k, zbstar_sq_k, za2star_k, za2star_sq_k, zdiv_k, zgstar_k
  Real(Kind=jprb) :: z1f_sq_z1_sq_k, z2f_sq_z2_sq_k
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !---------------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_K',0_jpim,ZHOOK_HANDLE)

  !----------------------------------------
  !2.Assemble layer optical depths
  !----------------------------------------
  ! note that optical depth in the calculations is negative

  opticaldepth_k(:,:) = 0._JPRB

  Do lev = nlevels, 2, -1
    opdp_path_k % atm_layer(lev-1,:) = opdp_path_k % atm_layer(lev-1,:) &
                                    & + opdp_path_k % atm_layer(lev,:)
    opticaldepth_k(lev,:) = opticaldepth_k(lev,:)  + opdp_path_k % atm_layer(lev,:)
    opdp_path_k % atm_layer(lev,:) = 0.0_JPRB
  End Do
  opticaldepth_k(1,:) = opticaldepth_k(1,:) + opdp_path_k % atm_layer(1,:)

  Where( opdp_ref(:,:) > 0.0_JPRB )
     opticaldepth_k = 0.0_JPRB
  End Where

  !--------------------
  !1.9 add liquid water (MW only)
  !--------------------
  If ( coef % id_sensor == sensor_id_mw ) Then
     Do j = 1, nchannels
        chan = channels(j)
        prof = lprofiles(j)
        debye_prof    => aux(prof) % debye_prof
        debye_prof_k => aux_k(j) % debye_prof
        If( predictors % ncloud >= 1 ) Then
           Do lev = coef % mwcldtop, nlevels

              ! Repeat direct code
              zf           = coef % frequency_ghz(chan)
              zf_sq        = zf*zf
              z1_sq        = debye_prof(1,lev) * debye_prof(1,lev)
              z2_sq        = debye_prof(2,lev) * debye_prof(2,lev)
              z34_dif      = debye_prof(3,lev) - debye_prof(4,lev)
              z45_dif      = debye_prof(4,lev) - debye_prof(5,lev)
              z1f_sq_z1_sq = zf_sq + z1_sq
              z2f_sq_z2_sq = zf_sq + z2_sq
              z1_div       = 1.0_JPRB / z1f_sq_z1_sq
              z2_div       = 1.0_JPRB / z2f_sq_z2_sq
              z1_den       = z34_dif * z1_div
              z2_den       = z45_dif * z2_div
              zastar       = debye_prof(3,lev) - zf_sq * (z1_den + z2_den)
              z1_prod      = z34_dif * debye_prof(1,lev)
              z2_prod      = z1_prod * z1_div
              z3_prod      = z45_dif * debye_prof(2,lev)
              z4_prod      = z3_prod * z2_div
              zbstar       = -zf * (z2_prod + z4_prod)
              zbstar_sq    = zbstar * zbstar
              za2star      = zastar + 2.0_JPRB
              za2star_sq   = za2star * za2star
              zdiv         = za2star_sq + zbstar_sq
              zgstar       = -3.0_JPRB * zbstar / zdiv

              ! Now compute Adjoint code
              !opticaldepth_k(lev,j)= opticaldepth_k(lev,j)
              zgstar_k = opticaldepth_k(lev,j) *&
                     & (-1.5_JPRB * zf * predictors % clw(lev,prof))  
              predictors_k % clw(lev,j) = predictors_k % clw(lev,j) +&
                     & opticaldepth_k(lev,j) * (-1.5_JPRB * zf * zgstar)  

              zbstar_k = -3.0_JPRB * zgstar_k / zdiv
              zdiv_k   =  3.0_JPRB * zgstar_k * zbstar / (zdiv*zdiv)
              !zgstar_k =  0.

              za2star_sq_k = zdiv_k
              zbstar_sq_k  = zdiv_k
              !zdiv_k       = 0.

              za2star_k    = 2.0_JPRB * za2star * za2star_sq_k 
              !za2star_sq_k = 0.

              zastar_k     = za2star_k
              !za2star_k    = 0.

              zbstar_k     = zbstar_k + 2.0_JPRB * zbstar * zbstar_sq_k
              !zbstar_sq_k  = 0.

              z2_prod_k    = -zf * zbstar_k
              z4_prod_k    = -zf * zbstar_k
              !zbstar_k     = 0.

              z3_prod_k    = z2_div  * z4_prod_k
              z2_div_k     = z3_prod * z4_prod_k
              !z4_prod_k    = 0.

              z45_dif_k           = debye_prof(2,lev)             * z3_prod_k
              debye_prof_k(2,lev) = debye_prof_k(2,lev) + z45_dif* z3_prod_k
              !z3_prod_k           = 0.

              z1_prod_k    = z1_div  * z2_prod_k
              z1_div_k     = z1_prod * z2_prod_k
              !z2_prod_k    = 0.

              z34_dif_k           = debye_prof(1,lev)              * z1_prod_k
              debye_prof_k(1,lev) = debye_prof_k(1,lev) + z34_dif * z1_prod_k
              !z1_prod_k           = 0.

              debye_prof_k(3,lev) = debye_prof_k(3,lev) + zastar_k
              z1_den_k            =              -zf_sq  * zastar_k 
              z2_den_k            =              -zf_sq  * zastar_k 
              !zastar_k            = 0.

              z2_div_k     = z2_div_k  + z45_dif * z2_den_k
              z45_dif_k    = z45_dif_k + z2_div  * z2_den_k
              !z2_den_k     = 0.

              z1_div_k     = z1_div_k  + z34_dif * z1_den_k
              z34_dif_k    = z34_dif_k + z1_div  * z1_den_k
              !z1_den_k     = 0.

              z2f_sq_z2_sq_k = -z2_div_k / (z2f_sq_z2_sq * z2f_sq_z2_sq)
              !z2_div_k       = 0.

              z1f_sq_z1_sq_k = -z1_div_k / (z1f_sq_z1_sq * z1f_sq_z1_sq)
              !z1_div_k       = 0.

              z2_sq_k        = z2f_sq_z2_sq_k
              !z2f_sq_z2_sq_k = 0.

              z1_sq_k        = z1f_sq_z1_sq_k
              !z1f_sq_z1_sq_k = 0.

              debye_prof_k(4,lev) = debye_prof_k(4,lev) + z45_dif_k
              debye_prof_k(5,lev) = debye_prof_k(5,lev) - z45_dif_k
              !z45_dif_k           = 0.

              debye_prof_k(3,lev) = debye_prof_k(3,lev) + z34_dif_k
              debye_prof_k(4,lev) = debye_prof_k(4,lev) - z34_dif_k
              !z34_dif_k           = 0.

              debye_prof_k(2,lev) = debye_prof_k(2,lev) + z2_sq_k *&
                     & 2.0_JPRB * debye_prof(2,lev)  
              !z2_sq_k             = 0.

              debye_prof_k(1,lev) = debye_prof_k(1,lev) + z1_sq_k *&
                     & 2.0_JPRB * debye_prof(1,lev)  
              !z1_sq_k             = 0.

           End Do
        Endif
     End Do
  End If

  !-----------
  !1.8 add CH4
  !-----------

  If ( coef%nch4 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        Do lev=nlevels, 1, -1
           predictors_k % ch4(:,lev,j) =&
                     & predictors_k % ch4(:,lev,j) +&
                     & coef%ch4(lev,chan,:) * opticaldepth_k(lev,j)  
        End Do
     End Do
  End If

  !-----------
  !1.7 add CO
  !-----------

  If ( coef%nco > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)  
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

          Do lev=nlevels, 1, -1
            predictors_k % co(1:10,lev,j) =&
                       & predictors_k % co(1:10,lev,j) +&
                       & coef%co(lev,chan,1:10) * opticaldepth_k(lev,j) 
            predictors_k % co(12,lev,j) =&
                       & predictors_k % co(12,lev,j) +&
                       & coef%co(lev,chan,11) * opticaldepth_k(lev,j) 
            predictors_k % co(13,lev,j) =&
                       & predictors_k % co(13,lev,j) +&
                       & coef%co(lev,chan,12) * opticaldepth_k(lev,j) 
          End do

        else

          Do lev=nlevels, 1, -1
             predictors_k % co(1:11,lev,j) =&
                     & predictors_k % co(1:11,lev,j) +&
                     & coef%co(lev,chan,1:11) * opticaldepth_k(lev,j) 
           End Do
        End if
     End Do
  End If

  !-----------
  !1.6 add N2O
  !-----------

  If ( coef%nn2o > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)  
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

          if(chanx>=1050.and.chanx<=1350)then
          Do lev=nlevels, 1, -1

           predictors_k % ch4(10,lev,j)=                              &
           & predictors_k % ch4(10,lev,j)+coef%n2o(lev,chan,12)       * &
                                     & opticaldepth_k(lev,j)  

           predictors_k % ch4(1,lev,j)=                               &
           & predictors_k % ch4(1,lev,j)+coef%n2o(lev,chan,11)        * &
                                     & opticaldepth_k(lev,j) 
  
           predictors_k % n2o(1:10,lev,j) =&
                     & predictors_k % n2o(1:10,lev,j) +&
                     & coef%n2o(lev,chan,1:10) * opticaldepth_k(lev,j) 
        
          End Do

        else if((chanx>=1995.and.chanx<=1999.75).or.                              &
                & (chanx>=2250.and.chanx<=2295))then 

          Do lev=nlevels, 1, -1
            predictors_k % co(8,lev,j) = predictors_k % co(8,lev,j)                  - &
                & coef%n2o(lev,chan,12)*opticaldepth_k(lev,j)      * &
                & predictors % co(1,lev,prof)**3/predictors % co(8,lev,prof)**2 

            predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                 + &
               & coef%n2o(lev,chan,12)*opticaldepth_k(lev,j)      * &
               & 3*predictors % co(1,lev,prof)**2/predictors % co(8,lev,prof) 

            predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                 + &
               & coef%n2o(lev,chan,11)*opticaldepth_k(lev,j) 


            predictors_k % n2o(1:10,lev,j) =&
               & predictors_k % n2o(1:10,lev,j) +&
               & coef%n2o(lev,chan,1:10) * opticaldepth_k(lev,j) 

          End Do

        else if(chanx>=2000.and.chanx<=2249.75)then

          Do lev=nlevels, 1, -1

             predictors_k % n2o(13,lev,j) = predictors_k % n2o(13,lev,j)               + &
                             & coef%n2o(lev,chan,14)*opticaldepth_k(lev,j) 

             predictors_k % n2o(12,lev,j) = predictors_k % n2o(12,lev,j)              + &
                             & coef%n2o(lev,chan,13)*opticaldepth_k(lev,j) 

             predictors_k % co(8,lev,j) = predictors_k % co(8,lev,j)                   - &
                             & coef%n2o(lev,chan,12)*opticaldepth_k(lev,j)  * &
                             & predictors % co(1,lev,prof)**3/predictors % co(8,lev,prof)**2 

             predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                 + &
                             & coef%n2o(lev,chan,12)*opticaldepth_k(lev,j)     * &
                             & 3*predictors % co(1,lev,prof)**2 /predictors % co(8,lev,prof) 

             predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                + &
                                    & coef%n2o(lev,chan,11)*opticaldepth_k(lev,j) 

             predictors_k % n2o(10,lev,j) = predictors_k % n2o(10,lev,j)              + &
                                    & coef%n2o(lev,chan,10)*opticaldepth_k(lev,j) 

             predictors_k % n2o(11,lev,j) = predictors_k % n2o(11,lev,j)              + &
                                    & coef%n2o(lev,chan,9)*opticaldepth_k(lev,j) 

             predictors_k % n2o(1:8,lev,j) =&
                      & predictors_k % n2o(1:8,lev,j) +&
                      & coef%n2o(lev,chan,1:8) * opticaldepth_k(lev,j) 

          End Do

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=nlevels, 1, -1

             predictors_k % n2o(13,lev,j) = predictors_k % n2o(13,lev,j)               + &
                                    & coef%n2o(lev,chan,12)*opticaldepth_k(lev,j) 

             predictors_k % n2o(12,lev,j) = predictors_k % n2o(12,lev,j)               + &
                                    & coef%n2o(lev,chan,11)*opticaldepth_k(lev,j) 

             predictors_k % n2o(10,lev,j) = predictors_k % n2o(10,lev,j)               + &
                                    & coef%n2o(lev,chan,10)*opticaldepth_k(lev,j) 

             predictors_k % n2o(11,lev,j) = predictors_k % n2o(11,lev,j)              + &
                                    & coef%n2o(lev,chan,9)*opticaldepth_k(lev,j) 


             predictors_k % n2o(1:8,lev,j) =&
                      & predictors_k % n2o(1:8,lev,j) +&
                      & coef%n2o(lev,chan,1:8) * opticaldepth_k(lev,j) 

          End Do

        else

          Do lev=nlevels, 1, -1

             predictors_k % n2o(1:10,lev,j) =&
                      & predictors_k % n2o(1:10,lev,j) +&
                      & coef%n2o(lev,chan,1:10) * opticaldepth_k(lev,j) 

          End Do

        endif
     End Do
  End If

  !-----------
  !1.5 add CO2
  !-----------

  If ( coef%nco2 > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if(chanx>=2000.and.chanx<=2249.75)then

          Do lev=nlevels, 1, -1

            predictors_k % co2(15,lev,j) = predictors_k % co2(15,lev,j)                 + &
                                     & coef%co2(lev,chan,14)*opticaldepth_k(lev,j) 

            predictors_k % co2(14,lev,j) = predictors_k % co2(14,lev,j)                 + &
                                     & coef%co2(lev,chan,13)*opticaldepth_k(lev,j) 

            predictors_k % co2(13,lev,j) = predictors_k % co2(13,lev,j)                 + &
                                     & coef%co2(lev,chan,12)*opticaldepth_k(lev,j) 

            predictors_k % co2(12,lev,j) = predictors_k % co2(12,lev,j)                 + &
                                     & coef%co2(lev,chan,11)*opticaldepth_k(lev,j) 

            predictors_k % co2(11,lev,j) = predictors_k % co2(11,lev,j)                 + &
                                    & coef%co2(lev,chan,10)*opticaldepth_k(lev,j) 

            predictors_k % co2(10,lev,j) = predictors_k % co2(10,lev,j)                 + &
                                     & coef%co2(lev,chan,9)*opticaldepth_k(lev,j) 

            predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                   + &
                                      & coef%co2(lev,chan,8)*opticaldepth_k(lev,j) 

            predictors_k % co2(8,lev,j) = predictors_k % co2(8,lev,j)                   + &
                                      & coef%co2(lev,chan,7)*opticaldepth_k(lev,j) 

            predictors_k % co2(7,lev,j) = predictors_k % co2(7,lev,j)                   + &
                                      & coef%co2(lev,chan,6)*opticaldepth_k(lev,j) 


            predictors_k % co2(1:5,lev,j) =&
                     & predictors_k % co2(1:5,lev,j) +&
                     & coef%co2(lev,chan,1:5) * opticaldepth_k(lev,j) 
          enddo

        else if(chanx>=2250.and.chanx<=2295.or.chanx>=1995.and.chanx<=1999.75)then

          Do lev=nlevels, 1, -1

            predictors_k % co2(10,lev,j) = predictors_k % co2(10,lev,j)               + &
                                     & coef%co2(lev,chan,11)*opticaldepth_k(lev,j) 

            predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                    + &
                                    & coef%co2(lev,chan,10)*opticaldepth_k(lev,j) 

            predictors_k % co2(1:9,lev,j) =&
                     & predictors_k % co2(1:9,lev,j) +&
                     & coef%co2(lev,chan,1:9) * opticaldepth_k(lev,j) 
          enddo

        else if(chanx>=2295.25.and.chanx<=2380)then

          Do lev=nlevels, 1, -1 
            predictors_k % co2(1:10,lev,j) =&
                     & predictors_k % co2(1:10,lev,j) +&
                     & coef%co2(lev,chan,1:10) * opticaldepth_k(lev,j) 
          enddo

        else if(chanx>=2380.25.and.chanx<=2660)then

          Do lev=nlevels, 1, -1

            predictors_k % co2(15,lev,j) = predictors_k % co2(15,lev,j)                 + &
                                     & coef%co2(lev,chan,13)*opticaldepth_k(lev,j) 

            predictors_k % co2(14,lev,j) = predictors_k % co2(14,lev,j)                 + &
                                     & coef%co2(lev,chan,12)*opticaldepth_k(lev,j) 

            predictors_k % co2(13,lev,j) = predictors_k % co2(13,lev,j)                 + &
                                     & coef%co2(lev,chan,11)*opticaldepth_k(lev,j) 

            predictors_k % co2(12,lev,j) = predictors_k % co2(12,lev,j)                 + &
                                    & coef%co2(lev,chan,10)*opticaldepth_k(lev,j) 

            predictors_k % co2(11,lev,j) = predictors_k % co2(11,lev,j)                 + &
                                     & coef%co2(lev,chan,9)*opticaldepth_k(lev,j) 

            predictors_k % co2(10,lev,j) = predictors_k % co2(10,lev,j)                 + &
                                      & coef%co2(lev,chan,8)*opticaldepth_k(lev,j) 

            predictors_k % co2(8,lev,j) = predictors_k % co2(8,lev,j)                   + &
                                      & coef%co2(lev,chan,7)*opticaldepth_k(lev,j) 

            predictors_k % co2(7,lev,j) = predictors_k % co2(7,lev,j)                   + &
                                      & coef%co2(lev,chan,6)*opticaldepth_k(lev,j) 

            predictors_k % co2(1:5,lev,j) =&
                     & predictors_k % co2(1:5,lev,j) +&
                     & coef%co2(lev,chan,1:5) * opticaldepth_k(lev,j) 
          enddo

        else if(chanx>=690.5.and.chanx<=1100)then

          Do lev=nlevels, 1, -1
            predictors_k % co2(9,lev,j)=predictors_k % co2(9,lev,j)               +   &
                                   coef%co2(lev,chan,9)*opticaldepth_k(lev,j)

            predictors_k % co2(8,lev,j)=predictors_k % co2(8,lev,j)               +   &
                                   coef%co2(lev,chan,8)*opticaldepth_k(lev,j)

            predictors_k % co2(7,lev,j)=predictors_k % co2(7,lev,j)               +   &
                                   coef%co2(lev,chan,7)*2 * predictors % co2(7,lev,prof)  *   &
                                   opticaldepth_k(lev,j)

            predictors_k % co2(6,lev,j)=predictors_k % co2(6,lev,j)               +   &
                                   coef%co2(lev,chan,6)*2 * predictors % co2(6,lev,prof)  *   &
                                   opticaldepth_k(lev,j)

            predictors_k % co2(5,lev,j)=predictors_k % co2(5,lev,j)               +   &
                                   coef%co2(lev,chan,5)*opticaldepth_k(lev,j)

            predictors_k % co2(3,lev,j)=predictors_k % co2(3,lev,j)               +   &
                                   coef%co2(lev,chan,4)*2 *predictors % co2(3,lev,prof)   *   &
                                   opticaldepth_k(lev,j)

            predictors_k % co2(5,lev,j)=predictors_k % co2(5,lev,j)               +   &
                                   coef%co2(lev,chan,3)*predictors % co2(6,lev,prof)**2   *   &
                                   opticaldepth_k(lev,j)

            predictors_k % co2(6,lev,j)=predictors_k % co2(6,lev,j)               +   &
                                   coef%co2(lev,chan,3) *2*predictors % co2(6,lev,prof)   *   &
                                   predictors % co2(5,lev,prof)*opticaldepth_k(lev,j)

            predictors_k % co2(2,lev,j)=predictors_k % co2(2,lev,j)               +   &
                                   coef%co2(lev,chan,2)*opticaldepth_k(lev,j)

            predictors_k % co2(1,lev,j)=predictors_k % co2(1,lev,j)               +   &
                                   coef%co2(lev,chan,1)*opticaldepth_k(lev,j)

          enddo

        else

         Do lev=nlevels, 1, -1

           predictors_k % co2(1:9,lev,j) =&
                     & predictors_k % co2(1:9,lev,j) +&
                     & coef%co2(lev,chan,1:9) * opticaldepth_k(lev,j) 

          End Do

        endif
     End Do
  End If

  !------------------------------
  !1.4 add Water Vapour Continuum
  !------------------------------

  If ( coef%nwvcont > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        Do lev=nlevels, 1, -1

           predictors_k % wvcont(:,lev,j) =&
                     & predictors_k % wvcont(:,lev,j) +&
                     & coef%wvcont(lev,chan,:) * opticaldepth_k(lev,j)  
        End Do
     End Do
  End If

  !-------------
  !1.3 add ozone
  !-------------

  If ( coef%nozone > 0 ) Then
     Do j = 1, nchannels
        chan = channels(j)
        chanx= coef%ff_cwn(chan)
        prof = lprofiles(j)

        if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

        Do lev=nlevels, 1, -1
            predictors_k % ozone(15,lev,j) = predictors_k % ozone(15,lev,j)                    + &
                                     & coef%ozone(lev,chan,13)*opticaldepth_k(lev,j) 

            predictors_k % ozone(14,lev,j) = predictors_k % ozone(14,lev,j)                    + &
                                     & coef%ozone(lev,chan,12)*opticaldepth_k(lev,j) 

            predictors_k % ozone(11,lev,j) = predictors_k % ozone(11,lev,j)                    + &
                                     & coef%ozone(lev,chan,11)*opticaldepth_k(lev,j) 

            predictors_k % ozone(10,lev,j) = predictors_k % ozone(10,lev,j)                    + &
                                     & coef%ozone(lev,chan,10)*opticaldepth_k(lev,j) 

            predictors_k % ozone(9,lev,j) = predictors_k % ozone(9,lev,j)                     + &
                                     & coef%ozone(lev,chan,9)*opticaldepth_k(lev,j) 

            predictors_k % ozone(13,lev,j) = predictors_k % ozone(13,lev,j)                    + &
                                     & coef%ozone(lev,chan,8)*opticaldepth_k(lev,j) 

            predictors_k % ozone(7,lev,j) = predictors_k % ozone(7,lev,j)                     + &
                                     & coef%ozone(lev,chan,7)*opticaldepth_k(lev,j) 

            predictors_k % ozone(6,lev,j) = predictors_k % ozone(6,lev,j)                     + &
                                    & coef%ozone(lev,chan,6)*opticaldepth_k(lev,j) 

            predictors_k % ozone(5,lev,j) = predictors_k % ozone(5,lev,j)                     + &
                                     & coef%ozone(lev,chan,5)*opticaldepth_k(lev,j) 

            predictors_k % ozone(4,lev,j) = predictors_k % ozone(4,lev,j)                     + &
                                     & coef%ozone(lev,chan,4)*opticaldepth_k(lev,j) 

            predictors_k % ozone(12,lev,j) = predictors_k % ozone(12,lev,j)                    + &
                                     & coef%ozone(lev,chan,3)*opticaldepth_k(lev,j) 

            predictors_k % ozone(2,lev,j) = predictors_k % ozone(2,lev,j)                     + &
                                     & coef%ozone(lev,chan,2)*opticaldepth_k(lev,j) 

            predictors_k % ozone(1,lev,j) = predictors_k % ozone(1,lev,j)                     + &
                                     & coef%ozone(lev,chan,1)*opticaldepth_k(lev,j) 
        End Do

        else
          Do lev=nlevels, 1, -1
             predictors_k % ozone(1:11,lev,j) =&
                     & predictors_k % ozone(1:11,lev,j) +&
                     & coef%ozone(lev,chan,1:11) * opticaldepth_k(lev,j) 
          End Do
        End If
     End Do
  End If

  !--------------------
  !1.2 add water vapour
  !--------------------

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)

     if(chanx>=1095.25.and.chanx<=1750.00)then
       Do lev=nlevels, 1, -1

         predictors_k % ch4(1,lev,j)=                                   &
         & predictors_k % ch4(1,lev,j)+opticaldepth_k(lev,j)           * &
                                 & coef%watervapour(lev,chan,15)               * &
                                 & predictors % ch4(3,lev,prof) 

         predictors_k % ch4(3,lev,j)=                                   &
         & predictors_k % ch4(3,lev,j)+opticaldepth_k(lev,j)           * &
                                 & coef%watervapour(lev,chan,15)               * &
                                 & predictors % ch4(1,lev,prof) 

         predictors_k % ch4(1,lev,j)=                                   &
         & predictors_k % ch4(1,lev,j)+opticaldepth_k(lev,j)           * &
                                 & coef%watervapour(lev,chan,14) 

         predictors_k % watervapour(1:13,lev,j) =&
                  & predictors_k % watervapour(1:13,lev,j) +   &
                  & coef%watervapour(lev,chan,1:13) * opticaldepth_k(lev,j)  
       End Do

     else if(chanx>=1900.25.and.chanx<=1994.75)then

       Do lev=nlevels, 1, -1

         predictors_k % co2(1,lev,j)=                                   &
         & predictors_k % co2(1,lev,j)+opticaldepth_k(lev,j)            * &
                                 & coef%watervapour(lev,chan,14) 

         predictors_k % watervapour(1:13,lev,j) =&
                  & predictors_k % watervapour(1:13,lev,j) +   &
                  & coef%watervapour(lev,chan,1:13) * opticaldepth_k(lev,j) 
       End Do

     else if(chanx>=1995.00.and.chanx<=1999.75)then
       Do lev=nlevels, 1, -1

         predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                   + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_k(lev,j) 

         predictors_k % co2(1,lev,j) = predictors_k % co2(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_k(lev,j) 
         Do ii=1,13
          predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     else if(chanx>=2000.and.chanx<=2249.75)then

       if(coef%ss_val_chn(chan)==1)then

         Do lev=nlevels, 1, -1

           predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                   + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_k(lev,j) 

           predictors_k % co2(1,lev,j) = predictors_k % co2(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_k(lev,j) 

           predictors_k % watervapour(17,lev,j) = predictors_k % watervapour(17,lev,j)                    + &
                                      & coef%watervapour(lev,chan,13)*opticaldepth_k(lev,j) 

           predictors_k % watervapour(16,lev,j) = predictors_k % watervapour(16,lev,j)                    + &
                                     & coef%watervapour(lev,chan,12)*opticaldepth_k(lev,j) 

           predictors_k % watervapour(11,lev,j) = predictors_k % watervapour(11,lev,j)                    + &
                                      & coef%watervapour(lev,chan,11)*opticaldepth_k(lev,j) 
 
           predictors_k % watervapour(10,lev,j) = predictors_k % watervapour(10,lev,j)                    + &
                                      & coef%watervapour(lev,chan,10)*opticaldepth_k(lev,j) 

           predictors_k % watervapour(15,lev,j) = predictors_k % watervapour(15,lev,j)                   + &
                                      & coef%watervapour(lev,chan,9)*opticaldepth_k(lev,j) 

           predictors_k % watervapour(14,lev,j) = predictors_k % watervapour(14,lev,j)                   + &
                                      & coef%watervapour(lev,chan,8)*opticaldepth_k(lev,j) 
           Do ii=1,7
            predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
           End Do
         End Do

       else

         Do lev=nlevels, 1, -1

           predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_k(lev,j)

           predictors_k % co2(1,lev,j) = predictors_k % co2(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_k(lev,j)

           Do ii=1,13
            predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j)
           End Do
         End Do

       endif

     else if(chanx>=2250.and.chanx<=2295)then

       Do lev=nlevels, 1, -1

         predictors_k % co(1,lev,j) = predictors_k % co(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,15)*opticaldepth_k(lev,j) 

         predictors_k % co2(1,lev,j) = predictors_k % co2(1,lev,j)                  + &
                                      & coef%watervapour(lev,chan,14)*opticaldepth_k(lev,j) 

         Do ii=1,13
          predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     else if(chanx>=2295.25.and.chanx<=2380)then

      Do lev=nlevels, 1, -1
         Do ii=1,13
          predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     else if(chanx>=2380.25.and.chanx<=2760)then

       Do lev=nlevels, 1, -1

         predictors_k % ch4(3,lev,j) = predictors_k % ch4(3,lev,j)                   + &
                                      & coef%watervapour(lev,chan,15)               * &
                                   & predictors % ch4(1,lev,prof)**0.25*opticaldepth_k(lev,j) 

         predictors_k % ch4(1,lev,j) = predictors_k % ch4(1,lev,j)                   + &
                                      & coef%watervapour(lev,chan,15)*0.25          * &
                                      & predictors % ch4(1,lev,prof)**(-0.75)        * &
                                      & predictors % ch4(3,lev,prof)*opticaldepth_k(lev,j) 

         predictors_k % ch4(1,lev,j) = predictors_k % ch4(1,lev,j)                   + &
                                      & coef%watervapour(lev,chan,14)*1.25          * &
                                   & predictors % ch4(1,lev,prof)**0.25*opticaldepth_k(lev,j) 

         predictors_k % watervapour(19,lev,j) = predictors_k % watervapour(19,lev,j)                    + &
                                      & coef%watervapour(lev,chan,13)*opticaldepth_k(lev,j) 

         predictors_k % watervapour(16,lev,j) = predictors_k % watervapour(16,lev,j)                    + &
                                     & coef%watervapour(lev,chan,12)*opticaldepth_k(lev,j) 

         predictors_k % watervapour(11,lev,j) = predictors_k % watervapour(11,lev,j)                    + &
                                      & coef%watervapour(lev,chan,11)*opticaldepth_k(lev,j) 

         predictors_k % watervapour(18,lev,j) = predictors_k % watervapour(18,lev,j)                    + &
                                      & coef%watervapour(lev,chan,10)*opticaldepth_k(lev,j) 

         predictors_k % watervapour(15,lev,j) = predictors_k % watervapour(15,lev,j)                    + &
                                      & coef%watervapour(lev,chan,9)*opticaldepth_k(lev,j) 

         predictors_k % watervapour(14,lev,j) = predictors_k % watervapour(14,lev,j)                    + &
                                      & coef%watervapour(lev,chan,8)*opticaldepth_k(lev,j) 
         Do ii=1,7
          predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     else

       Do lev=nlevels, 1, -1
         Do ii=1,13
          predictors_k % watervapour(ii,lev,j) =&
                    & predictors_k % watervapour(ii,lev,j) +   &
                    & coef%watervapour(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     End if
  End Do

  !--------------------------
  !1.1 start with mixed gases
  !--------------------------

  Do j = 1, nchannels
     chan = channels(j)
     chanx= coef%ff_cwn(chan)
     prof = lprofiles(j)

     if((chanx>=2000.and.chanx<=2249.75).or.                                   &
         & (chanx>=2380.25.and.chanx<=2760))then 

       Do lev=nlevels, 1, -1
         predictors_k % mixedgas(10,lev,j) = predictors_k % mixedgas(10,lev,j)                     + &
                                     & coef%mixedgas(lev,chan,10)*opticaldepth_k(lev,j) 

         predictors_k % mixedgas(9,lev,j) = predictors_k % mixedgas(9,lev,j)                       + &
                                    & coef%mixedgas(lev,chan,9)*opticaldepth_k(lev,j) 
         Do ii=1,8
          predictors_k % mixedgas(ii,lev,j) =&
                  & predictors_k % mixedgas(ii,lev,j) +&
                   & coef%mixedgas(lev,chan,ii) * opticaldepth_k(lev,j) 
         End Do
       End Do

     else

       Do lev=nlevels, 1, -1

         predictors_k % mixedgas(1:8,lev,j) =&
                  & predictors_k % mixedgas(1:8,lev,j) +&
                  & coef%mixedgas(lev,chan,1:8) * opticaldepth_k(lev,j)  
       End Do
     endif
  End Do
IF (LHOOK) CALL DR_HOOK('RTTOV_OPDEP_9_K',1_jpim,ZHOOK_HANDLE)

End Subroutine rttov_opdep_9_k
