!!    Compute Optical parameters for aerosols and clouds                
Subroutine rttov_opdpscattir(       &
      & nchannels   ,                &! in
      & nprofiles   ,                &! in
      & nlevels     ,                &! in
      & channels    ,                &! in
      & lprofiles   ,                &! in
      & aux         ,                &! in
      & profiles    ,                &! in
      & coef        ,                &
      & coef_scatt_ir,               &
      & raytracing  ,                &
      & transmission_scatt_ir,       &
      & transmission_scatt_ir_stream,&
      & optp,                        &
      & ircld,                       &
      & addaerosl,                   &
      & addclouds )      ! out 

!     Description:
!     To set up profile-dependent variables for subsequent
!     rt calculations by other subroutines of RTIASI.
!
!     Method:
!     See: User's manual and scientific report for RTIASI-5
!          (Available from EUMETSAT)
!
!     Owner:
!     EUMETSAT
!
!     History:
!     Version      Date        Comment
!     1           30/7/2004    RTIASI-5. Marco Matricardi. ECMWF.
!
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".
!
!     Module used:

      Use rttov_types, Only :          &
!     Imported Type Definitions:
      & rttov_coef           ,        &
      & profile_Type         ,        &
      & raytracing_type      ,        &
      & transmission_scatt_ir_type,   &
      & rttov_coef_scatt_ir  ,        &
      & rttov_optpar_ir      ,        &
      & profile_aux          ,        &
      & ircld_type 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

      Use parkind1   , Only :   &
!     Imported parameters:
     & jpim,jprb 

      Use rttov_const, Only :   &
!     Imported parameters:
     & pi,deg2rad,jpazn,e00,t00,ti

      implicit none

!       Scalar arguments with intent in:
      logical, intent(in) :: addclouds 
      logical, intent(in) :: addaerosl
      integer(kind=jpim), intent(in) :: nprofiles
      integer(kind=jpim), intent(in) :: nchannels 
      integer(kind=jpim), intent(in) :: nlevels  
      integer(kind=jpim), intent(in) :: channels(nchannels)
      integer(kind=jpim), intent(in) :: lprofiles(nchannels)

!       Array arguments with intent in:

      type(profile_type)   , intent(in)                 :: profiles(nprofiles)
      Type(profile_aux)    , intent(inout)              :: aux( nprofiles )
      type(rttov_coef   )  , intent(in)                 :: coef
      type(transmission_scatt_ir_type) , intent(inout)  :: transmission_scatt_ir
      type(transmission_scatt_ir_type) , intent(inout)  :: transmission_scatt_ir_stream
      type(rttov_coef_scatt_ir) , intent(in)            :: coef_scatt_ir
      Type(rttov_optpar_ir), intent(in)                 :: optp
      Type(ircld_type)                                  :: ircld
      Type(raytracing_type),intent(inout)               :: raytracing

!INTF

!     End of subroutine arguments

!       Local scalars:
      Integer(Kind=jpim) :: j,jl,ich,ich1,i,ipf,ish,istream,k,kk,ityp,iconf,iae,ikk
      Integer(Kind=jpim) :: iang,iend,icount,icl
      Real(Kind=jprb)    :: opd,opdsun,opd1
      Real(Kind=jprb)    :: absch
      Real(Kind=jprb)    :: scach
      Real(Kind=jprb)    :: bparh
      Real(Kind=jprb)    :: afac
      Real(Kind=jprb)    :: sfac
      Real(Kind=jprb)    :: gfac
      Real(Kind=jprb)    :: sum
      Real(Kind=jprb)    :: sum1
      Real(Kind=jprb)    :: sum2
      Real(Kind=jprb)    :: sum3
      Real(Kind=jprb)    :: scattangle
      Real(Kind=jprb)    :: deltapaer
      Real(Kind=jprb)    :: deltapice
      Real(Kind=jprb)    :: deltapwcl
      Real(Kind=jprb)    :: deltaaer
      Real(Kind=jprb)    :: deltaice
      Real(Kind=jprb)    :: deltawcl
      Real(Kind=jprb)    :: deltadg
      Real(Kind=jprb)    :: delth
      Real(Kind=jprb)    :: zdelth
      Real(Kind=jprb)    :: frach
      Real(Kind=jprb)    :: phasint
      Real(Kind=jprb)    :: musat
      Real(Kind=jprb)    :: musun
      Real(Kind=jprb)    :: phup
      Real(Kind=jprb)    :: phdo

!       Local arrays:
      Real(Kind=jprb)    :: opdpaerl   (nchannels,nlevels)
      Real(Kind=jprb)    :: opdpcldl   (nchannels,nlevels)
      Real(Kind=jprb)    :: opdpcldlsun(nchannels,nlevels)
      Real(Kind=jprb)    :: opdpaerlsun(nchannels,nlevels)
      Real(Kind=jprb)    :: pfac       (coef_scatt_ir%fmv_aer_ph)
      Real(Kind=jprb)    :: phash      (coef_scatt_ir%fmv_aer_ph)
      Real(Kind=jprb)    :: phasice    (coef_scatt_ir%fmv_icl_ph)
      Real(Kind=jprb)    :: deltap     (coef_scatt_ir%fmv_icl_ph)
      REAL(KIND=JPRB) :: ZPI,ztmpx,ztmpy
      REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----End of header-------------------------------------------------------------


      IF (LHOOK) CALL DR_HOOK('RTTOV_OPDPSCATTIR',0_jpim,ZHOOK_HANDLE)
      bparh        =0._jprb
      opdpaerl     =0._jprb
      opdpaerlsun  =0._jprb
      opdpcldl     =0._jprb
      opdpcldlsun  =0._jprb
      ZPI=deg2rad/(2._jprb*PI)
      iang=360_jpim/(jpazn+1)
      iend=iang*jpazn

!-----------------------------------------------------------------------------------------
!         1.   CALCULATE OPTICAL DEPTHS OF AEROSOLS 
!-----------------------------------------------------------------------------------------

      if(addaerosl)then
!       opdpaerl(:,:)     =0._jprb
!       opdpaerlsun(:,:)  =0._jprb
        do j=1,nprofiles
          do jl=1,nlevels
            icount=0
            do icl=1,coef_scatt_ir % fmv_aer_comp
              if(profiles(j)%aerosols(icl,jl)/=0._jprb)then
                icount=icount+1
                aux(j)%iaertyp(icount,jl)=icl
              endif
            enddo
            aux(j)%iaernum(jl)=icount


!-----Compute relative humidity-----------------------------------------------------------

            if(jl==1)then
              ircld%tave     (jl,j) = profiles(j)%t(jl)
              ircld%wmixave  (jl,j) = profiles(j)%q(jl)
              ircld%xpresave (jl,j) = coef%ref_prfl_p(jl)
            else
              ircld%tave     (jl,j) = (profiles(j)%t(jl-1)+profiles(j)%t(jl))/2._jprb
              ircld%wmixave  (jl,j) = (profiles(j)%q(jl-1)+profiles(j)%q(jl))/2._jprb
              ircld%xpresave (jl,j) = (coef%ref_prfl_p(jl-1)+coef%ref_prfl_p(jl))/2._jprb
            endif
            ircld%esw        (jl,j) = e00*exp(17.502_jprb*(ircld%tave(JL,j)-T00)     / &
                                       & (ircld%tave(JL,j)-32.19_jprb)) 
            ircld%esi        (jl,j) = E00*exp(22.587_jprb*(ircld%tave(JL,j)-T00)     / &
                                       & (ircld%tave(JL,j)+0.7_jprb  )) 

            if(ircld%tave(JL,j)>t00)then
              ircld%ppv(jl,j) = ircld%esw(jl,j)                            ! Water phase
            elseif(ircld%tave(jl,j)>ti.and.ircld%tave(jl,j)<=t00)then
              ircld%ppv(jl,j) = ircld%esi(jl,j)+(ircld%esw(jl,j)-ircld%esi(jl,j))  * &
                          & ((ircld%tave(jl,j)-ti)/(t00-ti))**2        ! Mixed phase 
            elseif(ircld%tave(jl,j)<=ti)then
              ircld%ppv(jl,j) = ircld%esi(jl,j)                            ! Ice phase
            endif

            ircld%ppv(jl,j)    = ircld%ppv(jl,j)/100._jprb


            aux(j)%relhum(jl) = 100._jprb*ircld%wmixave(jl,j)*1.e-6_jprb*0.622_jprb * &
                                & ircld%xpresave(jl,j)                                  / &
                                & (ircld%ppv(jl,j)*(0.622_jprb+ircld%wmixave(jl,j)*1.e-6_jprb*0.622_jprb)) 

           aux(j)%relhumref(jl)=aux(j)%relhum (jl)

           if(aux(j)%relhum (jl)>99._jprb)then
             aux(j)%relhum(jl)=99._jprb
           endif

 !----------------------------------------------------------------------------------------
          enddo
        enddo
      endif

      if(addaerosl.or.addclouds)then
        transmission_scatt_ir%OPDPAAER  (:,:)  =0._jprb
        transmission_scatt_ir%OPDPSAER  (:,:)  =0._jprb
        transmission_scatt_ir%OPDPA     (:,:)  =0._jprb
        transmission_scatt_ir%OPDPS     (:,:)  =0._jprb
        transmission_scatt_ir%GPARAER   (:,:)  =0._jprb
        transmission_scatt_ir%GPAR      (:,:)  =0._jprb
        transmission_scatt_ir%OPDPAERLA (:,:)  =0._jprb
        transmission_scatt_ir%GPARAERA  (:,:)  =0._jprb
        transmission_scatt_ir%azphaerup (:,:)  =0._jprb
        transmission_scatt_ir%azphaerupa(:,:)  =0._jprb
        transmission_scatt_ir%azphaerdo (:,:)  =0._jprb
        transmission_scatt_ir%azphaerdoa(:,:)  =0._jprb
      endif

      if(addaerosl)then
        frequency:do j=1,nchannels
        ich = channels(J)
        ipf = lprofiles(J)

        sum=0._jprb
        sum1=0._jprb
        sum2=0._jprb
        sum3=0._jprb

          levels:do jl=1,nlevels

            numaer:do I=1,aux(ipf)%iaernum(jl)
             iae=aux(ipf)%iaertyp(I,JL)
              if(coef_scatt_ir%fmv_aer_rh(iae)/=1)then
!
!---------------Interpolate scattering parameters to actual value of relative-------------
!               humidity.

                do k=1,coef_scatt_ir % fmv_aer_rh(iae)-1
                  IF(aux(ipf)%relhum(jl)>=optp%optpaer(iae)% fmv_aer_rh_val(k).and.      &
                     & aux(ipf)%relhum(jl)<=optp%optpaer(iae)% fmv_aer_rh_val(k+1))then 

                    delth=(optp%optpaer(iae)% fmv_aer_rh_val(K+1)                      - &
                          & optp%optpaer(iae)% fmv_aer_rh_val(K)) 
                    zdelth=1.0_JPRB/delth
                    frach=(aux(ipf)%relhum(jl)-optp%optpaer(iae)% fmv_aer_rh_val(K))
                    afac=(optp%optpaer(iae)%abs(ich,k+1)-optp%optpaer(iae)%abs(ich,k))*zdelth
                    sfac=(optp%optpaer(iae)%sca(ich,k+1)-optp%optpaer(iae)%sca(ich,k))*zdelth
                    gfac=(optp%optpaer(iae)%bpr(ich,k+1)-optp%optpaer(iae)%bpr(ich,k))*zdelth
                    absch=optp%optpaer(iae)%abs(ich,k)+afac*frach 
                    scach=optp%optpaer(iae)%sca(ich,k)+sfac*frach 
                    bparh=optp%optpaer(iae)%bpr(ich,k)+gfac*frach 

                      if(profiles(ipf)%addsolar)then
                        if(coef%ss_val_chn(ich)==1)then
                          ich1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1
                          PFAC(1:coef_scatt_ir%fmv_aer_ph)=                              &
                            & (optp%optpaer(iae)%PHA(ICH1,K+1,1:coef_scatt_ir%fmv_aer_ph)- &
                            & optp%optpaer(iae)%PHA(ICH1,K,1:coef_scatt_ir%fmv_aer_ph))*zdelth 
                          PHASH(1:coef_scatt_ir%fmv_aer_ph)=                             &
                            & optp%optpaer(iae)%PHA(ICH1,K,1:coef_scatt_ir%fmv_aer_ph)   + &
                            & PFAC(1:coef_scatt_ir%fmv_aer_ph)*FRACH 
                        endif
                      endif
                    exit 
                  endif
                enddo 

              else
                absch=optp%optpaer(iae)%abs(ich,1)
                scach=optp%optpaer(iae)%sca(ich,1)
!!                EXTCH=EXTC(ICH,IAE,1)
                bparh=optp%optpaer(iae)%bpr(ich,1)

                  if(profiles(ipf)%addsolar)then
                    if(coef%ss_val_chn(ich)==1)then
                      ich1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1
                      PHASH(1:coef_scatt_ir%fmv_aer_ph)=                                 &
                            & optp%optpaer(iae)%pha(ich1,1,1:coef_scatt_ir%fmv_aer_ph) 
                    endif
                  endif
              endif

!-------------Compute optical parameters considering the contribution of------------------
!             all the aerosol components present in the layer

              if(i==1)then
                transmission_scatt_ir%OPDPAAER(J,JL)=profiles(IPF)%aerosols(IAE,JL)    * &
                                 & ABSCH*raytracing%LTICK(nlevels+1-JL,IPF) 
                transmission_scatt_ir%OPDPSAER(J,JL)=profiles(IPF)%aerosols(IAE,JL)    * &
                                 & SCACH*raytracing%LTICK(nlevels+1-JL,IPF) 
!!                OPDPEAER(J,JL)=OPDPEAER(J,JL)+PROFAER(IAE,JL,IPF)                    * &
!!                                 EXTCH*LTICK(90-JL,IPF)
                transmission_scatt_ir%GPARAERA(J,JL)=profiles(IPF)%aerosols(IAE,JL)    * &
                                 & SCACH*BPARH*raytracing%LTICK(nlevels+1-JL,IPF) 
              else
                transmission_scatt_ir%OPDPAAER(J,JL)=                                    &
                    & transmission_scatt_ir%OPDPAAER(J,JL)+profiles(IPF)%aerosols(IAE,JL)* &
                         & ABSCH*raytracing%LTICK(nlevels+1-JL,IPF) 
                transmission_scatt_ir%OPDPSAER(J,JL)=                                    &
                    & transmission_scatt_ir%OPDPSAER(J,JL)+profiles(IPF)%aerosols(IAE,JL)* &
                         & SCACH*raytracing%LTICK(nlevels+1-JL,IPF) 
!                OPDPEAER(J,JL)=OPDPEAER(J,JL)+PROFAER(IAE,JL,IPF)                     * &
!                                 EXTCH*LTICK(90-JL,IPF)
                transmission_scatt_ir%GPARAERA(J,JL)=                                    &
                    & transmission_scatt_ir%GPARAERA(J,JL)+profiles(IPF)%aerosols(IAE,JL)* &
                         & SCACH*BPARH*raytracing%LTICK(nlevels+1-JL,IPF) 
              endif

              if(j==1421)then
              sum=sum+profiles(IPF)%aerosols(IAE,JL)    * &
                               & ABSCH*raytracing%LTICK(nlevels+1-JL,IPF) 
              sum1=sum1+profiles(IPF)%aerosols(IAE,JL)              * &
                               & SCACH*raytracing%LTICK(nlevels+1-JL,IPF) 
              endif

!-------------If solar radiation is present,compute the azimuthally averaged--------------
!             value of the phase function for the given value of the viewing--------------
!             angle and solar zenith angle.

              IF(profiles(ipf)%addsolar)THEN              
                if(coef%ss_val_chn(ich)==1)then
                  ICH1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1

!-------------Average phase function for the upward scattered solar beam------------------

                  PHASINT= 0._jprb
!                 COSAN_aer(1:coef_scatt_ir%fmv_aer_ph)  =                               &
!                    & COS(coef_scatt_ir%fmv_aer_ph_val(1:coef_scatt_ir%fmv_aer_ph)*deg2rad) 
                  MUSAT  =  1._jprb/raytracing%PATHSAT(JL,ipf)
                  MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)

                  ztmpx=SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))
                  ztmpy=1.E0_jprb/(coef_scatt_ir%fmv_aer_ph_val_min*deg2rad)
                  LOOP1: DO k=0,IEND,IANG
                    SCATTANGLE=MUSAT*MUSUN+ztmpx*COS(K*deg2rad)
!                     DO kk=1,coef_scatt_ir%fmv_aer_ph-1
!                       IF(SCATTANGLE>=coef_scatt_ir%fmv_aer_ph_val_cos(KK+1) &
!                         &  .AND.SCATTANGLE<=coef_scatt_ir%fmv_aer_ph_val_cos(KK))THEN
                          ikk = max(1_jpim,int(acos(SCATTANGLE)*ztmpy,jpim))
                          KK=coef_scatt_ir%ifmv_aer_ph_val(ikk)-1._jprb
                          DELTAPAER=(PHASH(KK+1)-PHASH(KK))
                          DELTAAER=(coef_scatt_ir%fmv_aer_ph_val_cos(KK)- &
                                  & coef_scatt_ir%fmv_aer_ph_val_cos(KK+1))
                          PHUP    =PHASH(KK)+DELTAPAER*(coef_scatt_ir%fmv_aer_ph_val_cos(KK)- &
                                  & SCATTANGLE)/DELTAAER
!                         EXIT 
!                       ENDIF
!                     ENDDO
                      PHASINT=PHASINT+PHUP*IANG*ZPI
                  ENDDO LOOP1

                  transmission_scatt_ir%PHASINTUPREF(J,JL,I)=PHASINT

                  transmission_scatt_ir%AZPHAERUP(J,JL)=                                 &
                   & transmission_scatt_ir%AZPHAERUP(J,JL)+profiles(ipf)%aerosols(IAE,JL)* &
                   & PHASINT*SCACH*raytracing%LTICK(nlevels+1-JL,ipf) 

!-------------Average phase function for the downward scattered solar beam----------------

                  PHASINT= 0._jprb
!                 COSAN_aer(1:coef_scatt_ir%fmv_aer_ph)  =                               &
!                    & COS(coef_scatt_ir%fmv_aer_ph_val(1:coef_scatt_ir%fmv_aer_ph)*deg2rad)  
                  MUSAT  = -1._jprb/raytracing%PATHSAT(JL,ipf)
                  MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)
                  ztmpx=SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))
                  ztmpy=1.E0_jprb/(coef_scatt_ir%fmv_aer_ph_val_min*deg2rad)
                  LOOP2: DO k=0,IEND,IANG
                    SCATTANGLE=MUSAT*MUSUN+ztmpx*COS(K*deg2rad)
!                     DO kk=1,coef_scatt_ir%fmv_aer_ph-1
!                       IF(SCATTANGLE>=coef_scatt_ir%fmv_aer_ph_val_cos(KK+1) &
!                         & .AND.SCATTANGLE<=coef_scatt_ir%fmv_aer_ph_val_cos(KK))THEN
                          ikk=max(1_jpim,int(acos(SCATTANGLE)*ztmpy,jpim))
                          KK=coef_scatt_ir%ifmv_aer_ph_val(ikk)-1._jprb
                          DELTAPAER=(PHASH(KK+1)-PHASH(KK))
                          DELTAAER=(coef_scatt_ir%fmv_aer_ph_val_cos(KK)- &
                                  & coef_scatt_ir%fmv_aer_ph_val_cos(KK+1))
                          PHDO    =PHASH(KK)+DELTAPAER*(coef_scatt_ir%fmv_aer_ph_val_cos(KK)- &
                                  & SCATTANGLE)/DELTAAER
!                         EXIT 
!                       ENDIF
!                     ENDDO
                      PHASINT=PHASINT+PHDO*IANG*ZPI
                  ENDDO LOOP2

                  transmission_scatt_ir%PHASINTDOREF(J,JL,I)=PHASINT

                  transmission_scatt_ir%AZPHAERDO(J,JL)=                                 &
                   & transmission_scatt_ir%AZPHAERDO(J,JL)+profiles(ipf)%aerosols(IAE,JL)* &
                   & PHASINT*SCACH*raytracing%LTICK(nlevels+1-JL,ipf) 

                ENDIF
              ENDIF
            enddo numaer
          enddo levels

!---------Compute final values for optical parameters------------------------------------- 

          DO JL=1,nlevels

            IF(transmission_scatt_ir%OPDPSAER(J,JL)/=0._JPRB)THEN

              transmission_scatt_ir%GPARAER (J,JL)=transmission_scatt_ir%GPARAERA(J,JL)/ &
                                                   & transmission_scatt_ir%OPDPSAER(J,JL) 

              IF(profiles(IPF)%addsolar)THEN
                if(coef%ss_val_chn(ich)==1)then
                  transmission_scatt_ir%AZPHAERUPA(J,JL)=                                &
                                                & transmission_scatt_ir%AZPHAERUP(J,JL)  / &
                                                & transmission_scatt_ir%OPDPSAER(J,JL) 
                  transmission_scatt_ir%AZPHAERDOA(J,JL)=                                &
                                                & transmission_scatt_ir%AZPHAERDO(J,JL)  / &
                                                & transmission_scatt_ir%OPDPSAER(J,JL) 
                ENDIF
              ENDIF
            ENDIF

            transmission_scatt_ir%OPDPAERLA(J,JL)=transmission_scatt_ir%OPDPAAER(J,JL) + &
               & transmission_scatt_ir%OPDPSAER(J,JL)*transmission_scatt_ir%GPARAER (J,JL) 

            OPDPAERL(J,JL)   =transmission_scatt_ir%OPDPAERLA(J,JL)                    * &
                                    & RAYTRACING%PATHSAT(JL,IPF)*coef%ff_gam(ICH) 
            IF(profiles(ipf)%addsolar)THEN
              if(coef%ss_val_chn(ich)==1)then
                OPDPAERLSUN(J,JL)=transmission_scatt_ir%OPDPAERLA(J,JL)                * &
                                  & (RAYTRACING%PATHSUN(JL,IPF)                         + &
                                  & RAYTRACING%PATHSAT(JL,IPF))*coef%ff_gam(ICH) 
              ENDIF
            ENDIF
          ENDDO

!        if(j==1421)print*,sum+sum1,'EXTOPD_AER'

        enddo frequency

      ENDIF

!-----------------------------------------------------------------------------------------
!         2.   CALCULATE OPTICAL DEPTHS OF CLOUDS
!-----------------------------------------------------------------------------------------

        IF(addclouds )THEN
!          opdpcldl(:,:)      =0._jprb
!          opdpcldlsun(:,:)   =0._jprb

          do j=1,nchannels
            ich=channels(j)
            ipf=lprofiles(j)
            ish=profiles(ipf)%ish
         
            sum=0._jprb
            sum1=0._jprb
            sum2=0._jprb
            sum3=0._jprb

!            IOFF=NUMAE

            DO JL=1,nlevels
              transmission_scatt_ir%azphup(j,jl)  =0._jprb
              transmission_scatt_ir%azphdo(j,jl)  =0._jprb
              transmission_scatt_ir%OPDPA(j,jl)   =0._jprb
              transmission_scatt_ir%OPDPS(j,jl)   =0._jprb
              transmission_scatt_ir%GPAR(j,jl)    =0._jprb
!              transmission_scatt_ir%OPDPAAER(j,jl)=0._jprb
!              transmission_scatt_ir%OPDPSAER(j,jl)=0._jprb
!              transmission_scatt_ir%GPARAER(j,jl) =0._jprb

              IF(ircld%CLDTYP(JL,ipf)/=0)THEN
                ITYP = ircld%CLDTYP (JL,ipf)
                ICONF= ITYP
!                ITMP = ITEMP  (JL,IPF)
!                ICONF= CLDTYP (JL,IPF)+ITEMP(JL,IPF)-1

!---------------If solar radiation is present,compute the azimuthally averaged------------
!               value of the phase function for the given value of the viewing------------
!               angle and solar zenith angle.
!-----------------------------------------------------------------------------------------

                if(profiles(ipf)%addsolar)then
                  if(coef%ss_val_chn(ich)==1)then

!-----------------If ice clouds are present the phase function for for the current value 
!                 of the effective generalized diameter is obtained by linear 
!                 interpolation and then the azimuthally averaged value is computed 
!----------------------------------------------------------------------------------------


                    if(ITYP==6_jpim)then

                      ich1=ich-coef_scatt_ir % fmv_icl_pha_ioff+1
                      do k=1,coef_scatt_ir % fmv_icl_comp
                        if(aux(ipf)%dg(jl)>=coef_scatt_ir %fmv_icl_dg(k,ish).and.       &
                          & aux(ipf)%dg(jl)<=coef_scatt_ir %fmv_icl_dg(k+1,ish))then 

                          deltap=(optp%optpicl(ish)%PHA(ICH1,k+1,:)                      - &
                                                 & optp%optpicl(ish)%PHA(ICH1,k,:)) 

                          DELTADG=(coef_scatt_ir %fmv_icl_dg(k+1,ish)                       - &
                                      & coef_scatt_ir %fmv_icl_dg(k,ish)) 

                          phasice(:)=optp%optpicl(ish)%PHA(ICH1,k,:)+deltap(:)           * &
                             & (aux(ipf)%dg(jl)-coef_scatt_ir %fmv_icl_dg(k,ish))/DELTADG 
                         exit
                        endif
                      enddo

!---------------------Average phase function for the upward scattered solar beam----------

                      PHASINT= 0._jprb
!                     COSAN_icl(1:coef_scatt_ir%fmv_icl_ph)  =                           &
!                     & COS(coef_scatt_ir%fmv_icl_ph_val(1:coef_scatt_ir%fmv_icl_ph)*deg2rad) 
                      MUSAT  =  1._jprb/raytracing%PATHSAT(JL,ipf)
                      MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)

                      LOOP3: DO K=0,IEND,IANG
                        SCATTANGLE=MUSAT*MUSUN+SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))      * &
                             & COS(K*deg2rad) 
!                       DO KK=1,coef_scatt_ir%fmv_icl_ph-1
!                         IF(SCATTANGLE>=coef_scatt_ir%fmv_icl_ph_val_cos(KK+1) &
!                           &  .AND.SCATTANGLE<=coef_scatt_ir%fmv_icl_ph_val_cos(KK))THEN
                          ikk=max(1_jpim,int(acos(SCATTANGLE)/(coef_scatt_ir%fmv_icl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_icl_ph_val(ikk)-1._jprb
                            DELTAPICE=(phasice(KK+1) - PHAsice(KK)) 
                            DELTAICE=(coef_scatt_ir%fmv_icl_ph_val_cos(KK)- &
                               &  coef_scatt_ir%fmv_icl_ph_val_cos(KK+1))
                            PHASINT=PHAsice(KK)+DELTAPICE                              * &
                                             & (coef_scatt_ir%fmv_icl_ph_val_cos(KK)-SCATTANGLE)/DELTAICE 
!                           EXIT
!                         ENDIF
!                       ENDDO
                        transmission_scatt_ir%AZPHUP(J,JL)=                              &
                          & transmission_scatt_ir%AZPHUP(J,JL)+PHASINT*IANG*ZPI
                      ENDDO LOOP3

!---------------------Average phase function for the downward scattered solar beam--------

                      PHASINT= 0._jprb
!                     COSAN_icl(1:coef_scatt_ir%fmv_icl_ph)  =                           &
!                     & COS(coef_scatt_ir%fmv_icl_ph_val(1:coef_scatt_ir%fmv_icl_ph)*deg2rad) 
                      MUSAT  = -1._jprb/raytracing%PATHSAT(JL,ipf)
                      MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)

                      LOOP4: DO K=0,IEND,IANG
                        SCATTANGLE=MUSAT*MUSUN+SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))        * &
                              & COS(K*deg2rad) 
!                     DO KK=1,coef_scatt_ir%fmv_icl_ph-1
!                       IF(SCATTANGLE>=coef_scatt_ir%fmv_icl_ph_val_cos(KK+1) &
!                         &  .AND.SCATTANGLE<=coef_scatt_ir%fmv_icl_ph_val_cos(KK))THEN
                          ikk=max(1_jpim,int(acos(SCATTANGLE)/(coef_scatt_ir%fmv_icl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_icl_ph_val(ikk)-1._jprb
                          DELTAPICE=(PHAsice(KK+1)-PHAsice(KK))
                          DELTAICE=(coef_scatt_ir%fmv_icl_ph_val_cos(KK)- &
                               &  coef_scatt_ir%fmv_icl_ph_val_cos(KK+1))
                          PHASINT=PHAsice(KK)+DELTAPICE                                   * &
                                             & (coef_scatt_ir%fmv_icl_ph_val_cos(KK)-SCATTANGLE)/DELTAICE 
!                         EXIT
!                       ENDIF
!                     ENDDO
                       transmission_scatt_ir%AZPHDO(J,JL)=                               &
                          & transmission_scatt_ir%AZPHDO(J,JL)+PHASINT*IANG*ZPI
                      ENDDO LOOP4

                    else 

!-----------------------------------------------------------------------------------------
!                     Water clouds                                                        
!-----------------------------------------------------------------------------------------

!---------------------Average phase function for the upward scattered solar beam----------

                      ich1=ich-coef_scatt_ir % fmv_wcl_pha_ioff+1
                      PHASINT= 0._jprb
!                     COSAN_wcl(1:coef_scatt_ir%fmv_wcl_ph)  =                           &
!                     & COS(coef_scatt_ir%fmv_wcl_ph_val(1:coef_scatt_ir%fmv_wcl_ph)*deg2rad) 
                      MUSAT  =  1._jprb/raytracing%PATHSAT(JL,ipf)
                      MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)

                      LOOP5: DO K=0,IEND,IANG
                        SCATTANGLE=MUSAT*MUSUN+SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))       * &
                             & COS(K*deg2rad) 
!                     DO KK=1,coef_scatt_ir%fmv_wcl_ph-1
!                       IF(SCATTANGLE>=coef_scatt_ir%fmv_wcl_ph_val_cos(KK+1) &
!                         & .AND.SCATTANGLE<=coef_scatt_ir%fmv_wcl_ph_val_cos(KK))THEN
                          ikk=max(1_jpim,int(acos(SCATTANGLE)/(coef_scatt_ir%fmv_wcl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_wcl_ph_val(ikk)-1._jprb
                          DELTAPWCL=(optp%optpwcl(ityp)%PHA(ICH1,1,KK+1)                  - &
                                                & optp%optpwcl(ityp)%PHA(ICH1,1,KK)) 
                          DELTAWCL=(coef_scatt_ir%fmv_wcl_ph_val_cos(KK)-coef_scatt_ir%fmv_wcl_ph_val_cos(KK+1))
                          PHASINT=optp%optpwcl(ityp)%PHA(ICH1,1,KK)+DELTAPWCL             * &
                                             & (coef_scatt_ir%fmv_wcl_ph_val_cos(KK)-SCATTANGLE)/DELTAWCL 
!                       EXIT 
!                       ENDIF
!                     ENDDO
                      transmission_scatt_ir%AZPHUP(J,JL)=                                &
                          & transmission_scatt_ir%AZPHUP(J,JL)+PHASINT*IANG*ZPI
                      ENDDO LOOP5


!---------------------Average phase function for the downward scattered solar beam--------

                      PHASINT= 0._jprb
!                     COSAN_wcl(1:coef_scatt_ir%fmv_wcl_ph)  =                           &
!                     & COS(coef_scatt_ir%fmv_wcl_ph_val(1:coef_scatt_ir%fmv_wcl_ph)*deg2rad) 
                      MUSAT  = -1._jprb/raytracing%PATHSAT(JL,ipf)
                      MUSUN  = -1._jprb/raytracing%PATHSUN(JL,ipf)

                      LOOP6: DO K=0,IEND,IANG
                      SCATTANGLE=MUSAT*MUSUN+SQRT((1._jprb-MUSAT**2)*(1._jprb-MUSUN**2))         * &
                             & COS(K*deg2rad) 
!                     DO KK=1,coef_scatt_ir%fmv_wcl_ph-1
!                       IF(SCATTANGLE>=coef_scatt_ir%fmv_wcl_ph_val_cos(KK+1) &
!                         &   .AND.SCATTANGLE<=coef_scatt_ir%fmv_wcl_ph_val_cos(KK))THEN
                          ikk = max(1_jpim,int(acos(SCATTANGLE)/(coef_scatt_ir%fmv_wcl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_wcl_ph_val(ikk)-1._jprb
                          DELTAPWCL=(optp%optpwcl(ityp)%PHA(ICH1,1,KK+1)                  - &
                                                & optp%optpwcl(ityp)%PHA(ICH1,1,KK))   
                          DELTAWCL=(coef_scatt_ir%fmv_wcl_ph_val_cos(KK)-coef_scatt_ir%fmv_wcl_ph_val_cos(KK+1))
                          PHASINT=optp%optpwcl(ityp)%PHA(ICH1,1,KK)+DELTAPWCL             * &
                                             & (coef_scatt_ir%fmv_wcl_ph_val_cos(KK)-SCATTANGLE)/DELTAWCL 
!                         EXIT 
!                       ENDIF
!                     ENDDO
                      transmission_scatt_ir%AZPHDO(J,JL)=                                &
                         & transmission_scatt_ir%AZPHDO(J,JL)+PHASINT*IANG*ZPI

                      ENDDO LOOP6
                    endif
                  ENDIF
                ENDIF

!---------------Compute cloud  optical parameters ----------------------------------------

                if(ityp<=5_jpim)then

!-----------------------------------------------------------------------------------------
!                 For water clouds use stored optical parameters
!-----------------------------------------------------------------------------------------

                  transmission_scatt_ir%OPDPA(J,JL)=profiles(IPF)%cloud(Ityp,JL)       * &
                       & coef_scatt_ir%CONFAC(ICONF)                                     * &
                       & optp%optpwcl(ityp)%abs(ich,1)*raytracing%LTICK(nlevels+1-JL,IPF) 

                  transmission_scatt_ir%OPDPS(J,JL)=profiles(IPF)%cloud(Ityp,JL)       * &
                       & coef_scatt_ir%CONFAC(ICONF)                                     * &
                       & optp%optpwcl(ityp)%sca(ich,1)*raytracing%LTICK(nlevels+1-JL,IPF) 

                  sum2=sum2+profiles(IPF)%cloud(Ityp,JL)*coef_scatt_ir%CONFAC(ICONF)               * &
                              & optp%optpwcl(ityp)%abs(ich,1)*raytracing%LTICK(nlevels+1-JL,IPF) 
                  sum3=sum3+profiles(IPF)%cloud(Ityp,JL)*coef_scatt_ir%CONFAC(ICONF)               * &
                              & optp%optpwcl(ityp)%sca(ich,1)*raytracing%LTICK(nlevels+1-JL,IPF) 

!                OPDPE(J,JL)=PROFCLD(ITYP,JL,IPF)*coef_scatt_ir%CONFAC(ICONF)          * &
!                              EXTC(ICH,IOFF+ITYP,ITMP)*LTICK(90-JL,IPF)
                  transmission_scatt_ir%GPAR (J,JL)=optp%optpwcl(ityp)%bpr(ich,1)

                else
!-----------------------------------------------------------------------------------------
!                 For ice clouds optical parameters are computed using regression 
!                 coefficients
!-----------------------------------------------------------------------------------------
                  transmission_scatt_ir%OPDPA(J,JL)=profiles(IPF)%cloud(Ityp,JL)       * &
                     & (optp%optpicl(ish)%abs(ich,1)                                     + &
                      & optp%optpicl(ish)%abs(ich,2)*aux(ipf)%dg(jl)                     + &
                      & optp%optpicl(ish)%abs(ich,3)/aux(ipf)%dg(jl)                     + &
                      & optp%optpicl(ish)%abs(ich,4)/aux(ipf)%dg(jl)**2)                 * &
                     & raytracing%LTICK(nlevels+1-JL,IPF) 

                  transmission_scatt_ir%OPDPS(J,JL)=profiles(IPF)%cloud(Ityp,JL)       * &
                     & (optp%optpicl(ish)%sca(ich,1)      +                               &
                      & optp%optpicl(ish)%sca(ich,2)*aux(ipf)%dg(jl)+                     &
                      & optp%optpicl(ish)%sca(ich,3)/aux(ipf)%dg(jl)+                     &
                      & optp%optpicl(ish)%sca(ich,4)/aux(ipf)%dg(jl)**2)                 * &
                     & raytracing%LTICK(nlevels+1-JL,IPF) 

                  transmission_scatt_ir%GPAR (J,JL)=                                     &
                     & (optp%optpicl(ish)%bpr(ich,1)                                          + &
                      & optp%optpicl(ish)%bpr(ich,2)*aux(ipf)%dg(jl)                     + &
                      & optp%optpicl(ish)%bpr(ich,3)*aux(ipf)%dg(jl)**2                  + &
                      & optp%optpicl(ish)%bpr(ich,4)*aux(ipf)%dg(jl)**3) 

                endif

                transmission_scatt_ir%OPDPCLDLA(J,JL)  =                                 &
                    & transmission_scatt_ir%OPDPA(J,JL)                                  + &
                    & transmission_scatt_ir%GPAR (J,JL)*transmission_scatt_ir%OPDPS(J,JL) 

                OPDPCLDL(J,JL)   = transmission_scatt_ir%OPDPCLDLA(J,JL)               * &
                & raytracing%PATHSAT(JL,ipf)*coef%ff_gam(ICH) 

                IF(profiles(IPF)%addsolar)THEN
                  if(coef%ss_val_chn(ich)==1)then
                    OPDPCLDLSUN(J,JL)= transmission_scatt_ir%OPDPCLDLA(J,JL)           * &
                                   & (raytracing%PATHSAT(JL,ipf)                        + &
                                   & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH) 
                  ENDIF
                ENDIF
              ENDIF
            ENDDO

!      if(j==1421)print*,sum2+sum3,'EXTOPD_CLD'
          enddo 

        endif
     
!-----Compute optical parameters for each stream------------------------------------------

!      if(addaerosl.or.addclouds )then
      DO J=1,nchannels
        ICH = channels(J)
!opd=0
        IPF=lprofiles(J)
          DO ISTREAM=0,ircld%NSTREAM(IPF)
            
            IF(ISTREAM==0)THEN
!              OPD=0._jprb
!              OPDSUN=0._jprb
!              OPD1=0._jprb
              IF(addaerosl)THEN
                OPD=0._jprb
                OPDSUN=0._jprb
                OPD1=0._jprb
                DO JL=1,nlevels
                IF(profiles(ipf)%addsolar)THEN              
                  if(coef%ss_val_chn(ich)==1)then
                      transmission_scatt_ir_stream%AZPHACUP(ISTREAM,J,JL)         = &
                           & transmission_scatt_ir%AZPHAERUPA(J,JL) 

                      transmission_scatt_ir_stream%AZPHACDO(ISTREAM,J,JL)         = &
                           & transmission_scatt_ir%AZPHAERDOA(J,JL) 

                      transmission_scatt_ir_stream%BCKSP   (ISTREAM,J,JL)         = &
                           & transmission_scatt_ir%GPARAER   (J,JL)  

                      transmission_scatt_ir_stream%OPDPABS (ISTREAM,J,JL)         = &
                           & transmission_scatt_ir%OPDPAAER(J,JL)                        * &
                           & (raytracing%PATHSAT(JL,ipf)+raytracing%PATHSUN(JL,ipf))   * &
                           & coef%ff_gam(ICH) 

                      transmission_scatt_ir_stream%OPDPSCA (ISTREAM,J,JL)         = &
                           & transmission_scatt_ir%OPDPSAER(J,JL)                        * &
                           & (raytracing%PATHSAT(JL,ipf)+raytracing%PATHSUN(JL,ipf))   * &
                           & coef%ff_gam(ICH) 

                      transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL)       = &
                           & OPDPAERLSUN(J,JL) 

                      OPDSUN  =                                                          &
                         & OPDSUN+transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL) 

                      transmission_scatt_ir_stream%OPDPACSUN(ISTREAM,J,JL)  =OPDSUN
                  endif
                endif
                    transmission_scatt_ir_stream%OPDPACL   (ISTREAM,J,JL)=          &
                         & OPDPAERL(J,JL) 

                    OPD  = OPD   +transmission_scatt_ir_stream%OPDPACL(ISTREAM,J,JL)

                    transmission_scatt_ir_stream%OPDPAC(ISTREAM,J,JL)    = OPD
!                    OPD1                    = OPD1+OPDPEAER(J,nlevels-JL+1)
                ENDDO
              ELSE IF((.not.addaerosl).and.(addclouds ))then
                transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,:) =0._jprb
                transmission_scatt_ir_stream%OPDPACSUN (ISTREAM,J,:) =0._jprb
                transmission_scatt_ir_stream%OPDPACL   (ISTREAM,J,:) =0._jprb
                transmission_scatt_ir_stream%OPDPAC    (ISTREAM,J,:) =0._jprb
                transmission_scatt_ir_stream%AZPHACUP  (ISTREAM,J,:) =0._jprb
                transmission_scatt_ir_stream%AZPHACDO  (ISTREAM,J,:) =0._jprb
              ENDIF
            ELSE
              if(addclouds )then
              OPD=0._jprb
              OPDSUN=0._jprb
              DO JL=1,nlevels
                IF(profiles(IPF)%addsolar)THEN
                  if(coef%ss_val_chn(ich)==1)then

                    IF((ircld%ICLDARR(ISTREAM,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+&
                                           & transmission_scatt_ir%OPDPSAER(J,JL))/=0)THEN 

                      transmission_scatt_ir_stream%AZPHACUP(ISTREAM,J,JL)=          &
                        & transmission_scatt_ir%AZPHAERUPA(J,JL)                         * &
                        & transmission_scatt_ir%OPDPSAER(J,JL)                           / &
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL))                          + &
                        & transmission_scatt_ir%AZPHUP(J,JL)                             * &
                        & transmission_scatt_ir%OPDPS(J,JL)*ircld%ICLDARR(ISTREAM,JL,ipf)/&
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL)) 

                      transmission_scatt_ir_stream%AZPHACDO(ISTREAM,J,JL)=          &
                        & transmission_scatt_ir%AZPHAERDOA(J,JL)                         * &
                        & transmission_scatt_ir%OPDPSAER(J,JL)                           / &
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL))                          + &
                        & transmission_scatt_ir%AZPHDO(J,JL)                             * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              * &
                        & ircld%ICLDARR(ISTREAM,JL,ipf)                                 / &
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL)) 

                      transmission_scatt_ir_stream%BCKSP   (ISTREAM,J,JL)=          &
                        & transmission_scatt_ir%GPARAER   (J,JL)                         * &
                        & transmission_scatt_ir%OPDPSAER(J,JL)                           / &
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL))                          + &
                        & transmission_scatt_ir%GPAR  (J,JL)                             * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              * &
                        & ircld%ICLDARR(ISTREAM,JL,ipf)                                 / &
                        & (ircld%ICLDARR(ISTREAM,JL,ipf)                                * &
                        & transmission_scatt_ir%OPDPS(J,JL)                              + &
                        & transmission_scatt_ir%OPDPSAER(J,JL)) 

                      transmission_scatt_ir_stream%OPDPABS (ISTREAM,J,JL)=          &
                        & (transmission_scatt_ir%OPDPAAER(J,JL)                          + &
                        & ircld%ICLDARR(ISTREAM,JL,ipf)                                 * &
                        & transmission_scatt_ir%OPDPA(J,JL))                             * &
                        & (raytracing%PATHSAT(JL,ipf)+raytracing%PATHSUN(JL,ipf))      * &
                        & coef%ff_gam(ICH)  

                      transmission_scatt_ir_stream%OPDPSCA (ISTREAM,J,JL)=          &
                        & (transmission_scatt_ir%OPDPSAER(J,JL)                          + &
                        & ircld%ICLDARR(ISTREAM,JL,ipf)                                 * &
                        & transmission_scatt_ir%OPDPS(J,JL))                             * &
                        & (raytracing%PATHSAT(JL,ipf)+raytracing%PATHSUN(JL,ipf))      * &
                        & coef%ff_gam(ICH) 
                    ELSE
                      transmission_scatt_ir_stream%AZPHACUP(ISTREAM,J,JL)=0._jprb
                      transmission_scatt_ir_stream%AZPHACDO(ISTREAM,J,JL)=0._jprb
                      transmission_scatt_ir_stream%BCKSP   (ISTREAM,J,JL)=0._jprb
                      transmission_scatt_ir_stream%OPDPABS (ISTREAM,J,JL)=0._jprb
                      transmission_scatt_ir_stream%OPDPSCA (ISTREAM,J,JL)=0._jprb

                    ENDIF

                    transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL) =             &
                    & OPDPCLDLSUN(J,JL)*ircld%ICLDARR(ISTREAM,JL,ipf)+OPDPAERLSUN(J,JL) 

                    OPDSUN= OPDSUN+transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL)

                    transmission_scatt_ir_stream%OPDPACSUN(ISTREAM,J,JL)  = OPDSUN

                  ENDIF
                ENDIF

                transmission_scatt_ir_stream%OPDPACL(ISTREAM,J,JL)    =             &
                   & OPDPCLDL(J,JL)*ircld%ICLDARR(ISTREAM,JL,ipf)+OPDPAERL(J,JL) 

!                transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL) =             &
!                   OPDPCLDLSUN(J,JL)*ircld%ICLDARR(ISTREAM,JL,ipf)+OPDPAERLSUN(J,JL)

                OPD   = OPD+transmission_scatt_ir_stream%OPDPACL(ISTREAM,J,JL)

!                OPDSUN= OPDSUN+transmission_scatt_ir_stream%OPDPACLSUN(ISTREAM,J,JL)

                transmission_scatt_ir_stream%OPDPAC(ISTREAM,J,JL)     = OPD

!                transmission_scatt_ir_stream%OPDPACSUN(ISTREAM,J,JL)  = OPDSUN

              ENDDO
            ENDIF
          ENDIF
!if(j==1421)print*,opd,istream
        ENDDO
      ENDDO
!      endif

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDPSCATTIR',1_jpim,ZHOOK_HANDLE)
      END SUBROUTINE rttov_opdpscattir 
