!     Compute Optical parameters for aerosols and clouds
Subroutine rttov_opdpscattir_tl( &
      & nchannels   , &! in
      & nprofiles   , &! in
      & nlevels     , &! in
      & channels    , &! in
      & lprofiles   , &! in
      & aux         , &! in
      & aux_tl      , &! inout
      & profiles    , &! in
      & profiles_tl , &
      & coef        , &
      & coef_scatt_ir,&
      & raytracing  , &
      & raytracing_tl,&
      & transmission_scatt_ir,&
      & transmission_scatt_ir_tl,&
      & transmission_scatt_ir_stream,&
      & transmission_scatt_ir_stream_tl,&
      & optp        , &
      & ircld,        &
      & ircld_tl,     &
      & addaerosl,    &
      & addclouds)      ! out 

!     Description:
!     To set up profile-dependent variables for subsequent
!     rt calculations by other subroutines of RTIASI.
!
!     Method:
!     See: User's manual and scientific report for RTIASI-5
!          (Available from EUMETSAT)
!
!     Owner:
!     EUMETSAT
!
!     History:
!     Version      Date        Comment
!     1           30/07/2004   Marco Matricardi. ECMWF.
!     1.1         05/02/2007   Removed polarisation R Saunders
!
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".
!
!     Module used:

      Use rttov_types, Only :   &
!     Imported Type Definitions:
      & rttov_coef           ,   &
      & profile_Type         ,   &
      & raytracing_type      ,   &
      & transmission_scatt_ir_type       ,   &
      & rttov_coef_scatt_ir  ,   &
      & rttov_optpar_ir      ,   &
      & profile_aux          ,   &
      & ircld_type 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

      Use parkind1   , Only :   &
!     Imported parameters:
     & jpim,jprb 

      Use rttov_const, Only :   &
!     Imported parameters:
     & pi,deg2rad,jpazn,e00,t00,ti 

      implicit none

!       Scalar arguments with intent in:
      logical, intent(in) :: addclouds
      logical, intent(in) :: addaerosl
      integer(kind=jpim), intent(in) :: nprofiles
      integer(kind=jpim), intent(in) :: nchannels
      integer(kind=jpim), intent(in) :: nlevels
      integer(kind=jpim), intent(in) :: channels(nchannels)
      integer(kind=jpim), intent(in) :: lprofiles(nchannels)

!       Array arguments with intent in:

      type(profile_type)   , intent(in)  :: profiles(nprofiles)
      type(profile_type)   , intent(in)  :: profiles_tl(nprofiles)
      Type(profile_aux)    , intent(in)  :: aux( nprofiles )
      Type(profile_aux)    , intent(inout)  :: aux_tl( nprofiles )
      type(rttov_coef   )  , intent(in)  :: coef
      type(transmission_scatt_ir_type) , intent(in)  :: transmission_scatt_ir
      type(transmission_scatt_ir_type) , intent(inout)  :: transmission_scatt_ir_tl      
      type(transmission_scatt_ir_type) , intent(in)  :: transmission_scatt_ir_stream
      type(transmission_scatt_ir_type) , intent(inout)  :: transmission_scatt_ir_stream_tl
      type(rttov_coef_scatt_ir) , intent(in)  :: coef_scatt_ir
      Type(rttov_optpar_ir), intent(in)  :: optp
      Type(ircld_type)                   :: ircld
      Type(ircld_type)                   :: ircld_tl
      
      Type(raytracing_type),intent(in)   :: raytracing
      Type(raytracing_type),intent(inout):: raytracing_tl

!INTF



!     End of subroutine arguments
!       Local scalars:
      Integer(Kind=jpim) :: j,jl,ich,ich1,i,ipf,ish,k,kk,ityp,iconf,iae,ikk
      Integer(Kind=jpim) :: iang,iend,ist
      Real(Kind=jprb)    :: opd,opdsun,opd1
      Real(Kind=jprb)    :: absch
      Real(Kind=jprb)    :: scach
      Real(Kind=jprb)    :: bparh
      Real(Kind=jprb)    :: afac
      Real(Kind=jprb)    :: sfac
      Real(Kind=jprb)    :: gfac
      Real(Kind=jprb)    :: scattangle
      Real(Kind=jprb)    :: deltapice
      Real(Kind=jprb)    :: deltapice_d
      Real(Kind=jprb)    :: deltapaer
      Real(Kind=jprb)    :: deltapaer_d
      Real(Kind=jprb)    :: deltapwcl_d
      Real(Kind=jprb)    :: deltaaer
      Real(Kind=jprb)    :: deltaice
      Real(Kind=jprb)    :: deltawcl
      Real(Kind=jprb)    :: delth
      Real(Kind=jprb)    :: frach
      Real(Kind=jprb)    :: phasint
      Real(Kind=jprb)    :: musat
      Real(Kind=jprb)    :: musun
      Real(Kind=jprb)    :: phup
      Real(Kind=jprb)    :: phdo

      REAL(Kind=jprb)    :: phup_d
      REAL(Kind=jprb)    :: phdo_d
      REAL(Kind=jprb)    :: scattangle_d
      REAL(Kind=jprb)    :: musat_d
      REAL(Kind=jprb)    :: musun_d
      REAL(Kind=jprb)    :: frach_d
      REAL(Kind=jprb)    :: absch_d
      REAL(Kind=jprb)    :: scach_d
      REAL(Kind=jprb)    :: bparh_d

!       Local arrays:
!      REAL OPDPE      (KNCHPF,JPLAY)
!      REAL OPDPEAER   (KNCHPF,JPLAY)
      Real(Kind=jprb)    :: opdpaerl   (nchannels,nlevels)
      Real(Kind=jprb)    :: opdpcldl   (nchannels,nlevels)
      Real(Kind=jprb)    :: opdpcldlsun(nchannels,nlevels)
      Real(Kind=jprb)    :: opdpaerlsun(nchannels,nlevels)
      Real(Kind=jprb)    :: pfac       (coef_scatt_ir%fmv_aer_ph)
      Real(Kind=jprb)    :: phash      (coef_scatt_ir%fmv_aer_ph)
      Real(Kind=jprb)    :: phasice    (coef_scatt_ir%fmv_icl_ph)
      Real(Kind=jprb)    :: phasice_d  (coef_scatt_ir%fmv_icl_ph)
      Real(Kind=jprb)    :: deltap     (coef_scatt_ir%fmv_icl_ph)
      Real(Kind=jprb)    :: phash_d (coef_scatt_ir%fmv_aer_ph)
      REAL(KIND=JPRB) :: ZPI
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!-----End of header-------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDPSCATTIR_TL',0_jpim,ZHOOK_HANDLE)
      OPDPCLDL   =0._jprb
      OPDPAERL   =0._jprb
      OPDPCLDLSUN=0._jprb
      OPDPAERLSUN=0._jprb
      bparh      =0._jprb
       ZPI=deg2rad/(2._jprb*PI)
      IANG=360_jpim/(JPAZN+1)
      IEND=IANG*JPAZN

!-----------------------------------------------------------------------------------------
!         1.   CALCULATE OPTICAL DEPTHS OF AEROSOLS 
!-----------------------------------------------------------------------------------------
!-----Compute relative humidity-----------------------------------------------------------

      if(addaerosl)then
        do j=1,nprofiles
          do jl=1,nlevels

            if(jl==1)then
              ircld_tl%tave     (jl,j) = profiles_tl(j)%t(jl)
              ircld_tl%wmixave  (jl,j) = profiles_tl(j)%q(jl)
            else
              ircld_tl%tave     (jl,j) = (profiles_tl(j)%t(jl-1)+profiles_tl(j)%t(jl))/ &
                                          & 2._jprb 
              ircld_tl%wmixave  (jl,j) = (profiles_tl(j)%q(jl-1)+profiles_tl(j)%q(jl))/ &
                                          & 2._jprb 
            endif

!-----------Compute vater vapour partial pressure-----------------------------------------

            ircld_tl%ESW(JL,j)=ircld_tl%TAVE(JL,j)*ircld%ESW(JL,j)*17.502_jprb      * &
                                & (T00-32.19_jprb)/(ircld%TAVE(JL,j)-32.19_jprb)**2 
            ircld_tl%ESI(JL,j)=ircld_tl%TAVE(JL,j)*ircld%ESI(JL,j)*22.587_jprb      * &
                                & (T00+0.7_jprb) /(ircld%TAVE(JL,j)+0.7_jprb  )**2 

            IF(ircld%TAVE(JL,j)>T00)THEN
              ircld_tl%PPV(JL,j) = ircld_tl%ESW(JL,j)                                 ! Water phase
            ELSEIF(ircld%TAVE(JL,j)>TI.AND.ircld%TAVE(JL,j)<=T00)then
              ircld_tl%PPV(JL,j) = ircld_tl%ESI(JL,j)                                + &
                          & ircld_tl%ESW(JL,j)*((ircld%TAVE(JL,j)-TI)/(T00-TI))**2     - &
                          & ircld_tl%ESI(JL,j)*((ircld%TAVE(JL,j)-TI)/(T00-TI))**2     + &
                          & ircld_tl%TAVE(JL,j)*(ircld%ESW(JL,j)-ircld%ESI(JL,j))*2   * &
                                   & ((ircld%TAVE(JL,j)-TI)/(T00-TI)**2)! Mixed phase 
            ELSEIF(ircld%TAVE(JL,j)<=TI)THEN
              ircld_tl%PPV(JL,j) = ircld_tl%ESI(JL,j)                                  ! Ice phase
            ENDIF

            ircld_tl%PPV(JL,j)=ircld_tl%PPV(JL,j)/100._jprb
!-----------------------------------------------------------------------------------------

            aux_tl(j)%RELHUM(JL)=ircld_tl%wmixave(JL,j)*100._jprb*1e-6_jprb*0.622_jprb * &
                & ircld%xpresave(jl,j)                                     / &
                & (ircld%PPV(JL,j)*(0.622_jprb+ircld%wmixave(JL,j)*1e-6_jprb*0.622_jprb))- &
                & ircld_tl%wmixave(JL,j)*100._jprb*(1e-6_jprb*0.622_jprb)**2*ircld%xpresave(jl,j)* &
                & ircld%wmixave(JL,j)*ircld%PPV(JL,j)                         / &
                & (ircld%PPV(JL,j)*(0.622_jprb+ircld%wmixave(JL,j)*1.e-6_jprb*0.622_jprb))**2 - &
                & ircld_tl%PPV(JL,j)*100._jprb*ircld%wmixave(JL,j)*1.e-6_jprb*0.622_jprb  * &
                & ircld%xpresave(jl,j)*(0.622_jprb+ircld%wmixave(JL,j)*1.e-6_jprb*0.622_jprb) / &
                & (ircld%PPV(JL,j)*(0.622_jprb+ircld%wmixave(JL,j)*1e-6_jprb*0.622_jprb))**2 

             IF(aux(j)%RELHUMREF (JL)>99._jprb)THEN
               aux_tl(j)%RELHUM(JL)=0._jprb
             ENDIF

          enddo
        enddo
      endif

!-----------------------------------------------------------------------------------------

      if(addaerosl.or.addclouds)then
        transmission_scatt_ir_tl%OPDPAAER  (:,:)  =0._jprb
        transmission_scatt_ir_tl%OPDPSAER  (:,:)  =0._jprb
        transmission_scatt_ir_tl%OPDPA     (:,:)  =0._jprb
        transmission_scatt_ir_tl%OPDPS     (:,:)  =0._jprb
        transmission_scatt_ir_tl%GPARAER   (:,:)  =0._jprb
        transmission_scatt_ir_tl%GPAR      (:,:)  =0._jprb
        transmission_scatt_ir_tl%OPDPAERLA (:,:)  =0._jprb
        transmission_scatt_ir_tl%GPARAERA  (:,:)  =0._jprb
        transmission_scatt_ir_tl%azphaerup (:,:)  =0._jprb
        transmission_scatt_ir_tl%azphaerupa(:,:)  =0._jprb
        transmission_scatt_ir_tl%azphaerdo (:,:)  =0._jprb
        transmission_scatt_ir_tl%azphaerdoa(:,:)  =0._jprb
      endif

      if(addaerosl)then
        do j=1,nchannels
        ich = channels(J)
        ipf = lprofiles(J)

          do jl=1,nlevels

            do i=1,aux(ipf)%iaernum(jl)
              iae=aux(ipf)%iaertyp(I,JL)

              if(coef_scatt_ir%fmv_aer_rh(iae)/=1)then

!---------------Interpolate scattering parameters to actual value of relative-------------
!               humidity.

                do k=1,coef_scatt_ir % fmv_aer_rh(iae)-1
                  if(aux(ipf)%relhum(jl)>=optp%optpaer(iae)% fmv_aer_rh_val(k).and.      &
                     & aux(ipf)%relhum(jl)<=optp%optpaer(iae)% fmv_aer_rh_val(k+1))then 

                    delth=(optp%optpaer(iae)% fmv_aer_rh_val(K+1)                      - &
                           & optp%optpaer(iae)% fmv_aer_rh_val(K)) 

                    frach=aux_tl(ipf)%relhum(JL)

                    afac=(optp%optpaer(iae)%abs(ich,k+1)                               - &
                          & optp%optpaer(iae)%abs(ich,k) )/delth 
                    sfac=(optp%optpaer(iae)%sca(ich,k+1)                               - &
                          & optp%optpaer(iae)%sca(ich,k) )/delth 
!!                    efac=(EXTC(ICH,IAE,K+1)-EXTC(ICH,IAE,K))/delth
                    gfac=(optp%optpaer(iae)%bpr(ich,k+1)                               - &
                          & optp%optpaer(iae)%bpr(ich,k) )/delth 

                    frach_d=(aux(ipf)%relhum(jl)-optp%optpaer(iae)% fmv_aer_rh_val(k))
                    absch_d=optp%optpaer(iae)%abs(ich,k)+afac*frach_d
                    scach_d=optp%optpaer(iae)%sca(ich,k)+sfac*frach_d
!!                    EXTCH_D=EXTC(ICH,IAE,K)+EFAC*FRACH_D
                    bparh_d=optp%optpaer(iae)%bpr(ich,k)+gfac*frach_d

                    absch=afac*frach 
                    scach=sfac*frach 
                    bparh=gfac*frach 

                      if(profiles(ipf)%addsolar)then
                       if(coef%ss_val_chn(ich)==1)then
                          ich1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1
                          pfac(1:coef_scatt_ir%fmv_aer_ph)=                              &
                            & (optp%optpaer(iae)%pha(ich1,K+1,1:coef_scatt_ir%fmv_aer_ph)- &
                             & optp%optpaer(iae)%pha(ICH1,K,1:coef_scatt_ir%fmv_aer_ph)) / &
                             & delth 

                          phash_d(1:coef_scatt_ir%fmv_aer_ph)=                           &
                             & optp%optpaer(iae)%pha(ICH1,K,1:coef_scatt_ir%fmv_aer_ph)  + &
                                             & pfac(1:coef_scatt_ir%fmv_aer_ph)*frach_d 
                          phash(1:coef_scatt_ir%fmv_aer_ph)  =                           &
                                             & pfac(1:coef_scatt_ir%fmv_aer_ph)*frach 
                        endif
                      endif
                    exit 
                  endif
                enddo 
              else
                absch_d=optp%optpaer(iae)%abs(ich,1)
                scach_d=optp%optpaer(iae)%sca(ich,1)
                bparh_d=optp%optpaer(iae)%bpr(ich,1)

                absch=0._jprb
                scach=0._jprb
                bparh=0._jprb

                  if(profiles(ipf)%addsolar)then
                    if(coef%ss_val_chn(ich)==1)then
                      ich1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1

                      phash_d(1:coef_scatt_ir%fmv_aer_ph)=                               &
                              & optp%optpaer(iae)%pha(ich1,1,1:coef_scatt_ir%fmv_aer_ph) 

                      phash(1:coef_scatt_ir%fmv_aer_ph)=0._jprb
                    endif
                  endif
              endif

!-------------Compute optical parameters considering the contribution of------------------
!             all the aerosol components present in the layer

              if(i==1)then

                transmission_scatt_ir_tl%opdpaaer(j,jl)=                                 &
                               & profiles_tl(ipf)%aerosols(iae,JL)*absch_d               * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & absch*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*absch_d          * &
                               & profiles(ipf)%aerosols(iae,jl) 

                transmission_scatt_ir_tl%opdpsaer(j,jl)=                                 &
                               & profiles_tl(ipf)%aerosols(iae,JL)*scach_d               * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & scach*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*scach_d          * &
                               & profiles(ipf)%aerosols(iae,jl)  

                transmission_scatt_ir_tl%gparaera(j,jl)=                                 &
                               & profiles_tl(ipf)%aerosols(iae,JL)*scach_d*bparh_d       * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & scach*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf) *bparh_d            +  &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*scach_d          * &
                               & profiles(ipf)%aerosols(iae,jl)  *bparh_d                +  &
                               & bparh*profiles(ipf)%aerosols(iae,jl)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)* scach_d  
              else
                transmission_scatt_ir_tl%opdpaaer(j,jl)=                                 &
                               & transmission_scatt_ir_tl%opdpaaer(j,jl)                 + &
                               & profiles_tl(ipf)%aerosols(iae,JL)*absch_d               * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & absch*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*absch_d          * &
                               & profiles(ipf)%aerosols(iae,jl) 

                transmission_scatt_ir_tl%opdpsaer(j,jl)=                                 &
                               & transmission_scatt_ir_tl%opdpsaer(j,jl)                 + &
                               & profiles_tl(ipf)%aerosols(iae,JL)*scach_d               * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & scach*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*scach_d          * &
                               & profiles(ipf)%aerosols(iae,jl) 

                transmission_scatt_ir_tl%gparaera(j,jl)=                                 &
                               & transmission_scatt_ir_tl%gparaera(j,jl)                 + &
                               & profiles_tl(ipf)%aerosols(iae,JL)*scach_d*bparh_d       * &
                               & raytracing%ltick(nlevels+1-jl,ipf)                     + &
                               & scach*profiles(ipf)%aerosols(iae,JL)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf) *bparh_d            + &
                               & raytracing_tl%ltick(nlevels+1-jl,ipf)*scach_d          * &
                               & profiles(ipf)%aerosols(iae,jl)  *bparh_d                + &
                               & bparh*profiles(ipf)%aerosols(iae,jl)                    * &
                               & raytracing%ltick(nlevels+1-jl,ipf)* scach_d 
              endif

!-------------If solar radiation is present,compute the azimuthally averaged--------------
!             value of the phase function for the given value of the viewing--------------
!             angle and solar zenith angle.

              if(profiles(ipf)%addsolar)then
                if(coef%ss_val_chn(ich)==1)then
                  ich1=ich-coef_scatt_ir % fmv_aer_pha_ioff+1

!-------------Average phase function for the upward scattered solar beam------------------

                  phasint         =  0._jprb
!                 cosan_aer(1:coef_scatt_ir%fmv_aer_ph)  =                               &
!                   & cos(coef_scatt_ir%fmv_aer_ph_val(1:coef_scatt_ir%fmv_aer_ph)*deg2rad) 

                  musat_d         =  1._jprb/raytracing%pathsat(jl,ipf)
                  musun_d         = -1._jprb/raytracing%pathsun(jl,ipf)

!                  musat           = -raytracing_tl%pathsat(jl,ipf)                    / &
!                                   raytracing%pathsat(jl,ipf)*raytracing%pathsat(jl,ipf)
!                  musun           =  raytracing_tl%pathsun(jl,ipf)                    / &
!                                   raytracing%pathsun(jl,ipf)*raytracing%pathsun(jl,ipf)
                  musat           = -raytracing_tl%pathsat(jl,ipf)                    / &
                                   & raytracing%pathsat(jl,ipf)**2 
                  musun           =  raytracing_tl%pathsun(jl,ipf)                    / &
                                   & raytracing%pathsun(jl,ipf)**2 

                  loop1: do K=0,iend,iang

                    scattangle_d=musat_d*musun_d+sqrt((1-musat_d**2)*(1-musun_d**2)) *cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d+musun*musat_d                          
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1._jprb-musun_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)                 - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif

!                     do kk=1,coef_scatt_ir%fmv_aer_ph-1
!                       if(scattangle_d>=coef_scatt_ir%fmv_aer_ph_val_cos(kk+1).and.   &
!                                                  & scattangle_d<=coef_scatt_ir%fmv_aer_ph_val_cos(kk))then 
                          ikk = max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_aer_ph_val_min*deg2rad),jpim) )
                          KK=coef_scatt_ir%ifmv_aer_ph_val(ikk)-1._jprb

                          deltapaer  =(phash(kk+1)-phash(kk))
                          deltapaer_d=(phash_d(kk+1)-phash_d(kk))

                          deltaaer  =(coef_scatt_ir%fmv_aer_ph_val_cos(kk)-coef_scatt_ir%fmv_aer_ph_val_cos(kk+1))

                          phup_d  =phash_d(kk)+deltapaer_d                                * &
                                   & (coef_scatt_ir%fmv_aer_ph_val_cos(kk)-scattangle_d)/deltaaer 
                          phup    =phash(kk)                                           + &
                                   & deltapaer*(coef_scatt_ir%fmv_aer_ph_val_cos(kk)-scattangle_d)/deltaaer              - &
                                   & scattangle*deltapaer_d/deltaaer 
!                         exit       
!                       endif
!                     enddo
                      phasint=phasint+phup*iang*ZPI
                  enddo loop1

                  transmission_scatt_ir_tl%azphaerup(j,jl)=                              &
                                    & transmission_scatt_ir_tl%azphaerup(j,jl)           + &
                       & profiles_tl(ipf)%aerosols(IAE,JL)                               * &
                       & transmission_scatt_ir%phasintupref(j,jl,i)*scach_d              * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & phasint*profiles(ipf)%aerosols(iae,jl)*scach_d                  * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & scach*profiles(ipf)%aerosols(iae,jl)                            * &
                       & transmission_scatt_ir%phasintupref(j,jl,i)                      * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & raytracing_tl%LTICK(nlevels+1-JL,ipf)*scach_d                  * &
                       & profiles(ipf)%aerosols(iae,jl)                                  * &
                       & transmission_scatt_ir%phasintupref(j,jl,i) 

!-------------Average phase function for the downward scattered solar beam------

                  phasint= 0._jprb

!                 cosan_aer(1:coef_scatt_ir%fmv_aer_ph)  =                               &
!                    & cos(coef_scatt_ir%fmv_aer_ph_val(1:coef_scatt_ir%fmv_aer_ph)*deg2rad)  

                  musat_d         = -1._jprb/raytracing%PATHSAT(JL,ipf)
                  musun_d         = -1._jprb/raytracing%PATHSUN(JL,ipf)
                  
                  musat           = raytracing_tl%pathsat(jl,ipf)                     / &
                                    & raytracing%PATHSAT(JL,ipf)**2  !                * &
!                                    raytracing%PATHSAT(JL,ipf)
                  musun           = raytracing_tl%pathsun(jl,ipf)                     / &
                                    & raytracing%PATHSUN(JL,ipf)**2  !                * &
!                                    raytracing%PATHSUN(JL,ipf)
                    loop2: do k=0,iend,iang 
                    scattangle_d=musat_d*musun_d+sqrt((1-musat_d**2)*(1-musun_d**2)) *cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d+musun*musat_d                         
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1-musun_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)           - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)                * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif
!                     do kk=1,coef_scatt_ir%fmv_aer_ph-1
!                       if(scattangle_d>=coef_scatt_ir%fmv_aer_ph_val_cos(kk+1).and.                            &
!                                                  & scattangle_d<=coef_scatt_ir%fmv_aer_ph_val_cos(kk))then 
                          ikk = max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_aer_ph_val_min*deg2rad),jpim) )
                          KK=coef_scatt_ir%ifmv_aer_ph_val(ikk)-1._jprb

                          deltapaer_d=(phash_d(kk+1)-phash_d(kk))
                          deltapaer  =(phash  (kk+1)-phash  (kk))

                          deltaaer  =(coef_scatt_ir%fmv_aer_ph_val_cos(kk)-coef_scatt_ir%fmv_aer_ph_val_cos(kk+1))

                          phdo_d  =phash_d(kk)+deltapaer_d                                * &
                                   & (coef_scatt_ir%fmv_aer_ph_val_cos(kk)-scattangle_d)/deltaaer 
                          phdo    =phash(kk)                                           + &
                                   & deltapaer*(coef_scatt_ir%fmv_aer_ph_val_cos(kk)-scattangle_d)/deltaaer              - &
                                   & scattangle*deltapaer_d/deltaaer 
!                         exit        
!                       endif
!                     enddo
                      phasint  =phasint  +phdo  *iang*ZPI
                  enddo loop2

                  transmission_scatt_ir_tl%azphaerdo(j,jl)=                              &
                           & transmission_scatt_ir_tl%azphaerdo(j,jl)                    + &
                       & profiles_tl(ipf)%aerosols(IAE,JL)                               * &
                       & transmission_scatt_ir%phasintdoref(j,jl,i)*scach_d              * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & phasint*profiles(ipf)%aerosols(IAE,JL)*scach_d                  * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & scach*profiles(ipf)%aerosols(IAE,JL)                            * &
                       & transmission_scatt_ir%phasintdoref(j,jl,i)                      * &
                       & raytracing%LTICK(nlevels+1-JL,ipf)                             + &
                       & raytracing_tl%ltick(nlevels+1-JL,ipf)*scach_d                  * &
                       & profiles(ipf)%aerosols(IAE,JL)                                  * &
                       & transmission_scatt_ir%phasintdoref(j,jl,i) 
                endif
              endif
            enddo
          enddo

!---------Compute final values for optical parameters-------------------------------------

          do jl=1,nlevels

            if(transmission_scatt_ir%opdpsaer(j,jl)/=0._jprb)then
              transmission_scatt_ir_tl%gparaer (j,jl)=                                   &
                             & transmission_scatt_ir_tl%gparaera(j,jl)                   / &
                             & transmission_scatt_ir%opdpsaer(J,JL)                      - &
                             & transmission_scatt_ir_tl%opdpsaer(j,jl)                   * &
                             & transmission_scatt_ir%gparaera (J,JL)                     / &
                             & transmission_scatt_ir%opdpsaer(j,jl)**2 

              IF(profiles(IPF)%addsolar)THEN
                if(coef%ss_val_chn(ich)==1)then
                  transmission_scatt_ir_tl%azphaerupa(j,jl)=                             &
                             & transmission_scatt_ir_tl%azphaerup(j,jl)                  / &
                             & transmission_scatt_ir%opdpsaer(j,jl)                      - &
                             & transmission_scatt_ir_tl%opdpsaer(j,jl)                   * &
                             & transmission_scatt_ir%azphaerup(j,jl)                     / &
                             & transmission_scatt_ir%opdpsaer(j,jl)**2 

                  transmission_scatt_ir_tl%azphaerdoa(j,jl)=                             &
                             & transmission_scatt_ir_tl%azphaerdo(j,jl)                  / &
                             & transmission_scatt_ir%opdpsaer(j,jl)                      - &
                             & transmission_scatt_ir_tl%opdpsaer(j,jl)                   * &
                             & transmission_scatt_ir%azphaerdo(j,jl)                     / &
                             & transmission_scatt_ir%opdpsaer(J,JL)**2 
                  endif
              endif
            endif

            transmission_scatt_ir_tl%opdpaerla(j,jl) =                                   &
                                   & transmission_scatt_ir_tl%opdpaaer(j,jl)             + &
                             & transmission_scatt_ir_tl%opdpsaer(j,jl)                   * &
                             & transmission_scatt_ir%gparaer(j,jl)                       + &
                             & transmission_scatt_ir_tl%gparaer (j,jl)                   * &
                             & transmission_scatt_ir%opdpsaer(j,jl) 

            opdpaerl(j,jl)  =transmission_scatt_ir_tl%opdpaerla(j,jl)                  * &
                             & raytracing%pathsat(jl,ipf)*coef%ff_gam(ich)              + &
                             & raytracing_tl%pathsat(jl,ipf)                            * &
                             & transmission_scatt_ir%opdpaerla(j,jl)*coef%ff_gam(ich) 

            IF(profiles(ipf)%addsolar)THEN
              if(coef%ss_val_chn(ich)==1)then
                opdpaerlsun(j,jl)=transmission_scatt_ir_tl%opdpaerla(j,jl)             * &
                       & (raytracing%pathsun(jl,ipf)+raytracing%pathsat(jl,ipf))       * &
                        & coef%ff_gam(ich)                                               + &
                        & raytracing_tl%pathsat(jl,ipf)                                 * &
                        & transmission_scatt_ir%opdpaerla(j,jl)*coef%ff_gam(ich)         + &
                        & raytracing_TL%pathsun(jl,ipf)                                 * &
                        & transmission_scatt_ir%opdpaerla(j,jl)*coef%ff_gam(ich) 
              endif
            endif
            enddo
          enddo
        endif
!-------------------------------------------------------------------------------
!         2.   CALCULATE OPTICAL DEPTHS OF CLOUDS
!-------------------------------------------------------------------------------

        if(addclouds)then
          do j=1,nchannels

          ich=channels(j)
          ipf=lprofiles(j)
          ish=profiles(ipf)%ish

!        IOFF=NUMAE

          do jl=1,nlevels

            transmission_scatt_ir_tl%azphup(j,jl)  =0._jprb
            transmission_scatt_ir_tl%azphdo(j,jl)  =0._jprb
            transmission_scatt_ir_tl%OPDPA(j,jl)   =0._jprb 
            transmission_scatt_ir_tl%OPDPS(j,jl)   =0._jprb
            transmission_scatt_ir_tl%GPAR(j,jl)    =0._jprb
!            transmission_scatt_ir_tl%OPDPAAER(j,jl)=0._jprb
!            transmission_scatt_ir_tl%OPDPSAER(j,jl)=0._jprb
!            transmission_scatt_ir_tl%GPARAER(j,jl) =0._jprb

            if(ircld%cldtyp(jl,ipf)/=0)then

              ityp = ircld%cldtyp (jl,ipf)
              iconf= ityp

!              ITMP = ITEMP  (JL,IPF)
!              ICONF= CLDTYP (JL,IPF)+ITEMP(JL,IPF)-1

!-------------If solar radiation is present,compute the azimuthally averaged----
!             value of the phase function for the given value of the viewing----
!             angle and solar zenith angle.

              if(profiles(ipf)%addsolar)then
                if(coef%ss_val_chn(ich)==1)then

!-----------------If ice clouds are present the phase function for for the current value
!                 of the effective generalized diameter is obtained by linear
!                 interpolation and then the azimuthally averaged value is computed 
!----------------------------------------------------------------------------------------

                if(ITYP==6_jpim)then

                      ich1=ich-coef_scatt_ir % fmv_icl_pha_ioff+1
                      do k=1,coef_scatt_ir % fmv_icl_comp
                        if(aux(ipf)%dg(jl)>=coef_scatt_ir %fmv_icl_dg(k,ish).and.       &
                          & aux(ipf)%dg(jl)<=coef_scatt_ir %fmv_icl_dg(k+1,ish))then 

                          deltap   =(optp%optpicl(ish)%PHA(ICH1,k+1,:)                      - &
                                                 & optp%optpicl(ish)%PHA(ICH1,k,:)) 

                          DELTAICE=(coef_scatt_ir %fmv_icl_dg(k+1,ish)                       - &
                                      & coef_scatt_ir %fmv_icl_dg(k,ish)) 

                          phasice_d(:)=optp%optpicl(ish)%PHA(ICH1,k,:)+deltap(:)         * &
                             & (aux(ipf)%dg(jl)-coef_scatt_ir %fmv_icl_dg(k,ish))/DELTAICE 

                          do kk=1,coef_scatt_ir%fmv_icl_ph
                            phasice(kk)=deltap(kk)*aux_tl(ipf)%dg(jl)/DELTAICE
                          enddo
                         exit
                        endif
                      enddo

!---------------------Average phase function for the upward scattered solar beam----------

                  phasint= 0._jprb

!                 cosan_icl(1:coef_scatt_ir%fmv_icl_ph)  =                               &
!                    & cos(coef_scatt_ir%fmv_icl_ph_val(1:coef_scatt_ir%fmv_icl_ph)*deg2rad) 

                  musat_d=  1._jprb/raytracing%pathsat(jl,ipf)
                  musun_d= -1._jprb/raytracing%pathsun(jl,ipf)

                  musat  =  -raytracing_tl%pathsat(jl,ipf)/raytracing%pathsat(jl,ipf)**2
                  musun  =   raytracing_tl%pathsun(jl,ipf)/raytracing%pathsun(jl,ipf)**2

                  loop3: do k=0,iend,iang
                    scattangle_d=musat_d*musun_d+sqrt((1._jprb-musat_d**2)*(1._jprb-musun_d**2))&
                                 & * cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d 
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1._jprb-musun_d**2)                * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)           - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)               * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif

!                   do kk=1,coef_scatt_ir%fmv_icl_ph-1
!                     if(scattangle_d>=coef_scatt_ir%fmv_icl_ph_val_cos(kk+1) &
!                       &.and.scattangle_d<=coef_scatt_ir%fmv_icl_ph_val_cos(kk))then
                          ikk = max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_icl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_icl_ph_val(ikk)-1._jprb

                        deltapice=(PHAsice(KK+1)-PHAsice(KK)) 
                        deltapice_d=(PHAsice_d(KK+1)-PHAsice_d(KK))
                        deltaice=(coef_scatt_ir%fmv_icl_ph_val_cos(KK)-coef_scatt_ir%fmv_icl_ph_val_cos(KK+1))
                        phasint=PHAsice(KK)                                            + &
                                     & DELTAPice                                            * &
                                        & (coef_scatt_ir%fmv_icl_ph_val_cos(KK)-SCATTANGLE_d)/DELTAice- &
                                     & SCATTANGLE*DELTAPice_d/DELTAice 
!                       exit
!                     endif
!                   enddo

                    transmission_scatt_ir_tl%azphup(j,jl)=                               &
                                                  & transmission_scatt_ir_tl%AZPHUP(J,JL)+ &
                                                  & phasint*iang*ZPI
                  enddo loop3

!---------------------Average phase function for the downward scattered solar beam--------

                  phasint= 0._jprb
!                 cosan_icl(1:coef_scatt_ir%fmv_icl_ph)  =                               &
!                    & cos(coef_scatt_ir%fmv_icl_ph_val(1:coef_scatt_ir%fmv_icl_ph)*deg2rad) 
                  musat_d= -1._jprb/raytracing%pathsat(jl,ipf)
                  musun_d= -1._jprb/raytracing%pathsun(jl,ipf)

                  musat  = raytracing_tl%pathsat(jl,ipf)                              / &
                           & raytracing%pathsat(jl,ipf)**2 
                  musun  = raytracing_tl%pathsun(jl,ipf)                              / &
                           & raytracing%pathsun(jl,ipf)**2 

                  loop4: do k=0,iend,iang
                    scattangle_d=musat_d*musun_d+sqrt((1._jprb-musat_d**2)*(1._jprb-musun_d**2))*cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d 
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1._jprb-musun_d**2)                * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)           - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)                * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif

!                   do kk=1,coef_scatt_ir%fmv_icl_ph-1
!                     if(scattangle_d>=coef_scatt_ir%fmv_icl_ph_val_cos(kk+1) &
!                       &.and.scattangle_d<=coef_scatt_ir%fmv_icl_ph_val_cos(kk))then
                          ikk=max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_icl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_icl_ph_val(ikk)-1._jprb

                        deltapice_d=(PHAsice_d(KK+1)-PHAsice_d(KK))
                        deltapice=(PHAsice(KK+1)-PHAsice(KK))

                        deltaice=(coef_scatt_ir%fmv_icl_ph_val_cos(kk)-coef_scatt_ir%fmv_icl_ph_val_cos(kk+1))

                        phasint=PHAsice(KK)                                            + &
                                    & DELTAPice*(coef_scatt_ir%fmv_icl_ph_val_cos(KK)-SCATTANGLE_d)/DELTAice         - &
                                    & SCATTANGLE*DELTAPice_d/DELTAice  
!                       exit
!                     endif
!                   enddo
                    transmission_scatt_ir_tl%azphdo  (j,jl) =                            &
                            & transmission_scatt_ir_tl%azphdo  (J,JL)                    + &
                            & phasint  *iang*ZPI
                  enddo loop4

                else

!-----------------------------------------------------------------------------------------
!                     Water clouds                                                        
!-----------------------------------------------------------------------------------------

                  ich1=ich-coef_scatt_ir % fmv_wcl_pha_ioff+1

!-----------------Average phase function for the upward scattered solar beam--------------

                  phasint= 0._jprb

!                 cosan_wcl(1:coef_scatt_ir%fmv_wcl_ph)  =                               &
!                    & cos(coef_scatt_ir%fmv_wcl_ph_val(1:coef_scatt_ir%fmv_wcl_ph)*deg2rad) 

                  musat_d=  1._jprb/raytracing%pathsat(jl,ipf)
                  musun_d= -1._jprb/raytracing%pathsun(jl,ipf)

                  musat  =  -raytracing_tl%pathsat(jl,ipf)/raytracing%pathsat(jl,ipf)**2
                  musun  =   raytracing_tl%pathsun(jl,ipf)/raytracing%pathsun(jl,ipf)**2

                  loop5: do k=0,iend,iang
                    scattangle_d=musat_d*musun_d+sqrt((1._jprb-musat_d**2)*(1._jprb-musun_d**2))&
                        & * cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d 
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1._jprb-musun_d**2)             * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)         - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif

!                   do kk=1,coef_scatt_ir%fmv_wcl_ph-1
!                     if(scattangle_d>=coef_scatt_ir%fmv_wcl_ph_val_cos(kk+1) &
!                       &.and.scattangle_d<=coef_scatt_ir%fmv_wcl_ph_val_cos(kk))then
                          ikk = max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_wcl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_wcl_ph_val(ikk)-1._jprb
                                                                         
                        deltapwcl_d=(optp%optpwcl(ityp)%PHA(ICH1,1,KK+1)                    - &
                                                & optp%optpwcl(ityp)%PHA(ICH1,1,KK)) 
                        deltawcl=(coef_scatt_ir%fmv_wcl_ph_val_cos(KK)-coef_scatt_ir%fmv_wcl_ph_val_cos(KK+1))
                        phasint=-scattangle*deltapwcl_d/deltawcl
!                       exit         
!                     endif
!                   enddo
                    
                    transmission_scatt_ir_tl%azphup(j,jl)=                               &
                                                  & transmission_scatt_ir_tl%AZPHUP(J,JL)+ &
                                                  & phasint*iang*ZPI
                  enddo loop5

!-----------------Average phase function for the downward scattered solar beam------------

                  phasint= 0._jprb
!                 cosan_wcl(1:coef_scatt_ir%fmv_wcl_ph)  =                               &
!                    & cos(coef_scatt_ir%fmv_wcl_ph_val(1:coef_scatt_ir%fmv_wcl_ph)*deg2rad) 
                  musat_d= -1._jprb/raytracing%pathsat(jl,ipf)
                  musun_d= -1._jprb/raytracing%pathsun(jl,ipf)

                  musat  = raytracing_tl%pathsat(jl,ipf)                              / &
                           & raytracing%pathsat(jl,ipf)**2 
                  musun  = raytracing_tl%pathsun(jl,ipf)                              / &
                           & raytracing%pathsun(jl,ipf)**2 

                  loop6: do k=0,iend,iang
                    scattangle_d=musat_d*musun_d+sqrt((1._jprb-musat_d**2)*(1._jprb-musun_d**2))&
                           & * cos(k*deg2rad) 

                    if(abs(musat_d)==1._jprb.or.abs(musun_d)==1._jprb)then
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                
                    else
                      scattangle=musat*musun_d                                         + &
                                 & musun*musat_d                                         - &
                                 & musat*musat_d*sqrt(1._jprb-musun_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musat_d**2)                 - &
                                 & musun*musun_d*sqrt(1._jprb-musat_d**2)                      * &
                                     & cos(k*deg2rad)/sqrt(1._jprb-musun_d**2) 
                    endif

!                   do kk=1,coef_scatt_ir%fmv_wcl_ph-1
!                     if(scattangle_d>=coef_scatt_ir%fmv_wcl_ph_val_cos(kk+1) &
!                       &.and.scattangle_d<=coef_scatt_ir%fmv_wcl_ph_val_cos(kk))then 
                          ikk=max(1_jpim,int(acos(scattangle_d)/(coef_scatt_ir%fmv_wcl_ph_val_min*deg2rad),jpim))
                          KK=coef_scatt_ir%ifmv_wcl_ph_val(ikk)-1._jprb

                        deltapwcl_d=(optp%optpwcl(ityp)%pha(ich1,1,kk+1)                    - &
                                                & optp%optpwcl(ityp)%pha(ich1,1,kk)) 
                        deltawcl=(coef_scatt_ir%fmv_wcl_ph_val_cos(kk)-coef_scatt_ir%fmv_wcl_ph_val_cos(kk+1))

                        phasint=-scattangle*deltapwcl_d/deltawcl
!                       exit         
!                     endif
!                   enddo
                    transmission_scatt_ir_tl%azphdo  (j,jl) =                            &
                            & transmission_scatt_ir_tl%azphdo  (J,JL)                    + &
                            & phasint  *iang*ZPI
                  enddo loop6

                  endif
                endif
              endif

!---------------Compute cloud  optical parameters ----------------------------------------

                if(ityp<=5_jpim)then

!-----------------------------------------------------------------------------------------
!                 For water clouds use stored optical parameters
!-----------------------------------------------------------------------------------------

                transmission_scatt_ir_tl%opdpa(J,JL)=profiles_tl(IPF)%cloud(Ityp,JL)  * &
                              & coef_scatt_ir%CONFAC(ICONF)                              * &
                              & optp%optpwcl(ityp)%abs(ich,1)                            * &
                              & raytracing%LTICK(nlevels+1-JL,IPF)                      + &
                              & raytracing_tl%LTICK(nlevels+1-JL,IPF)                   * &
                              & profiles(IPF)%cloud(Ityp,JL)                            * &
                              & coef_scatt_ir%CONFAC(ICONF)                              * &
                              & optp%optpwcl(ityp)%abs(ich,1) 
                transmission_scatt_ir_tl%opdps(j,jl)=profiles_tl(IPF)%cloud(Ityp,JL)  * &
                              & coef_scatt_ir%CONFAC(ICONF)                              * &
                              & optp%optpwcl(ityp)%sca(ich,1)                            * &
                              & raytracing%LTICK(nlevels+1-JL,IPF)                      + &
                              & raytracing_tl%LTICK(nlevels+1-JL,IPF)                   * &
                              & profiles(IPF)%cloud(Ityp,JL)                            * &
                              & coef_scatt_ir%CONFAC(ICONF)                              * &
                              & optp%optpwcl(ityp)%sca(ich,1) 
!                OPDPE(J,JL)=PROFCLD(ITYP,JL,IPF)*CONFAC(ICONF)               * &
!                              EXTC(ICH,IOFF+ITYP,ITMP)*LTICK(90-JL,IPF)      + &
!                            LTICK(90-JL,IPF)*PROFCLD_D(ITYP,JL,IPF)          * &
!                              CONFAC(ICONF)*EXTC(ICH,IOFF+ITYP,ITMP)

                transmission_scatt_ir_tl%GPAR (J,JL)=0._jprb

                else

!-----------------------------------------------------------------------------------------
!                 For ice clouds optical parameters are computed using regression
!                 coefficients
!-----------------------------------------------------------------------------------------

                  transmission_scatt_ir_tl%OPDPA(J,JL)=profiles_tl(IPF)%cloud(Ityp,JL) * &
                     & (optp%optpicl(ish)%abs(ich,1)                                          + &
                      & optp%optpicl(ish)%abs(ich,2)*aux(ipf)%dg(jl)                          + &
                      & optp%optpicl(ish)%abs(ich,3)/aux(ipf)%dg(jl)                          + &
                      & optp%optpicl(ish)%abs(ich,4)/aux(ipf)%dg(jl)**2)                      * &
                      & raytracing%LTICK(nlevels+1-JL,IPF)                              + &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%abs(ich,2)                         * &
                      & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF) - &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%abs(ich,3)                         * &
                      & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF) /  &
                      & aux(ipf)%dg(jl)**2                                               - &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%abs(ich,4)                         * &
                     & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF)  * &
                     & 2._jprb/aux(ipf)%dg(jl)**3                                        + &
                    & raytracing_tl%LTICK(nlevels+1-JL,IPF)*profiles(IPF)%cloud(Ityp,JL)* &
                     & (optp%optpicl(ish)%abs(ich,1)                                          + &
                     & optp%optpicl(ish)%abs(ich,2)*aux(ipf)%dg(jl)                           + &
                     & optp%optpicl(ish)%abs(ich,3)/aux(ipf)%dg(jl)                           + &
                     & optp%optpicl(ish)%abs(ich,4)/aux(ipf)%dg(jl)**2) 

                  transmission_scatt_ir_tl%OPDPS(J,JL)=profiles_tl(IPF)%cloud(Ityp,JL) * &
                     & (optp%optpicl(ish)%sca(ich,1)                                          + &
                      & optp%optpicl(ish)%sca(ich,2)*aux(ipf)%dg(jl)                          + &
                      & optp%optpicl(ish)%sca(ich,3)/aux(ipf)%dg(jl)                          + &
                      & optp%optpicl(ish)%sca(ich,4)/aux(ipf)%dg(jl)**2)                      * &
                      & raytracing%LTICK(nlevels+1-JL,IPF)                              + &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%sca(ich,2)                         * &
                      & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF) - &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%sca(ich,3)                         * &
                      & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF) /   &
                      & aux(ipf)%dg(jl)**2                                               - &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%sca(ich,4)                         * &
                     & profiles(IPF)%cloud(Ityp,JL)*raytracing%LTICK(nlevels+1-JL,IPF)  * &
                     & 2._jprb/aux(ipf)%dg(jl)**3                                        + &
                    & raytracing_tl%LTICK(nlevels+1-JL,IPF)*profiles(IPF)%cloud(Ityp,JL)* &
                     & (optp%optpicl(ish)%sca(ich,1)                                          + &
                     & optp%optpicl(ish)%sca(ich,2)*aux(ipf)%dg(jl)                           + &
                     & optp%optpicl(ish)%sca(ich,3)/aux(ipf)%dg(jl)                           + &
                     & optp%optpicl(ish)%sca(ich,4)/aux(ipf)%dg(jl)**2) 

                  transmission_scatt_ir_tl%GPAR (J,JL)=                                  &
                    & aux_tl(ipf)%dg(jl)*optp%optpicl(ish)%bpr(ich,2)                         + &
                    & aux_tl(ipf)%dg(jl)*2._jprb*aux(ipf)%dg(jl)*optp%optpicl(ish)%bpr(ich,3)       + &
                    & aux_tl(ipf)%dg(jl)*3._jprb*aux(ipf)%dg(jl)**2*optp%optpicl(ish)%bpr(ich,4) 
                    

                endif


                transmission_scatt_ir_tl%opdpcldla(j,jl)   =                             &
                              & transmission_scatt_ir_tl%opdpa(j,jl)                     + &
                              & transmission_scatt_ir_tl%opdps(j,jl)                     * &
                              & transmission_scatt_ir%gpar (J,JL)                        + &
                              & transmission_scatt_ir_tl%gpar (j,jl)                     * &
                              & transmission_scatt_ir%opdps(J,JL) 

                OPDPCLDL(J,JL)   = transmission_scatt_ir_tl%opdpcldla(j,jl)            * &
                              & raytracing%PATHSAT(JL,ipf)                              * &
                              & coef%ff_gam(ICH)                                         + &
                              & raytracing_tl%pathsat(jl,ipf)                           * &
                              & transmission_scatt_ir%opdpcldla(j,jl)                    * &
                              & coef%ff_gam(ICH) 

                IF(profiles(IPF)%addsolar)THEN
                  if(coef%ss_val_chn(ich)==1)then
                    OPDPCLDLSUN(J,JL)= transmission_scatt_ir_tl%OPDPCLDLA(J,JL)        * &
                       & (raytracing%PATHSAT(JL,ipf)+raytracing%PATHSUN(JL,ipf))       * &
                       & coef%ff_gam(ICH)                                                + &
                       & raytracing_tl%PATHSAT(JL,ipf)                                  * &
                       & transmission_scatt_ir%OPDPCLDLA(J,JL)*coef%ff_gam(ICH)          + &
                       & raytracing_tl%PATHSUN(JL,ipf)                                  * &
                       & transmission_scatt_ir%OPDPCLDLA(J,JL)*coef%ff_gam(ICH) 
                  endif
                endif

            endif
          enddo

        enddo
      endif 
      
!-----Compute optical parameters for each stream--------------------------------

      do j=1,nchannels
        ich = channels(J) 
        ipf = lprofiles(J)
          do ist=0,ircld%nstream(ipf)

            if(ist==0)then
              IF(addaerosl)THEN
                OPD=0._jprb
                OPDSUN=0._jprb
                OPD1=0._jprb
                do jl=1,nlevels
                  if(profiles(ipf)%addsolar)then
                    if(coef%ss_val_chn(ich)==1)then
                      transmission_scatt_ir_stream_tl%azphacup(ist,j,jl)=           &
                               & transmission_scatt_ir_tl%azphaerupa(j,jl) 

                      transmission_scatt_ir_stream_tl%azphacdo(ist,j,jl)=           &
                               & transmission_scatt_ir_tl%azphaerdoa(j,jl) 

                      transmission_scatt_ir_stream_tl%bcksp   (ist,j,jl)=           &
                               & transmission_scatt_ir_tl%gparaer   (j,jl) 

                      transmission_scatt_ir_stream_tl%opdpabs (ist,j,jl)=           &
                               & transmission_scatt_ir_tl%opdpaaer(j,jl)                 * &
                                          & (raytracing%PATHSAT(JL,ipf)                 + &
                                          & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH)+ &
                                          & raytracing_tl%PATHSAT(JL,ipf)               * &
                                          & transmission_scatt_ir%OPDPAAER(J,JL)         * &
                                          & coef%ff_gam(ICH)                             + &
                                          & raytracing_tl%PATHSUN(JL,ipf)               * &
                                          & transmission_scatt_ir%OPDPAAER(J,JL)         * &
                                          & coef%ff_gam(ICH) 

                      transmission_scatt_ir_stream_tl%opdpsca (ist,j,jl)  =         &
                                          & transmission_scatt_ir_tl%opdpsaer(j,jl)      * &
                                          & (raytracing%PATHSAT(JL,ipf)                 + &
                                          & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH)+ &
                                          & raytracing_tl%PATHSAT(JL,ipf)               * &
                                          & transmission_scatt_ir%opdpsaer(j,jl)         * &
                                          & coef%ff_gam(ICH)                             + &
                                          & raytracing_tl%PATHSUN(JL,ipf)               * &
                                          & transmission_scatt_ir%opdpsaer(j,jl)         * &
                                          & coef%ff_gam(ICH)  

                      transmission_scatt_ir_stream_tl%opdpaclsun(ist,j,jl)    =     &
                                          & opdpaerlsun(j,jl) 

                      opdsun = opdsun                                                  + &
                              & transmission_scatt_ir_stream_tl%OPDPACLSUN(IST,J,JL) 

                      transmission_scatt_ir_stream_tl%OPDPACSUN(IST,J,JL) = OPDSUN 
                    endif
                  endif
                    transmission_scatt_ir_stream_tl%OPDPACL(IST,J,JL) = OPDPAERL(J,JL)

                    OPD                     = OPD                                      + &
                              & transmission_scatt_ir_stream_tl%OPDPACL(IST,J,JL) 

                    transmission_scatt_ir_stream_tl%opdpac(ist,j,jl)     = OPD

!                    OPD1                    = OPD1+OPDPEAER(J,JPLAY-JL+1)
                enddo
              ELSE IF((.not.addaerosl).and.(addclouds))then 
                transmission_scatt_ir_stream_tl%opdpaclsun(ist,j,:)=0._jprb
                transmission_scatt_ir_stream_tl%OPDPACSUN (IST,J,:)=0._jprb
                transmission_scatt_ir_stream_tl%OPDPACL   (IST,J,:)=0._jprb
                transmission_scatt_ir_stream_tl%opdpac    (ist,j,:)=0._jprb
                transmission_scatt_ir_stream_tl%azphacup  (ist,j,:)=0._jprb
                transmission_scatt_ir_stream_tl%azphacdo  (ist,j,:)=0._jprb
              endif
            else
              if(addclouds)then
              opd=0._jprb
              opdsun=0._jprb
              do jl=1,nlevels
                IF(profiles(IPF)%addsolar)THEN
                  if(coef%ss_val_chn(ich)==1)then
                    IF((ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)   + &
                               & transmission_scatt_ir%OPDPSAER(J,JL))/=0._jprb)THEN 

                      transmission_scatt_ir_stream_tl%AZPHACUP(IST,J,JL)=           &
                          & transmission_scatt_ir_tl%AZPHAERUPA(J,JL)                    * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        + &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHAERUPA(J,JL)                       / &
                          & (ircld%ICLDARR(IST,JL,ipf)                                  * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            + &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHAERUPA(J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHAERUPA(J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     + &
                          & transmission_scatt_ir_tl%AZPHUP(J,JL)                        * &
                          & transmission_scatt_ir%OPDPS(J,JL)*ircld%ICLDARR(IST,JL,ipf) / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        + &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHUP(J,JL)                           / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHUP(J,JL)                           * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
   & 		      			  transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHUP(J,JL)                           * &
& 			     		  transmission_scatt_ir%OPDPS(J,JL)*ircld%ICLDARR(IST,JL,ipf) / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2  

                      transmission_scatt_ir_stream_tl%AZPHACDO(IST,J,JL)=           &
                          & transmission_scatt_ir_tl%AZPHAERDOA(J,JL)                    * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        + &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHAERDOA(J,JL)                       / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHAERDOA(J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHAERDOA(J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     + &
                          & transmission_scatt_ir_tl%AZPHDO(J,JL)                        * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL)) + &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHDO(J,JL)                           / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%AZPHDO(J,JL)                           * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%AZPHDO(J,JL)                           * &
                          & transmission_scatt_ir%OPDPS(J,JL)*ircld%ICLDARR(IST,JL,ipf) / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2 

                      transmission_scatt_ir_stream_tl%BCKSP   (IST,J,JL)=           &
                          & transmission_scatt_ir_tl%GPARAER   (J,JL)                    * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        + &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%GPARAER   (J,JL)                       / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%GPARAER   (J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%GPARAER   (J,JL)                       * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     + &
                          & transmission_scatt_ir_tl%GPAR  (J,JL)                        * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        + &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%GPAR  (J,JL)                           / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))                        - &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & transmission_scatt_ir%GPAR  (J,JL)                           * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2                     - &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & transmission_scatt_ir%GPAR  (J,JL)                           * &
                          & transmission_scatt_ir%OPDPS(J,JL)                            * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   / &
                          & (ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL)+ &
                          & transmission_scatt_ir%OPDPSAER(J,JL))**2 

                      transmission_scatt_ir_stream_tl%OPDPABS (IST,J,JL)=           &
                          & raytracing_tl%PATHSAT(JL,ipf)*coef%ff_gam(ICH)              * &
                          & transmission_scatt_ir%OPDPAAER(J,JL)                         + &
                          & raytracing_tl%PATHSAT(JL,ipf)*coef%ff_gam(ICH)              * &
                          & ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPA(J,JL) + &
                          & transmission_scatt_ir_tl%OPDPAAER(J,JL)                      * &
                          & (raytracing%PATHSAT(JL,ipf)                                 + &
                          & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH)                + &
                          & transmission_scatt_ir_tl%OPDPA(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & (raytracing%PATHSAT(JL,ipf)                                 + &
                          & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH)                + &
                          & raytracing_tl%PATHSUN(JL,ipf)*coef%ff_gam(ICH)              * &
                          & transmission_scatt_ir%OPDPAAER(J,JL)                         + &
                          & raytracing_tl%PATHSUN(JL,ipf)*coef%ff_gam(ICH)              * &
                          & ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPA(J,JL)  

                      transmission_scatt_ir_stream_tl%OPDPSCA (IST,J,JL)=           &
                          & raytracing_tl%PATHSAT(JL,ipf)*coef%ff_gam(ICH)              * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         + &
                          & raytracing_tl%PATHSAT(JL,ipf)*coef%ff_gam(ICH)              * &
                          & ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL) + &
                          & transmission_scatt_ir_tl%OPDPSAER(J,JL)                      * &
                          & (raytracing%PATHSAT(JL,ipf)                                 + &
                          & raytracing%PATHSUN(JL,ipf))* coef%ff_gam(ICH)               + &
                          & transmission_scatt_ir_tl%OPDPS(J,JL)                         * &
                          & ircld%ICLDARR(IST,JL,ipf)                                   * &
                          & (raytracing%PATHSAT(JL,ipf)                                 + &
                          & raytracing%PATHSUN(JL,ipf))*coef%ff_gam(ICH)                + &
                          & raytracing_tl%PATHSUN(JL,ipf)*coef%ff_gam(ICH)              * &
                          & transmission_scatt_ir%OPDPSAER(J,JL)                         + &
                          & raytracing_tl%PATHSUN(JL,ipf)*coef%ff_gam(ICH)              * &
                          & ircld%ICLDARR(IST,JL,ipf)*transmission_scatt_ir%OPDPS(J,JL) 
                    ELSE
    
                      transmission_scatt_ir_stream_tl%AZPHACUP(IST,J,JL)=0._jprb 
                      transmission_scatt_ir_stream_tl%AZPHACDO(IST,J,JL)=0._jprb
                      transmission_scatt_ir_stream_tl%BCKSP   (IST,J,JL)=0._jprb
                      transmission_scatt_ir_stream_tl%OPDPSCA (IST,J,JL)=0._jprb
                      transmission_scatt_ir_stream_tl%OPDPABS (IST,J,JL)=0._jprb
                      
                    ENDIF

                    transmission_scatt_ir_stream_tl%OPDPACLSUN(IST,J,JL)=               &
                          & OPDPCLDLSUN(J,JL)*ircld%ICLDARR(IST,JL,ipf)+OPDPAERLSUN(J,JL) 

                    OPDSUN                   = OPDSUN                                      + &
                          & transmission_scatt_ir_stream_tl%OPDPACLSUN(IST,J,JL) 

                    transmission_scatt_ir_stream_tl%OPDPACSUN(IST,J,JL)  = OPDSUN
                  ENDIF
                ENDIF
                transmission_scatt_ir_stream_tl%OPDPACL(IST,J,JL) =                 &
                          & OPDPCLDL(J,JL)*ircld%ICLDARR(IST,JL,ipf)+OPDPAERL(J,JL) 
                OPD                      = OPD                                         + &
                          & transmission_scatt_ir_stream_tl%OPDPACL(IST,J,JL) 
!                OPDSUN                   = OPDSUN                                      + &
!                          transmission_scatt_ir_stream_tl%OPDPACLSUN(IST,J,JL)
                transmission_scatt_ir_stream_tl%OPDPAC(IST,J,JL)     = OPD
!                transmission_scatt_ir_stream_tl%OPDPACSUN(IST,J,JL)  = OPDSUN
              ENDDO
            ENDIF
            ENDIF
          ENDDO
      ENDDO

IF (LHOOK) CALL DR_HOOK('RTTOV_OPDPSCATTIR_TL',1_jpim,ZHOOK_HANDLE)
      END SUBROUTINE rttov_opdpscattir_tl
