!
Subroutine rttov_setpredictors_7_k( &
      & nchannels,     &! in
      & nprofiles,     &! in
      & nlevels,       &! in
      & angles,        &! in
      & lprofiles,     &! in
      & profiles,      &! in
      & profiles_k,    &! inout
      & coef,          &! in
      & aux_prof,      &! in
      & predictors,    &! in
      & predictors_k,  &! inout
      & raytracing,    &!
      & raytracing_k ) ! inout 
  ! Description
  ! RTTOV-9 Model
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2005, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  ! see RTTOV7 science and validation report pages 18/19
  ! variable names are close to the documentation
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       22/06/2005  initial (P Brunel)
  !                        based on version 1.2 (29/03/05) of AD code
  !  1.1       07/12/2005  Add surface humidity (R. Saunders) 
  !  1.2       06/03/2006  Add RTIASI features (RTTOV-8M) (R. Saunders) 
  !  1.3       12/02/2007  Removed polarisation indexing ( R Saunders)
  !  1.4       12/08/2008  Zeeman effect based on Yong Han (P. Rayer)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
      & gravity      ,&
      & sensor_id_mw ,&
      & use_q2m 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only : &
    & rttov_coef     ,&
    & profile_Type   ,&
    & geometry_Type  ,&
    & profile_aux    ,&
    & predictors_Type,&
    & raytracing_type  

  Use parkind1, Only : jpim     ,jprb
USE PARKIND1  ,ONLY : JPRB
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),    Intent(in)    :: nchannels               ! Number of output radiances
  Integer(Kind=jpim),    Intent(in)    :: nprofiles               ! Number of profiles
  Integer(Kind=jpim),    Intent(in)    :: nlevels                 ! Number of levels 
  Integer(Kind=jpim),    Intent(in)    :: lprofiles(nchannels)    ! Profiles indices

  Type(profile_Type),    Intent(in)     :: profiles(nprofiles)
  Type(profile_Type),    Intent(inout)  :: profiles_k(nchannels)
  Type(geometry_Type),   Intent(in)     :: angles(nprofiles)
  Type(predictors_Type), Intent(in)     :: predictors
  Type(predictors_Type), Intent(inout)  :: predictors_k
  Type(raytracing_type), Intent(in)     :: raytracing
  Type(raytracing_type), Intent(inout)  :: raytracing_k
  Type(profile_aux),     Intent(in)     :: aux_prof(nprofiles)    ! auxillary profiles info.
  Type(rttov_coef),      Intent(in)     :: coef

!INTF

  !local variables:
  Integer(Kind=jpim) :: level
  Integer(Kind=jpim) :: i          ! channel indice
  Integer(Kind=jpim) :: j          ! profile indice
  Integer(Kind=jpim) :: iv2, iv3(nprofiles)

  ! user profile
  Real(Kind=jprb) :: t(nprofiles, nlevels)
  Real(Kind=jprb) :: w(nprofiles, nlevels)
  Real(Kind=jprb) :: o(nprofiles, nlevels)

  ! reference profile
  Real(Kind=jprb) :: tr(nprofiles, nlevels)
  Real(Kind=jprb) :: wr(nprofiles, nlevels)
  Real(Kind=jprb) :: or(nprofiles, nlevels)

  ! user - reference
  Real(Kind=jprb) :: dt(nprofiles, nlevels)

  ! pressure weighted
  Real(Kind=jprb) :: tw(nprofiles, nlevels)
  Real(Kind=jprb) :: ww(nprofiles, nlevels)
  Real(Kind=jprb) :: ow(nprofiles, nlevels)


  Real(Kind=jprb) :: sum1(nchannels),sum2(nchannels)
  Real(Kind=jprb) :: deltac(nlevels)
  Real(Kind=jprb) :: sec_wr(nlevels)
  Real(Kind=jprb) :: sum2_ww(nprofiles, nlevels)
  Real(Kind=jprb) :: sum2_ow(nprofiles, nlevels)

  ! K variables
  Real(Kind=jprb) :: t_k(nchannels, nlevels)
  Real(Kind=jprb) :: w_k(nchannels, nlevels)
  Real(Kind=jprb) :: o_k(nchannels, nlevels)

  Real(Kind=jprb) :: tr_k(nchannels, nlevels)
  Real(Kind=jprb) :: wr_k(nchannels, nlevels)
  Real(Kind=jprb) :: or_k(nchannels, nlevels)

  Real(Kind=jprb) :: dt_k(nchannels, nlevels)
  Real(Kind=jprb) :: dto_k(nchannels, nlevels)

  Real(Kind=jprb) :: tw_k(nchannels, nlevels)
  Real(Kind=jprb) :: ww_k(nchannels, nlevels)
  Real(Kind=jprb) :: ow_k(nchannels, nlevels)


  Real(Kind=jprb) :: sec_or_k(nlevels)
  Real(Kind=jprb) :: sec_wr_k(nlevels)
  Real(Kind=jprb) :: zsqrt, zrecip
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------
  !
  ! Keep use of prof%nlevels in the code instead of input argument nlevels
  ! This is to allow profiles on variable levels, for future version.
!!!profiles_k(:) % nlevels =nlevels
  !
  ! profile layer quantities
  ! Direct variables
IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_K',0_jpim,ZHOOK_HANDLE)
  Do j=1,nprofiles
    t(j,1) = profiles(j) % t(1)
    w(j,1) = profiles(j) % q(1)
  Enddo
!CDIR NOLOOPCHG
  Do level=2,profiles(1)%nlevels
    Do j = 1, nprofiles
      t(j,level) = ( profiles(j) % t(level-1) +  profiles(j) % t(level ) ) *0.5_JPRB
      w(j,level) = ( profiles(j) % q(level-1) +  profiles(j) % q(level ) ) *0.5_JPRB
    Enddo
  Enddo
     !
  If( use_q2m )Then
    Do j=1,nprofiles
     ! include surface humidity
       iv3(j) = aux_prof(j) % nearestlev_surf - 1
       iv2 = aux_prof(j) % nearestlev_surf
       If ( iv2 <= coef % nlevels) Then
         w(j,iv2) = (profiles(j) % s2m % q  + profiles(j) % q(iv3(j))) *0.5_JPRB
       Endif
    Enddo
  Endif
     !
  Do j=1,nprofiles
    If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
      o(j,1) = profiles(j) % o3(1)
      Else
      o(j,1 ) = 0.0_JPRB
    Endif
  EndDo
!CDIR NOLOOPCHG
  Do level=2,profiles(1)%nlevels
    Do j=1,nprofiles
      If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
        o(j,level ) = ( profiles(j) % o3(level-1) + profiles(j) % o3(level  ) ) *0.5_JPRB
      Else
        o(j,level ) = 0.0_JPRB
      Endif
    EndDo
  EndDo

     !3) calculate deviations from reference profile (layers)
     ! if no input O3 profile, set to reference value (dto =0)
     ! Direct variables
!CDIR NOLOOPCHG
  Do level=1,profiles(1)%nlevels
    Do j=1,nprofiles
      dt(j,level)  = t(j,level) - coef % tstar(level)

     !2) calculate (profile / reference profile) ratios; tr wr or
     ! if no input O3 profile, set to reference value (or =1)
     ! Direct variables
      tr(j,level) = t(j,level) / coef % tstar(level)
      wr(j,level) = w(j,level) / coef % wstar(level)
  ! if no input O3 profile, set to reference value (or =1)
      If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
        or(j,level) = o(j,level) / coef % ostar(level)
      Else
        or(j,level) = 1._JPRB
      Endif
    EndDo
  EndDo


     ! calculate profile / reference profile sums: tw ww ow
     ! if no input O3 profile, set to reference value (ow =1)
     ! Direct variables
  Do j=1,nprofiles
    tw(j,1) = 0._JPRB
  Enddo
  Do level = 2 , profiles(1) % nlevels
    Do j=1,nprofiles
      tw(j, level ) = tw(j, level-1 ) + coef % dpp( level ) * tr (j, level -1 )
    End Do
  End Do

  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, profiles(1) % nlevels
    Do j=1,nprofiles
      sum1(j) = sum1(j) + coef % dpp( level ) *      w       (j, level )
      sum2(j) = sum2(j) + coef % dpp( level ) * coef % wstar ( level )
      sum2_ww(j,level ) = sum2(j)
      ww (j, level ) = sum1(j) / sum2(j)
    End Do
  End Do
  ! if no input O3 profile, set to reference value (ow =1)
  sum1 = 0._JPRB
  sum2 = 0._JPRB
  Do level = 1, profiles(1) % nlevels
    Do j=1,nprofiles
      If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(j) = sum1(j) + coef % dpp( level ) *      o       (j, level )
        sum2(j) = sum2(j) + coef % dpp( level ) * coef % ostar ( level )
        sum2_ow( j,level ) = sum2(j)
        ow (j, level ) = sum1(j) / sum2(j)
      Else
        sum2_ow( j,level ) = 0.0_JPRB
        ow (j, level )     = 1.0_JPRB
      End If
    End Do
  End Do

  ! Ajoint code
  !-------------
  w_k(:,:) = 0._JPRB
  wr_k(:,:)= 0._JPRB
  ww_k(:,:)= 0._JPRB
  dt_k(:,:)= 0._JPRB

  t_k(:,:) = 0._JPRB
  tr_k(:,:)= 0._JPRB
  tw_k(:,:)= 0._JPRB

  If ( coef % nozone > 0 ) Then
    o_k(:,:)      = 0._JPRB
    or_k(:,:)     = 0._JPRB
    ow_k(:,:)     = 0._JPRB
    dto_k(:,:)    = 0._JPRB
  End If

  !5.4 cloud
  !---------
  If ( coef % id_sensor == sensor_id_mw ) Then
     deltac(:) = 0.1820_JPRB * 100.0_JPRB * coef % dp(:) / (4.3429_JPRB * gravity)
!CDIR NOLOOPCHG
     Do level = 2, profiles(1) % nlevels
     Do i = 1, nchannels
        j = lprofiles( i )
        If ( profiles(j) % clw_Data ) Then
           profiles_k(i)%clw(level-1) = profiles_k(i)%clw(level-1) +&
                 & 0.5_JPRB * predictors_k % clw(level,i) * &
                 & deltac(level) * angles(j)%seczen 
           predictors_k % clw(level,i) = 0.5_JPRB *  predictors_k % clw(level,i)
        Endif
     End Do
     End Do
!CDIR NOLOOPCHG
     Do level = 1, profiles(1) % nlevels
       Do i=1,nchannels
         j = lprofiles( i )
         If ( profiles (j)% clw_Data ) Then
           profiles_k(i)%clw(level) = profiles_k(i)%clw(level)  + predictors_k % clw(level,i) *&
                 & deltac(level) * angles(j)%seczen 
        Endif
     End Do
     End Do
  End If

  !5.3 ozone
  !---------
  If ( coef % nozone > 0 ) Then
     Do level = 1,profiles_k(1) % nlevels
        Do i = 1, nchannels
           j = lprofiles( i )
           sec_or_k(level) = 0._JPRB

           ! One can pack all ow_k lines in one longer statement
           ! same for sec_or_k and dto_k

        ow_k(i,level) = ow_k(i,level) +  predictors_k % ozone(11,level,i)*2       * &
                     & raytracing%pathsat(level,j)*predictors % ozone(10,level,j) 
        raytracing_k%pathsat(level,i)  = raytracing_k%pathsat(level,i)           + &
                     & predictors_k % ozone(11,level,i)*2 *ow(j,level)           * &
                     & predictors % ozone(10,level,j) 
        ow_k(i,level) = ow_k(i,level) + predictors_k % ozone(10,level,i)         * &
                     & raytracing%pathsat(level,j) 
        raytracing_k%pathsat(level,i) = raytracing_k%pathsat(level,i)            + &
                     & predictors_k % ozone(10,level,i)*ow(j,level) 
        or_k(i,level)= or_k(i,level) + predictors_k % ozone(9,level,i)           * &
                     & SQRT(raytracing%pathsat(level,j)                           * &
                    & ow(j,level))*raytracing%pathsat(level,j) 
        ow_k(i,level) = ow_k(i,level) + predictors_k % ozone(9,level,i)           * &
                     & predictors % ozone(1,level,j)*0.5                         * &
                     & raytracing%pathsat(level,j)/SQRT(raytracing%pathsat(level,j)* &
                     & ow(j,level)) 
        raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)              + &
                     & predictors_k % ozone(9,level,i)*1.5*or(j,level)          * &
                     & SQRT(predictors % ozone(10,level,j)) 
        or_k(i,level)=or_k(i,level) + predictors_k % ozone(8,level,i)             * &
                     & predictors % ozone(10,level,j) 
        ow_k(i,level)=ow_k(i,level) + predictors_k % ozone(8,level,i)              * &
                      & raytracing%pathsat(level,j)  *or(j,level) 
        raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)              + &
                     & predictors_k % ozone(8,level,i)*or(j,level)*ow(j,level) 
        or_k(i,level)=or_k(i,level) + predictors_k % ozone(7,level,i)*1.5         * &
                   & predictors % ozone(2,level,j)/ow(j,level) 
        ow_k(i,level) = ow_k(i,level)  - predictors_k % ozone(7,level,i)          * &
                     & predictors % ozone(7,level,j)/ow(j,level) 
        raytracing_k%pathsat(level,i) = raytracing_k%pathsat(level,i)           + &
                     & predictors_k % ozone(7,level,i)*0.5*or(j,level)**1.5    / &
                     & (raytracing%pathsat(level,j)**0.5*ow(j,level)) 
        or_k(i,level)=or_k(i,level) + predictors_k % ozone(6,level,i)*2           * &
                     & predictors % ozone(1,level,j)*ow(j,level) 
        ow_k(i,level) = ow_k(i,level) + predictors_k % ozone(6,level,i)           * &
                     & predictors % ozone(4,level,j)/raytracing%pathsat(level,j) 
        raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)              + &
                     & predictors_k % ozone(6,level,i)*or(j,level)**2*ow(j,level) 
        sec_or_k(level)=sec_or_k(level)+predictors_k % ozone(5,level,i)       * &
                     & 0.5_JPRB* predictors % ozone(3,level,j)                   / &
                     & ( predictors % ozone(1,level,j) *predictors % ozone(2,level,j)) 
        dto_k(i,level)=dto_k(i,level)+predictors_k % ozone(5,level,i)             * &
                     & predictors % ozone(2,level,j) 
        sec_or_k(level)=sec_or_k(level)+ predictors_k % ozone(4,level,i)*2    * &
                     & predictors % ozone(1,level,j) 
        sec_or_k(level)=sec_or_k(level)+predictors_k % ozone(3,level,i)       * &
                     & predictors % ozone(3,level,j) / predictors % ozone(1,level,j) 
        dto_k(i,level)=dto_k(i,level)+predictors_k % ozone(3,level,i)             * &
                     & predictors % ozone(1,level,j) 
        sec_or_k(level)=sec_or_k(level)+predictors_k % ozone(2,level,i)       * &
                     & 0.5_JPRB/ predictors % ozone(2,level,j) 
        sec_or_k(level)=sec_or_k(level)+predictors_k % ozone(1,level,i)
        or_k(i,level)=or_k(i,level)+sec_or_k(level) *raytracing%pathsat(level,j)
        raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)              + &
                     & sec_or_k(level)*or(j,level) 
        End Do
     End Do
  Endif

  !5.2 water vapour  ( numbers in right hand are predictorsictor numbers
  ! in the reference document for RTTOV7 (science and validation report)
  !----------------

  Do level = 1,profiles_k(1) % nlevels
    Do i = 1, nchannels
      j = lprofiles( i )
      sec_wr_k(level) = 0._JPRB

      sec_wr(level)    = raytracing%pathsat(level,j)*wr(j,level) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(15,level,i)  * &
                     & wr(j,level) / tr(j,level)**4 
      wr_k(i,level)=wr_k(i,level)+predictors_k % watervapour(15,level,i)          * &
                     & sec_wr(level)/ tr(j,level)**4 
      tr_k(i,level)=tr_k(i,level)-predictors_k % watervapour(15,level,i)*4        * &
                     & sec_wr(level) * wr(j,level) / tr(j,level)**5 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(14,level,i)  * &
                     & wr(j,level) / tr(j,level) 
      wr_k(i,level)=wr_k(i,level)+predictors_k % watervapour(14,level,i)          * &
                     & sec_wr(level) / tr(j,level) 
      tr_k(i,level)=tr_k(i,level)-predictors_k % watervapour(14,level,i)          * &
                     & sec_wr(level) * wr(j,level) / tr(j,level)**2 
      zsqrt=Sqrt(predictors % watervapour(13,level,j))
      ww_k(i,level)=ww_k(i,level)+predictors_k % watervapour(13,level,i)*2        * &
                 & raytracing%pathsat(level,j)*zsqrt 
      raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)                + &
                 & predictors_k % watervapour(13,level,i)*2 *ww(j,level) * zsqrt 
      ww_k(i,level)=ww_k(i,level)+predictors_k % watervapour(12,level,i)*4        * &
                 & raytracing%pathsat(level,j)*predictors % watervapour(12,level,j)/ &
                 & zsqrt 
      raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)                 + &
                 & predictors_k % watervapour(12,level,i)*4 *ww(j,level)          * &
                 & predictors %watervapour(12,level,j)/zsqrt 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(11,level,i)  * &
                 & Abs(dt(j,level))*dt(j,level) 
      dt_k(i,level)=dt_k(i,level)+predictors_k % watervapour(11,level,i)          * &
                 & Abs(dt(j,level)) *2 * sec_wr(level) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(10,level,i)  * &
                 & 4 *predictors % watervapour(9,level,j) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(9,level,i)   * &
                 & 3 *predictors % watervapour(5,level,j) 
      predictors_k % watervapour(2,level,i)=predictors_k % watervapour(2,level,i)+ &
                    & predictors_k % watervapour(8,level,i)*wr(j,level) / ww(j,level) 
      wr_k(i,level)=wr_k(i,level)+predictors_k % watervapour(8,level,i)           * &
                    & predictors % watervapour(2,level,j) / ww(j,level) 
      ww_k(i,level)=ww_k(i,level)-predictors_k % watervapour(8,level,i)           * &
                    & predictors % watervapour(2,level,j) * wr(j,level) /ww(j,level)**2 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(7,level,i)   * &
                    & 0.25_JPRB / predictors % watervapour(7,level,j)**3 
      zrecip=1.0_JPRB/ predictors % watervapour(1,level,j)
      dt_k(i,level)=dt_k(i,level)+predictors_k % watervapour(6,level,i)           * &
                    & predictors % watervapour(2,level,j) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(6,level,i)   * &
                    & 0.5_JPRB* predictors % watervapour(6,level,j) *zrecip 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(5,level,i)*2 * &
                    & predictors % watervapour(1,level,j) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(4,level,i)   * &
                    & predictors % watervapour(4,level,j) *zrecip 
      dt_k(i,level)=dt_k(i,level)+predictors_k % watervapour(4,level,i)           * &
                    & predictors % watervapour(1,level,j) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(3,level,i)   * &
                    & wr(j,level) / ww(j,level) 
      wr_k(i,level)=wr_k(i,level)+ predictors_k % watervapour(3,level,i)          * &
                    & sec_wr(level)/ ww(j,level) 
      ww_k(i,level)=ww_k(i,level)-predictors_k % watervapour(3,level,i)*wr(j,level) * &
                    & sec_wr(level)/ ww(j,level)**2 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(2,level,i)   * &
                     & 0.5_JPRB / predictors % watervapour(2,level,j) 
      sec_wr_k(level)=sec_wr_k(level)+predictors_k % watervapour(1,level,i)
      raytracing_k%pathsat(level,i)=raytracing_k%pathsat(level,i)                + &
                     & sec_wr_k(level) *wr(j,level) 
      wr_k(i,level)=wr_k(i,level)+sec_wr_k(level)*raytracing%pathsat(level,j)
     End Do
  End Do

  !5.1 mixed gases
  !---------------

  If(coef % id_inst /= 36)Then  

    If(coef % id_inst == 3 .and. coef % IncZeeman)Then  
  ! AMSUA ch14 with Zeeman effect (coef % nmixed = 14, otherwise = 10)
      Do level = 1,profiles_k(1) % nlevels
        Do i = 1, nchannels
          j = lprofiles( i )
          raytracing_k % pathsat(level,i) = raytracing_k % pathsat(level,i) + &                                     
                          & 2.0_JPRB * (profiles(j) % cosbk * profiles(j) % Be)**2 * &                                             
                          & raytracing % pathsat(level,j) * predictors_k % mixedgas(14,level,i) + &                         
                          & profiles(j) % Be**3 *           predictors_k % mixedgas(13,level,i) + &                                      
                          & 2.0_JPRB * profiles(j) % Be * &                                                           
                          & raytracing % pathsat(level,j) * predictors_k % mixedgas(12,level,i) + &                                                      
                          & profiles(j) % cosbk**2 *        predictors_k % mixedgas(11,level,i)                   
          predictors_k % mixedgas(11:14,level,i) = 0.0_JPRB   
        Enddo                                                   
      Enddo
    Endif           

  ! X10
    Do level = 2,profiles_k(1) % nlevels
      Do i = 1, nchannels
        j = lprofiles( i )
        raytracing_k % pathsat(level,i)=                                 &
                     & raytracing_k % pathsat(level,i)               + &
                     & predictors_k % mixedgas(10,level,i)*0.5_JPRB  * &
                     & tw(j,level)**0.25_JPRB                         / &
                     & predictors % mixedgas(9,level,j) 
        tw_k(i,level)=tw_k(i,level)                      + &
                     & predictors_k % mixedgas(10,level,i)  * &
                     & 0.25_JPRB*predictors % mixedgas(9,level,j)     / &
                     & tw(j,level)**0.75 
      End Do
    End Do
  ! X9
  ! X8
    Do level = 1,profiles_k(1) % nlevels
      Do i = 1, nchannels
        j = lprofiles( i )  
        raytracing_k % pathsat(level,i)=raytracing_k % pathsat(level,i)            + &
                    & predictors_k % mixedgas(9,level,i)*0.5_JPRB                 / &
                    & predictors % mixedgas(9,level,j) 
        raytracing_k % pathsat(level,i)=raytracing_k % pathsat(level,i)            + &
                    & predictors_k % mixedgas(8,level,i)*tw(j,level)/tr(j,level) 
        tw_k(i,level)=tw_k(i,level)+predictors_k % mixedgas(8,level,i)              * &
                    & raytracing % pathsat(level,j)  /tr(j,level) 
        tr_k(i,level)=tr_k(i,level)-predictors_k % mixedgas(8,level,i)*tw(j,level)  * &
                    & raytracing % pathsat(level,j)/tr(j,level)**2._JPRB 
        raytracing_k % pathsat(level,i)=raytracing_k % pathsat(level,i)          + &
                    & predictors_k % mixedgas(7,level,i)*tw(j,level) 
        tw_k(i,level)=tw_k(i,level)+predictors_k % mixedgas(7,level,i)          * &
                    & raytracing % pathsat(level,j) 
        tr_k(i,level) = tr_k(i,level) + predictors_k % mixedgas(6,level,i)*2      * &
                    & predictors % mixedgas(5,level,j) 
        tr_k(i,level) = tr_k(i,level) + predictors_k % mixedgas(5,level,i)
        tr_k(i,level) = tr_k(i,level) + predictors_k % mixedgas(4,level,i)*2      * &
                    & predictors % mixedgas(3,level,j) 
        raytracing_k % pathsat(level,i)= raytracing_k % pathsat(level,i)           + &
                    & predictors_k % mixedgas(4,level,i)                          * &
                    & predictors % mixedgas(5,level,j)**2 
        tr_k(i,level) = tr_k(i,level) + predictors_k % mixedgas(3,level,i)       * &
                    & raytracing % pathsat(level,j) 
        raytracing_k % pathsat(level,i) = raytracing_k % pathsat(level,i)          + &
                    & predictors_k % mixedgas(3,level,i)*tr(j,level) 
        raytracing_k % pathsat(level,i)=raytracing_k % pathsat(level,i)            + &
                    & predictors_k % mixedgas(2,level,i)*2*raytracing % pathsat(level,j) 
        raytracing_k % pathsat(level,i)=raytracing_k % pathsat(level,i)            + &
                    & predictors_k % mixedgas(1,level,i) 
      End Do
    End Do


  Else   ! ssmisz predictors (Zeeman-affected channels)
    Do level = 1, profiles_k(1) % nlevels
      Do i=1,nchannels
        j = lprofiles( i ) 
        Predictors_k % mixedgas(10,level,i) = Predictors_k % mixedgas(10,level,i) + &
                                            & Predictors_k % mixedgas(11,level,i) * profiles(j) % Be
        Predictors_k % mixedgas(3, level,i) = Predictors_k % mixedgas(3, level,i) + &
                                            & Predictors_k % mixedgas(10,level,i) * profiles(j) % Be
        raytracing_k % pathsat(level,i)     = raytracing_k % pathsat(level,i) + &
                                            & Predictors_k % mixedgas(9, level,i) * profiles(j) % Be**3 + &
                                            & Predictors_k % mixedgas(8, level,i) * profiles(j) % Be        
        Predictors_k % mixedgas(6,level,i)  = Predictors_k % mixedgas(6, level,i) + &
                                            & Predictors_k % mixedgas(7, level,i) / profiles(j) % Be
        raytracing_k % pathsat(level,i)     = raytracing_k % pathsat(level,i) + &
                                            & Predictors_k % mixedgas(6, level,i) / profiles(j) % Be        
        Predictors_k % mixedgas(2,level,i)  = Predictors_k % mixedgas(2,level,i) + &
                                            & Predictors_k % mixedgas(5, level,i) * profiles(j) % cosbk**2
        Predictors_k % mixedgas(2,level,i)  = Predictors_k % mixedgas(2,level,i) + &
                                            & Predictors_k % mixedgas(4, level,i) / profiles(j) % Be
        raytracing_k % pathsat(level,i)     = raytracing_k % pathsat(level,i) + &
                                            & Predictors_k % mixedgas(3, level,i) * profiles(j) % cosbk**2 + &
                                            & Predictors_k % mixedgas(2, level,i) * (300.0_JPRB / t(j,level))        
        t_k(i,level) = t_k(i,level) - &
                 (Predictors % mixedgas(2,level,j) / t(j,level)) * Predictors_k % mixedgas(2, level,i)         
        raytracing_k % pathsat(level,i) = raytracing_k % pathsat(level,i) + Predictors_k % mixedgas(1, level,i)

        Predictors_k % mixedgas(1:11, level,i) = 0.0_JPRB
      Enddo 
    Enddo

  Endif





  sum1 = 0._JPRB
  Do level = profiles_k(1) % nlevels, 1, -1
    Do i=1,nchannels
     j = lprofiles( i )
     If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
        sum1(i) = sum1(i) + ow_k (i,level ) / sum2_ow(j,level)
        o_k(i,level ) = o_k(i,level ) + sum1(i) * coef % dpp( level )
     Else
        o_k(i,level) = 0._JPRB
     End If
    End Do
  End Do

  sum1 = 0._JPRB
  Do level = profiles_k (1) % nlevels, 1, -1
    Do i=1,nchannels
     j = lprofiles( i )
     sum1(i) = sum1(i) + ww_k  ( i,level ) / sum2_ww(j,level)
     w_k (i,level ) = w_k ( i,level ) + sum1(i) * coef % dpp( level )
    End Do
  End Do

  Do level = profiles_k (1) % nlevels, 2, -1
    Do i=1,nchannels
      tw_k ( i,level-1 ) = tw_k ( i,level-1 ) + tw_k ( i,level )
      tr_k ( i,level-1 ) = tr_k ( i,level-1 ) + tw_k ( i,level ) *&
            & coef % dpp( level )
    End Do
  End Do

!CDIR NOLOOPCHG
  Do level=1,profiles(1)%nlevels
    Do i=1,nchannels
      j = lprofiles( i )
      If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
        o_k(i,level ) = o_k(i,level) + or_k(i,level) / coef % ostar(level)
      Endif
    EndDo
  EndDo


!CDIR NOLOOPCHG
  Do level=1,profiles(1)%nlevels
    Do i=1,nchannels
      w_k(i,level) = w_k(i,level) + wr_k(i,level) / coef % wstar(level)
      t_k(i,level) = t_k(i,level) + tr_k(i,level) / coef % tstar(level)
    Enddo
  Enddo


  Do level=1,profiles(1)%nlevels
    Do i=1,nchannels
     j = lprofiles( i )
      If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
       t_k(i,level) = t_k(i,level) + dto_k(i,level)
      Endif
      t_k(i,level) = t_k(i,level) + dt_k(i,level)
    Enddo
  Enddo

  Do level=2,profiles(1)%nlevels
    Do i=1,nchannels
     j = lprofiles( i )
     If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
       profiles_k(i) % o3(level-1) = profiles_k(i) % o3(level-1) + 0.5_JPRB *o_k(i,level )
       profiles_k(i) % o3(level)   = profiles_k(i) % o3(level) + 0.5_JPRB *o_k(i,level )
     Endif
    Enddo
  Enddo
  Do i=1,nchannels
    j = lprofiles( i )
    If ( profiles(j) % ozone_Data .And. coef % nozone > 0 ) Then
     profiles_k(i) % o3(1) = profiles_k(i) % o3(1) + o_k(i,1)
    End If
  Enddo

  If ( use_q2m )Then
    Do i=1,nchannels
      j = lprofiles( i )
  ! include surface humidity
      iv2 = aux_prof(j) % nearestlev_surf
      profiles_k(i) % s2m % q = profiles_k(i) % s2m % q + 0.5_JPRB *w_k(i,iv2)
      profiles_k(i) % q(iv3(j)) = profiles_k(i) % q(iv3(j)) + 0.5_JPRB *w_k(i,iv2)
    End Do
    Do i=1,nchannels
    j = lprofiles( i )
      Do level=2,iv3(j)
        profiles_k(i) % q(level-1) = profiles_k(i) % q(level-1) + 0.5_JPRB *w_k(i,level)
        profiles_k(i) % q(level)   = profiles_k(i) % q(level) + 0.5_JPRB *w_k(i,level )
      Enddo
    End Do
  Else
    Do level=2,profiles_k(1) % nlevels
      Do i=1,nchannels
        profiles_k(i) % q(level-1) = profiles_k(i) % q(level-1) + 0.5_JPRB *w_k(i,level )
        profiles_k(i) % q(level)  = profiles_k(i) % q(level) + 0.5_JPRB *w_k(i,level )
      Enddo
    Enddo
  Endif

  Do i=1,nchannels
    profiles_k(i) % q(1) = profiles_k(i) % q(1) + w_k(i,1)
  End Do

  Do level=2,profiles_k(1) % nlevels
    Do i=1,nchannels
      profiles_k(i) % t(level-1) = profiles_k(i) % t(level-1) + 0.5_JPRB *t_k(i,level )
      profiles_k(i) % t(level)   = profiles_k(i) % t(level) + 0.5_JPRB *t_k(i,level )
    Enddo
  Enddo
  Do i=1,nchannels
    profiles_k(i) % t(1) = profiles_k(i) % t(1) + t_k(i,1)
  End Do

IF (LHOOK) CALL DR_HOOK('RTTOV_SETPREDICTORS_7_K',1_jpim,ZHOOK_HANDLE)
End Subroutine rttov_setpredictors_7_k
